@@ -27,17 +27,18 @@ strip_makefile()
 
 check_autotool_versions()
 {
+required_autoconf_version="2.53 or newer"
 AUTOCONF_VERSION=`$AUTOCONF --version | head -n 1`
 case $AUTOCONF_VERSION in
   Autoconf*2.5* | autoconf*2.5* ) : ;;
   "" )
     echo "*** AUTOCONF NOT FOUND!."
-    echo "*** KDE requires autoconf >= 2.52"
+    echo "*** KDE requires autoconf $required_autoconf_version"
     exit 1
     ;;
   * )
     echo "*** YOU'RE USING $AUTOCONF_VERSION."
-    echo "*** KDE requires autoconf >= 2.52"
+    echo "*** KDE requires autoconf $required_autoconf_version"
     exit 1
     ;;
 esac
@@ -47,12 +48,12 @@ case $AUTOHEADER_VERSION in
   Autoconf*2.5* | autoheader*2.5* ) : ;;
   "" )
     echo "*** AUTOHEADER NOT FOUND!."
-    echo "*** KDE requires autoheader >= 2.52 (part of autoconf)"
+    echo "*** KDE requires autoheader $required_autoconf_version"
     exit 1
     ;;
   * )
     echo "*** YOU'RE USING $AUTOHEADER_VERSION."
-    echo "*** KDE requires autoheader >= 2.52 (part of autoconf)"
+    echo "*** KDE requires autoheader $required_autoconf_version"
     exit 1
     ;;
 esac
@@ -65,7 +66,7 @@ case $AUTOMAKE_STRING in
     echo "*** KDE requires automake $required_automake_version"
     exit 1
     ;;
-  automake*1.6.* | automake*1.7* ) : ;;
+  automake*1.6.* | automake*1.7* | automake*1.8* ) : ;;
   "" )
     echo "*** AUTOMAKE NOT FOUND!."
     echo "*** KDE requires automake $required_automake_version"
@@ -117,7 +118,7 @@ if test -r configure.in.in; then
 fi
 
 echo "*** Creating aclocal.m4"
-$ACLOCAL || exit 1
+$ACLOCAL $ACLOCALFLAGS || exit 1
 echo "*** Creating configure"
 call_and_fix_autoconf
 
@@ -174,7 +175,7 @@ if test -r configure.in.in; then
     configure_in
   fi
 fi
-$ACLOCAL
+$ACLOCAL $ACLOCALFLAGS
 if egrep "^AM_CONFIG_HEADER" configure.in >/dev/null 2>&1; then
   echo "*** Creating config.h template"
   $AUTOHEADER || exit 1
@@ -206,7 +207,7 @@ fi
 
 subdir_dist()
 {
-$ACLOCAL
+$ACLOCAL $ACLOCALFLAGS
 $AUTOHEADER
 $AUTOMAKE
 perl -w ../admin/am_edit --path=../admin
@@ -308,7 +309,7 @@ if test -f configure.in.in; then
    fi
 fi
 if test -z "$VERSION" || test "$VERSION" = "@VERSION@"; then
-     VERSION="\"3.1.93\""
+     VERSION="\"3.1.94\""
 fi
 if test -z "$modulename" || test "$modulename" = "@MODULENAME@"; then
    modulename=`pwd`; 
