@@ -53,7 +53,6 @@ bool AsciiSource::initFile() {
     _rowIndex = (int *)malloc(32768 * sizeof(int));
     _rowIndex[0] = 0;
     _numLinesAlloc = 32768;
-    _numFrames = 0;
     _byteLength = 0;
   }
 
@@ -67,10 +66,20 @@ bool AsciiSource::initFile() {
 #define MAXBUFREADLEN 32768
 KstObject::UpdateType AsciiSource::update(int u) {
   Q_UNUSED(u)
+  bool forceUpdate = false;
   static char *tmpbuf = 0L;
   QFile file(_filename);
 
+  if (!_valid) {
+    _numFrames = 0;
+    _byteLength = 0;
+    _rowIndex[0] = 0;
+  }
+
   if (file.exists()) {
+    if (uint(_byteLength) != file.size() || !_valid) {
+      forceUpdate = true;
+    }
     _byteLength = file.size();
   } else {
     _valid = false;
@@ -83,6 +92,8 @@ KstObject::UpdateType AsciiSource::update(int u) {
     return KstObject::NO_CHANGE;
   }
 
+  _valid = true;
+
   if (!tmpbuf) {
     tmpbuf = new char[MAXBUFREADLEN];
   }
@@ -112,8 +123,7 @@ KstObject::UpdateType AsciiSource::update(int u) {
         if (!is_comment) {
           has_dat = true;
         }
-      } else if (tmpbuf[i] == '\n' ||
-                 tmpbuf[i] == '\r') {
+      } else if (tmpbuf[i] == '\n' || tmpbuf[i] == '\r') {
         if (has_dat) {
           _numFrames++;
           if (_numFrames >= _numLinesAlloc) {
@@ -133,7 +143,7 @@ KstObject::UpdateType AsciiSource::update(int u) {
 
   file.close();
 
-  return new_data ? KstObject::UPDATE : KstObject::NO_CHANGE;
+  return forceUpdate ? KstObject::UPDATE : (new_data ? KstObject::UPDATE : KstObject::NO_CHANGE);
 }
 
 
@@ -170,20 +180,20 @@ int AsciiSource::readField(double *v, const QString& field, int s, int n) {
   file.at(bufstart);
   file.readBlock(tmpbuf, bufread);
 
+  // FIXME: I don't see why this was changed to memset() instead of just
+  //        setting the required elements to 0 inline when required, as it
+  //        did before.
   memset( v, 0, n * sizeof( double ) );
-  
+
   for (int i = 0; i < n; i++, s++) {
     bool done = false;
     bool incol = false;
     int i_col = 0;
     for (int ch = _rowIndex[s] - bufstart; !done && ch < bufread; ch++) {
       if (isspace(tmpbuf[ch])) {
-        if (tmpbuf[ch] == '\n' ||
-            tmpbuf[ch] == '\r' ) {
+        if (tmpbuf[ch] == '\n' || tmpbuf[ch] == '\r' ) {
           done = true;
-        }
-        else
-        {
+        } else {
           incol = false;
         }
       } else if (tmpbuf[ch] == '#' || tmpbuf[ch] == '!' ||
@@ -262,6 +272,7 @@ QStringList AsciiSource::fieldList() const {
 
   file.close();
 
+  rc += "INDEX";
   int cnt = QStringList::split(QRegExp("\\s"), line, false).count();
   for (int i = 1; i <= cnt; ++i) {
     rc += QString::number(i);
@@ -272,7 +283,6 @@ QStringList AsciiSource::fieldList() const {
 
 
 void AsciiSource::save(QTextStream &ts) {
-  // FIXME
   KstDataSource::save(ts);
 }
 
