@@ -94,8 +94,9 @@ KstObject::UpdateType LFIIOSource::update( int u )
         iResult = fits_get_num_rows( ffits, &lNumFrames, &iStatus );
         if( iResult == 0 )
         {
-          _strListColNames.clear( );
-
+          _fieldList.clear( );
+          _fieldList.append( "INDEX" );
+          
           _valid = true;
 
           //
@@ -110,12 +111,12 @@ KstObject::UpdateType LFIIOSource::update( int u )
             if( iResult == 0 )
             { 
               strName = charName;
-              _strListColNames.append( strName );
+              _fieldList.append( strName );
             }
             else
             {
               strName.setNum( i );
-              _strListColNames.append( strName );
+              _fieldList.append( strName );
             }
               
             iStatus = 0;
@@ -229,11 +230,11 @@ bool LFIIOSource::getColNumber( const QString& field, int* piColNumber ) const
   }
   else
   {
-    iCount = _strListColNames.count( );
+    iCount = _fieldList.count( );
 
     for( i=0; i<iCount; i++ )
     {
-      strName = _strListColNames[i].lower( );
+      strName = _fieldList[i].lower( );
       if( strName.compare( field.lower( ) ) == 0 )
       {
         bRetVal = true;
