@@ -44,7 +44,7 @@ class PlanckSource : public KstDataSource {
     virtual void save(QTextStream &ts);
 
   private:
-    KSharedPtr<Planck::Object> _planckObject;
+    KstSharedPtr<Planck::Object> _planckObject;
 };
 
 
