@@ -28,7 +28,7 @@
  *@author C. Barth Netterfield
  */
 
-typedef enum {KST_VCURVE, KST_PSDCURVE, KST_EQUATIONCURVE, KST_HISTOGRAM} KstCurveType;
+typedef enum {KST_VCURVE, KST_PSDCURVE, KST_EQUATIONCURVE, KST_HISTOGRAM, KST_FITCURVE} KstCurveType;
 
 class KstBaseCurve: public KstDataObject {
 public:
@@ -36,6 +36,7 @@ public:
   KstBaseCurve(QDomElement& e);
   virtual ~KstBaseCurve();
 
+  virtual void _showDialog() { }
   virtual void save(QTextStream &ts) = 0;
   virtual QString propertyString() const = 0;
   virtual void getPoint(int i, double &x1, double &y1) = 0;
@@ -76,15 +77,19 @@ public:
 
   virtual void setHasPoints(bool in_HasPoints) { HasPoints = in_HasPoints; }
   virtual void setHasLines(bool in_HasLines)   { HasLines = in_HasLines; }
+  virtual void setLineWidth(int in_LineWidth) { LineWidth = in_LineWidth; }
+  virtual void setLineStyle(int in_LineStyle)  { LineStyle = in_LineStyle; }
   virtual bool hasPoints() const { return HasPoints; }
   virtual bool hasLines()  const { return HasLines; }
+  virtual int lineWidth() const { return LineWidth; }
+  virtual int lineStyle()  const { return LineStyle; }
 
   virtual bool slaveVectorsUsed() const { return false; }
 
   virtual bool xIsRising() const {return false;}
 
   virtual int samplesPerFrame() const { return 1; }
-
+  
   KstPoint Point;
 
 protected:
@@ -102,7 +107,8 @@ protected:
 
   bool HasPoints;
   bool HasLines;
-
+  int  LineWidth;
+  int  LineStyle;
   QColor Color;
 
 private:
@@ -111,7 +117,7 @@ private:
 };
 
 
-typedef KSharedPtr<KstBaseCurve> KstBaseCurvePtr;
+typedef KstSharedPtr<KstBaseCurve> KstBaseCurvePtr;
 typedef KstObjectList<KstBaseCurvePtr> KstBaseCurveList;
 
 #endif
