@@ -26,6 +26,7 @@
 #include "kstrvector.h"
 #include "kstdatacollection.h"
 #include "kstchangenptsdialog_i.h"
+#include "datarangewidget.h"
 
 KstChangeNptsDialogI::KstChangeNptsDialogI(QWidget* parent,
                                            const char* name,
@@ -82,11 +83,11 @@ void KstChangeNptsDialogI::applyNptsChange() {
       vector = rvl[i_vector];
 
       vector->changeFrames(
-        (CountFromEnd->isChecked() ?  -1 : F0->value()),
-        (ReadToEnd->isChecked() ? -1 : N->value()),
-        Skip->value(),
-        DoSkip->isChecked(),
-        DoFilter->isChecked());
+        (_kstDataRange->CountFromEnd->isChecked() ? -1 : _kstDataRange->F0->value()),
+        (_kstDataRange->ReadToEnd->isChecked() ? -1 : _kstDataRange->N->value()),
+        _kstDataRange->Skip->value(),
+        _kstDataRange->DoSkip->isChecked(),
+        _kstDataRange->DoFilter->isChecked());
     }
   }
 
@@ -109,25 +110,26 @@ void KstChangeNptsDialogI::updateDefaults(int index) {
 
     /* fill the vector range entries */
   if (vector->countFromEOF()) {
-    CountFromEnd->setChecked(true);
+    _kstDataRange->CountFromEnd->setChecked(true);
   } else {
-    CountFromEnd->setChecked(false);
+    _kstDataRange->CountFromEnd->setChecked(false);
   }
-  F0->setValue(vector->reqStartFrame());
+  _kstDataRange->F0->setValue(vector->reqStartFrame());
 
   /* fill number of frames entries */
   if (vector->readToEOF()) {
-    ReadToEnd->setChecked(true);
+    _kstDataRange->ReadToEnd->setChecked(true);
   } else {
-    ReadToEnd->setChecked(false);
+    _kstDataRange->ReadToEnd->setChecked(false);
   }
-  N->setValue(vector->reqNumFrames());
+  _kstDataRange->N->setValue(vector->reqNumFrames());
 
   /* fill in frames to skip box */
-  Skip->setValue(vector->skip());
-  DoSkip->setChecked(vector->doSkip());
-  DoFilter->setChecked(vector->doAve());
-
+  _kstDataRange->Skip->setValue(vector->skip());
+  _kstDataRange->DoSkip->setChecked(vector->doSkip());
+  _kstDataRange->DoFilter->setChecked(vector->doAve());
+  _kstDataRange->updateEnables();
 }
 
+
 #include "kstchangenptsdialog_i.moc"
