@@ -16,19 +16,36 @@
  ***************************************************************************/
 
 #include "kstdatacollection.h"
+#include "kstrvector.h"
 
-  /** The list of data sources (files) */
-  KstDataSourceList KST::dataSourceList;
+/** The list of data sources (files) */
+KstDataSourceList KST::dataSourceList;
 
-  /** The list of vectors that are being read */
-  KstVectorList KST::vectorList;
+/** The list of vectors that are being read */
+KstVectorList KST::vectorList;
 
-  /** The list of Scalars which have been generated */
-  KstScalarList KST::scalarList;
+/** The list of Scalars which have been generated */
+KstScalarList KST::scalarList;
 
-  /** The list of data objects which are in use */
-  KstDataObjectList KST::dataObjectList;
+/** The list of data objects which are in use */
+KstDataObjectList KST::dataObjectList;
 
-  /** The list of filter sets defined */
-  KstFilterSetList KST::filterSetList;
+/** The list of filter sets defined */
+KstFilterSetList KST::filterSetList;
+
+QString KST::suggestPlotName() {
+  static int last = 0;
+  return QString("P%1").arg(++last);
+}
+
+QString KST::suggestWinName() {
+  static int last = 0;
+  return QString("W%1").arg(++last);
+}
+
+void KST::addVectorToList(KstVectorPtr v) {
+  KST::vectorList.lock().writeLock();
+  KST::vectorList.append(v);
+  KST::vectorList.lock().writeUnlock();
+}
 
