@@ -43,12 +43,18 @@ namespace KST {
   extern KstFilterSetList filterSetList;
 
   /** check that a tag has not been used by any other tags */
-  extern bool tagNameNotUnique(const QString &tag, bool warn = true, void *parent = 0L);
-  extern bool dataTagNameNotUnique(const QString &tag, bool warn = true, void *parent = 0L);
-  extern bool vectorTagNameNotUnique(const QString &tag, bool warn = true, void *parent = 0L);
+  extern bool tagNameNotUnique(const QString& tag, bool warn = true, void *parent = 0L);
+  extern bool dataTagNameNotUnique(const QString& tag, bool warn = true, void *parent = 0L);
+  extern bool vectorTagNameNotUnique(const QString& tag, bool warn = true, void *parent = 0L);
+
+  extern QString suggestPlotName();
+
+  extern QString suggestWinName();
 
   /** Save a vector to a file */
   extern int vectorToFile(KstVectorPtr v, QFile *f, void *parent = 0L);
+
+  extern void addVectorToList(KstVectorPtr v);
 }
 
 #endif
