@@ -15,39 +15,47 @@
  *   (at your option) any later version.                                   *
  *                                                                         *
  ***************************************************************************/
-#include "kstdatamanager_i.h"
-
 #include <assert.h>
 
-#include <kdebug.h>
-#include <klistview.h>
-#include <klocale.h>
-#include <kmessagebox.h>
-#include <kpopupmenu.h>
-
-#include <qmap.h>
+// include files for Qt
 #include <qlistview.h>
+#include <qmap.h>
 #include <qptrlist.h>
 #include <qptrstack.h>
 #include <qpushbutton.h>
 #include <qstring.h>
 #include <qwidget.h>
 
-#include "kstfilteredvector.h"
-#include "kstdoc.h"
-#include "kstplugin.h"
-#include "kstrvector.h"
-#include "kstdatacollection.h"
-#include "kstpsddialog_i.h"
-#include "ksteqdialog_i.h"
-#include "ksthsdialog_i.h"
+// include files for KDE
+#include <kdebug.h>
+#include <klistview.h>
+#include <klocale.h>
+#include <kmdimainfrm.h>
+#include <kmessagebox.h>
+#include <kpopupmenu.h>
+
+// application specific includes
+#include "kst.h"
+#include "kst2dplot.h"
 #include "kstcurvedialog_i.h"
+#include "kstdatacollection.h"
+#include "kstdatamanager_i.h"
+#include "kstdoc.h"
+#include "ksteqdialog_i.h"
+#include "ksteventmonitor_i.h"
+#include "kstfilteredvector.h"
+#include "ksthsdialog_i.h"
+#include "kstplotdialog_i.h"
+#include "kstplugin.h"
 #include "kstplugindialog_i.h"
+#include "kstpsddialog_i.h"
+#include "kstrvector.h"
 #include "kstvectordialog_i.h"
+#include "kstviewwindow.h"
 #include "vectorselector.h"
 
 
-static QMap<int,QString> PlotMap;
+static QMap<int,Kst2DPlotPtr> PlotMap;
 static QMap<int,QString> FilterMap;
 
 #define RTTI_OBJ_VECTOR          4201
@@ -108,6 +116,7 @@ void KstObjectItem::update(bool recursive) {
         assert(px.data());
         assert(dynamic_cast<KstRVector*>(px.data()));
         KstRVectorPtr x = static_cast<KstRVector*>(px.data());
+        x->readLock();
         setText(2, QString::number(x->getUsage() - 3)); // caller has a ptr
         setText(3, QString::number(x->sampleCount()));
         setText(4, i18n("%3: %4 [%1..%2]").arg(x->reqStartFrame())
@@ -115,6 +124,7 @@ void KstObjectItem::update(bool recursive) {
             .arg(x->filename())
             .arg(x->getField()));
         _removable = x->getUsage() == 3;
+        x->readUnlock();
         break;
       }
     case RTTI_OBJ_FILTERED_VECTOR:
@@ -123,19 +133,23 @@ void KstObjectItem::update(bool recursive) {
         assert(px.data());
         assert(dynamic_cast<KstFilteredVector*>(px.data()));
         KstFilteredVectorPtr x = static_cast<KstFilteredVector*>(px.data());
+        x->readLock();
         setText(2, QString::number(x->getUsage() - 3)); // caller has a ptr
         setText(3, QString::number(x->sampleCount()));
         setText(4, i18n("[%1..%2]").arg(x->min()).arg(x->max()));
         _removable = x->getUsage() == 3;
+        x->readUnlock();
         break;
       }
     case RTTI_OBJ_VECTOR:
       {
         KstVectorPtr x = *KST::vectorList.findTag(_name);
         assert(x.data());
+        x->readLock();
         setText(2, QString::number(x->getUsage() - 2)); //caller also points
         setText(3, QString::number(x->sampleCount()));
         setText(4, i18n("[%1..%2]").arg(x->min()).arg(x->max()));
+        x->readUnlock();
         _removable = false;
         break;
       }
@@ -143,6 +157,7 @@ void KstObjectItem::update(bool recursive) {
       {
         KstDataObjectPtr x = *KST::dataObjectList.findTag(_name);
         assert(x.data());
+        x->readLock();
         setText(1, x->typeString());
         setText(2, QString::number(x->getUsage() - 1)); // our pointer
         setText(3, QString::number(x->sampleCount()));
@@ -178,6 +193,7 @@ void KstObjectItem::update(bool recursive) {
           }
         }
         _removable = x->getUsage() == 1;
+        x->readUnlock();
         break;
       }
     default:
@@ -191,11 +207,14 @@ void KstObjectItem::updateButtons() {
 }
 
 void KstObjectItem::reload() {
+  KstReadLocker ml(&KST::vectorList.lock());
   KstVectorList::Iterator v = KST::vectorList.findTag(_name);
   if (v != KST::vectorList.end()) {
     KstRVector *r = dynamic_cast<KstRVector*>((*v).data());
     if (r) {
+      r->writeLock();
       r->reload();
+      r->writeUnlock();
     }
   }
 }
@@ -206,8 +225,12 @@ void KstObjectItem::makeCurve() {
 }
 
 void KstObjectItem::applyFilter(int id) {
+  KST::filterSetList.lock().readLock();
   KstFilterSetPtr p = KST::filterSetList.find(FilterMap[id]);
+  KST::filterSetList.lock().readUnlock();
+  KST::vectorList.lock().readLock();
   KstVectorPtr v = *KST::vectorList.findTag(_name);
+  KST::vectorList.lock().readUnlock();
   if (p && v) {
     KstFilteredVectorPtr fv = new KstFilteredVector(v, p);
     emit updated();
@@ -215,8 +238,8 @@ void KstObjectItem::applyFilter(int id) {
 }
 
 void KstObjectItem::addToPlot(int id) {
-  KstPlot *p = KST::plotList.FindKstPlot(PlotMap[id]);
-  KstBaseCurve *c = dynamic_cast<KstBaseCurve*>(dataObject().data());
+  Kst2DPlotPtr p = PlotMap[id];
+  KstBaseCurvePtr c = dynamic_cast<KstBaseCurve*>(dataObject().data());
   if (p && c) {
     p->addCurve(c);
     emit updated();
@@ -224,8 +247,8 @@ void KstObjectItem::addToPlot(int id) {
 }
 
 void KstObjectItem::removeFromPlot(int id) {
-  KstPlot *p = KST::plotList.FindKstPlot(PlotMap[id]);
-  KstBaseCurve *c = dynamic_cast<KstBaseCurve*>(dataObject().data());
+  Kst2DPlotPtr p = PlotMap[id];
+  KstBaseCurvePtr c = dynamic_cast<KstBaseCurve*>(dataObject().data());
   if (p && c) {
     p->removeCurve(c);
     emit updated();
@@ -244,6 +267,8 @@ KstDataManagerI::KstDataManagerI(KstDoc *in_doc, QWidget* parent, const char* na
             this, SLOT(edit_I()));
     connect(DataView, SIGNAL(currentChanged(QListViewItem *)),
             this, SLOT(currentChanged(QListViewItem *)));
+    connect(DataView, SIGNAL(selectionChanged(QListViewItem *)),
+            this, SLOT(currentChanged(QListViewItem *)));
 
     connect(NewCurve, SIGNAL(clicked()),
             KstCurveDialogI::globalInstance(), SLOT(show_New()));
@@ -253,12 +278,17 @@ KstDataManagerI::KstDataManagerI(KstDoc *in_doc, QWidget* parent, const char* na
             KstEqDialogI::globalInstance(), SLOT(show_New()));
     connect(NewPlugin, SIGNAL(clicked()),
             KstPluginDialogI::globalInstance(), SLOT(show_New()));
+    connect(NewEvent, SIGNAL(clicked()),
+            KstEventMonitorI::globalInstance(), SLOT(show_New()));
     connect(NewPSD, SIGNAL(clicked()),
             KstPsdDialogI::globalInstance(), SLOT(show_New()));
     connect(NewVector, SIGNAL(clicked()),
             KstVectorDialogI::globalInstance(), SLOT(show_New()));
+    connect(OpenPlotDialog, SIGNAL(clicked()),
+            KstApp::inst(), SLOT(showPlotDialog()));
 
-    connect(Filter, SIGNAL(clicked()), this, SLOT(filter_I()));
+// FIXME: Filters disabled for now...
+//  connect(Filter, SIGNAL(clicked()), this, SLOT(filter_I()));
 
     connect(DataView, SIGNAL(contextMenuRequested(QListViewItem*, const QPoint&, int)), this, SLOT(contextMenu(QListViewItem*, const QPoint&, int)));
 }
@@ -275,13 +305,28 @@ void KstDataManagerI::show_I() {
 }
 
 
-void KstDataManagerI::update() {
-if (!isShown()) {
-  return;
+void KstDataManagerI::updateContents() {
+  if (!isShown()) {
+    return;
+  }
+
+  for (QListViewItem *i = DataView->firstChild(); i; i = i->nextSibling()) {
+    KstObjectItem *oi = static_cast<KstObjectItem*>(i);
+    oi->update();
+  }
 }
 
-QPtrStack<QListViewItem> trash;
 
+void KstDataManagerI::update() {
+  if (!isShown()) {
+    return;
+  }
+
+  QListViewItem *currentItem = DataView->selectedItem();
+  QPtrStack<QListViewItem> trash;
+
+  KST::dataObjectList.lock().writeLock();
+  KST::vectorList.lock().writeLock();
   // Garbage collect first
   for (QListViewItem *i = DataView->firstChild(); i; i = i->nextSibling()) {
     KstObjectItem *oi = static_cast<KstObjectItem*>(i);
@@ -318,6 +363,8 @@ QPtrStack<QListViewItem> trash;
         connect(i, SIGNAL(updated()), this, SLOT(doUpdates()));
     }
   }
+  KST::vectorList.lock().writeUnlock();
+  KST::dataObjectList.lock().writeUnlock();
 
   KstRVectorList rvl = kstObjectSubList<KstVector,KstRVector>(KST::vectorList);
   for (KstRVectorList::iterator it = rvl.begin(); it != rvl.end(); ++it) {
@@ -353,6 +400,15 @@ QPtrStack<QListViewItem> trash;
     }
   }
 
+  // is this really necessary?  I would think not.....
+  for (QListViewItem *i = DataView->firstChild(); i; i = i->nextSibling()) {
+    if (i == currentItem) {
+      DataView->setCurrentItem(i);
+      DataView->setSelected(i, true);
+      break;
+    }
+  }
+
   if (DataView->selectedItem()) {
     static_cast<KstObjectItem*>(DataView->currentItem())->updateButtons();
   } else {
@@ -391,6 +447,8 @@ void KstDataManagerI::delete_I() {
   QListViewItem *qi = DataView->selectedItems().at(0);
   KstObjectItem *koi = static_cast<KstObjectItem*>(qi);
 
+  KstWriteLocker ml(&KST::vectorList.lock());
+
   if (qi->rtti() == RTTI_OBJ_OBJECT) {
     doc->removeDataObject(koi->tagName());
   } else if (qi->rtti() == RTTI_OBJ_DATA_VECTOR) {
@@ -410,7 +468,7 @@ void KstDataManagerI::delete_I() {
 void KstDataManagerI::contextMenu(QListViewItem *i, const QPoint& p, int col) {
   Q_UNUSED(col)
   KstObjectItem *koi = static_cast<KstObjectItem*>(i);
-  KstBaseCurve *c;
+  KstBaseCurvePtr c;
 
   KPopupMenu *m = new KPopupMenu(this);
 
@@ -427,11 +485,13 @@ void KstDataManagerI::contextMenu(QListViewItem *i, const QPoint& p, int col) {
     KPopupMenu *fMenu = new KPopupMenu(this);
     id = 500;
     FilterMap.clear();
+    KST::filterSetList.lock().readLock();
     for (KstFilterSetList::Iterator f = KST::filterSetList.begin(); f != KST::filterSetList.end(); ++f) {
       fMenu->insertItem((*f)->name(), koi, SLOT(applyFilter(int)), 0, id);
       FilterMap[id] = (*f)->name();
       ++id;
     }
+    KST::filterSetList.lock().readUnlock();
     id = m->insertItem(i18n("Apply &Filter"), fMenu);
     m->setItemEnabled(id, !FilterMap.isEmpty());
   }
@@ -449,16 +509,30 @@ void KstDataManagerI::contextMenu(QListViewItem *i, const QPoint& p, int col) {
     PlotMap.clear();
     id = 100;
     bool haveAdd = false, haveRemove = false;
-    for (KstPlot *p = KST::plotList.first(); p; p = KST::plotList.next()) {
-      if (!p->Curves.contains(c)) {
-        addMenu->insertItem(p->tagName(), koi, SLOT(addToPlot(int)), 0, id);
-        haveAdd = true;
-      } else {
-        removeMenu->insertItem(p->tagName(), koi, SLOT(removeFromPlot(int)), 0, id);
-        haveRemove = true;
+
+    KstApp *app = KstApp::inst();
+    KMdiIterator<KMdiChildView*> *it = app->createIterator();
+    while (it->currentItem()) {
+      KstViewWindow *v = dynamic_cast<KstViewWindow*>(it->currentItem());
+      if (v) {
+        Kst2DPlotList plots = v->view()->findChildrenType<Kst2DPlot>();
+        for (Kst2DPlotList::Iterator i = plots.begin(); i != plots.end(); ++i) {
+          Kst2DPlotPtr plot = *i;
+          if (!plot->Curves.contains(c)) {
+            addMenu->insertItem(i18n("%1 - %2").arg(v->caption()).arg(plot->tagName()), koi, SLOT(addToPlot(int)), 0, id);
+            haveAdd = true;
+          } else {
+            removeMenu->insertItem(i18n("%1 - %2").arg(v->caption()).arg(plot->tagName()), koi, SLOT(removeFromPlot(int)), 0, id);
+            haveRemove = true;
+          }
+          PlotMap[id++] = plot;
+        }
       }
-      PlotMap[id++] = p->tagName();
+      it->next();
     }
+
+    app->deleteIterator(it);
+    
     id = m->insertItem(i18n("&Add To Plot"), addMenu);
     m->setItemEnabled(id, haveAdd);
     id = m->insertItem(i18n("&Remove From Plot"), removeMenu);
@@ -476,9 +550,8 @@ void KstDataManagerI::contextMenu(QListViewItem *i, const QPoint& p, int col) {
 
 
 void KstDataManagerI::doUpdates() {
+  doc->forceUpdate();
   doc->setModified();
-  doc->update();
-  doc->updateDialogs();
 }
 
 
