@@ -23,7 +23,7 @@
 #include "kstvector.h"
 
 #include <qdom.h>
-#include <ksharedptr.h>
+#include "kstsharedptr.h"
 #include <qvaluelist.h>
 
 
@@ -40,7 +40,8 @@ public:
 
   virtual int sampleCount() const { return 0; }
 
-
+  // If you use these, you must lock() and unlock() the object as long as you
+  // hold the reference
   const KstVectorMap& inputVectors()  const { return _inputVectors;  }
   const KstVectorMap& outputVectors() const { return _outputVectors; }
         KstVectorMap& inputVectors()        { return _inputVectors;  }
@@ -55,10 +56,15 @@ public:
 
   void showDialog();
 
-  virtual void loadInputs();
+  virtual bool loadInputs();
 
   virtual int getUsage() const;
 
+  virtual void readLock() const;
+  virtual void readUnlock() const;
+  virtual void writeLock() const;
+  virtual void writeUnlock() const;
+
 protected slots:
   virtual void _showDialog() = 0;
 
@@ -75,7 +81,7 @@ protected:
   QValueList<QPair<QString,QString> > _inputScalarLoadQueue;
 };
 
-typedef KSharedPtr<KstDataObject> KstDataObjectPtr;
+typedef KstSharedPtr<KstDataObject> KstDataObjectPtr;
 typedef KstObjectList<KstDataObjectPtr> KstDataObjectList;
 
 
