@@ -35,9 +35,9 @@
 // Eventually this will move to another file but I leave it here until then
 // to avoid confusion between the function plugins and Kst applicaton plugins.
 namespace KST {
-  class Plugin : public KShared {
+  class Plugin : public KstShared {
     public:
-      Plugin(KService::Ptr svc) : KShared(), service(svc), _lib(0L) {
+      Plugin(KService::Ptr svc) : KstShared(), service(svc), _lib(0L) {
         assert(service);
         _plugLib = service->property("X-Kst-Plugin-Library").toString();
       }
@@ -47,8 +47,10 @@ namespace KST {
       KstDataSource *create(const QString& filename, const QString& type = QString::null) const {
         KstDataSource *(*sym)(const QString&, const QString&) = (KstDataSource*(*)(const QString&, const QString&))symbol("create");
         if (sym) {
+          //kdDebug() << "Trying to create " << filename << " type=" << type << " with " << service->property("Name").toString() << endl;
           KstDataSource *ds = (sym)(filename, type);
           ds->_source = service->property("Name").toString();
+          //kdDebug() << (ds ? "SUCCESS" : "FAILED") << endl;
           return ds;
         }
 
@@ -58,7 +60,10 @@ namespace KST {
       bool understands(const QString& filename) const {
         bool (*sym)(const QString&) = (bool(*)(const QString&))symbol("understands");
         if (sym) {
-          return (sym)(filename);
+          //kdDebug() << "Checking if " << service->property("Name").toString() << " understands " << filename << endl;
+          bool rc = (sym)(filename);
+          //kdDebug() << "result: " << rc << endl;
+          return rc;
         }
 
         return false;
@@ -67,7 +72,10 @@ namespace KST {
       bool provides(const QString& type) const {
         QStringList (*sym)() = (QStringList(*)())symbol("provides");
         if (sym) {
-          return ((sym)()).contains(type);
+          //kdDebug() << "Checking if " << service->property("Name").toString() << " provides " << type << endl;
+          bool rc = ((sym)()).contains(type);
+          //kdDebug() << "result: " << rc << endl;
+          return rc;
         }
 
         return false;
@@ -95,7 +103,7 @@ namespace KST {
         QCString libname = QFile::encodeName(QString("kstdata_") + _plugLib);
         _lib = KLibLoader::self()->library(libname);
         if (!_lib) {
-          KstDebug::self()->log(i18n("Error loading data source plugin [%1]: %2").arg(libname).arg(KLibLoader::self()->lastErrorMessage()), KstDebug::Error);
+          KstDebug::self()->log(i18n("Error loading data-source plugin [%1]: %2").arg(libname).arg(KLibLoader::self()->lastErrorMessage()), KstDebug::Error);
         }
         return _lib != 0L;
       }
@@ -106,7 +114,7 @@ namespace KST {
       mutable KLibrary *_lib;
   };
 
-  typedef QValueList<KSharedPtr<KST::Plugin> > PluginInfoList;
+  typedef QValueList<KstSharedPtr<KST::Plugin> > PluginInfoList;
 }
 
 
@@ -117,7 +125,7 @@ static KST::PluginInfoList pluginInfo;
 static void scanPlugins() {
   KST::PluginInfoList tmpList;
 
-  KstDebug::self()->log(i18n("Scanning for data source plugins."));
+  KstDebug::self()->log(i18n("Scanning for data-source plugins."));
 
   KService::List sl = KServiceType::offers("Kst Data Source");
   for (KService::List::ConstIterator it = sl.begin(); it != sl.end(); ++it) {
@@ -128,7 +136,7 @@ static void scanPlugins() {
       }
     }
 
-    KSharedPtr<KST::Plugin> p = new KST::Plugin(*it);
+    KstSharedPtr<KST::Plugin> p = new KST::Plugin(*it);
     tmpList.append(p);
   }
 
@@ -298,4 +306,20 @@ void KstDataSource::virtual_hook(int id, void *data) {
   //KstObject::virtual_hook(id, data);
 }
 
+
+void *KstDataSource::bufferMalloc(size_t size) {
+  return ::malloc(size);
+}
+
+
+void KstDataSource::bufferFree(void *ptr) {
+  return ::free(ptr);
+}
+
+
+void *KstDataSource::bufferRealloc(void *ptr, size_t size) {
+  return ::realloc(ptr, size);
+}
+
+
 // vim: ts=2 sw=2 et
