@@ -42,8 +42,8 @@ public:
   /** Returns a list of plugins found on the system. */
   static QStringList pluginList();
 
-  static KSharedPtr<KstDataSource> loadSource(const QString& filename, const QString& type = QString::null);
-  static KSharedPtr<KstDataSource> loadSource(QDomElement& e);
+  static KstSharedPtr<KstDataSource> loadSource(const QString& filename, const QString& type = QString::null);
+  static KstSharedPtr<KstDataSource> loadSource(QDomElement& e);
  
   virtual bool isValid() const; // generally you don't need to change this
                                 // returns _valid;
@@ -83,6 +83,10 @@ public:
 
   const QString& sourceName() const { return _source; }
 
+  virtual void *bufferMalloc(size_t size);
+  virtual void bufferFree(void *ptr);
+  virtual void *bufferRealloc(void *ptr, size_t size);
+
 protected:
   virtual void virtual_hook(int id, void *data);
 
@@ -111,7 +115,7 @@ private:
 };
 
 
-typedef KSharedPtr<KstDataSource> KstDataSourcePtr;
+typedef KstSharedPtr<KstDataSource> KstDataSourcePtr;
 class KstDataSourceList : public KstObjectList<KstDataSourcePtr> {
   public:
     KstDataSourceList() : KstObjectList<KstDataSourcePtr>() {}
