@@ -21,22 +21,25 @@
 #include <sys/stat.h>
 #endif
 
-#include <unistd.h>
+#include <assert.h>
 #include <fcntl.h>
+#include <unistd.h>
 
 // include files for Qt
+#include <qcheckbox.h>
+#include <qdeepcopy.h>
 #include <qdir.h>
 #include <qfileinfo.h>
-#include <qwidget.h>
-#include <qtextstream.h>
-#include <qlineedit.h>
-#include <qspinbox.h>
 #include <qfontdatabase.h>
-#include <qcheckbox.h>
+#include <qlineedit.h>
+#include <qlistbox.h>
 #include <qpushbutton.h>
 #include <qradiobutton.h>
+#include <qspinbox.h>
 #include <qstring.h>
-#include <qlistbox.h>
+#include <qtextstream.h>
+#include <qwidget.h>
+#include <qmessagebox.h>
 
 // include files for KDE
 #include <kapplication.h>
@@ -48,9 +51,11 @@
 #include <kprogress.h>
 #include <ksavefile.h>
 #include <kurl.h>
+#include <kmdimainfrm.h>
 
 // application specific includes
 #include "kst.h"
+#include "kst2dplot.h"
 #include "kstdatacollection.h"
 #include "kstdebug.h"
 #include "kstdoc.h"
@@ -61,15 +66,18 @@
 #include "kstpoint.h"
 #include "kstpsdcurve.h"
 #include "kstvcurve.h"
+#include "kstfitcurve.h"
+#include "kstvectordefaults.h"
+#include "kstviewwindow.h"
+#include "threadevents.h"
 
 static bool backupFile(const QString& qFilename, const QString& backupDir = QString::null, const QString& backupExtension = "~");
 
 KstDoc::KstDoc(QWidget *parent, const char *name)
 : QObject(parent, name) {
   _lock = 0;
-  stopping = false;
-  UpdateCounter = 0;
-  _updateDelay = 200;
+  _updating = false;
+  _stopping = false;
 }
 
 KstDoc::~KstDoc() {
@@ -77,59 +85,60 @@ KstDoc::~KstDoc() {
   // guaranteed by any means and KstDoc::vectorList is static and has a
   // pointer to KstDoc::scalarList, also static, which gets used during
   // vector destruction.
-  KST::filterSetList.clear();
-  KST::plotList.clear();
-  KST::vectorList.clear();
-  KST::scalarList.clear();
+  deleteContents();
 }
 
-void KstDoc::setAbsFilePath(const QString &filename) {
-  absFilePath = filename;
+void KstDoc::setAbsFilePath(const QString& filename) {
+  _absFilePath = filename;
 }
 
-const QString &KstDoc::getAbsFilePath() const {
-  return absFilePath;
+const QString& KstDoc::absFilePath() const {
+  return _absFilePath;
 }
 
-void KstDoc::setTitle(const QString &_t) {
-  title = _t;
+void KstDoc::setTitle(const QString& t) {
+  _title = t;
 }
 
-const QString &KstDoc::getTitle() const {
-  return title;
+const QString& KstDoc::title() const {
+  return _title;
 }
 
-bool KstDoc::saveModified() {
+bool KstDoc::saveModified(bool doDelete) {
   bool completed = true;
 
-  if (modified) {
+  if (_modified) {
     KstApp *win = static_cast<KstApp*>(parent());
     int want_save = KMessageBox::warningYesNoCancel( win, i18n("The current plot definition has been modified. Do you want to save it?"), i18n("Question"));
     switch (want_save) {
       case KMessageBox::Yes:
-           if (title == i18n("Untitled")) {
-             win->slotFileSaveAs();
-           } else {
-             saveDocument(getAbsFilePath());
-       	   }
+        if (_title == i18n("Untitled")) {
+          win->slotFileSaveAs();
+        } else {
+          saveDocument(absFilePath());
+        }
 
-       	   deleteContents();
-           completed = true;
-           break;
+        if (doDelete) {
+          deleteContents();
+        }
+        completed = true;
+        break;
 
       case KMessageBox::No:
-           setModified(false);
-           deleteContents();
-           completed = true;
-           break;
+        if (doDelete) {
+          setModified(false);
+          deleteContents();
+        }
+        completed = true;
+        break;
 
       case KMessageBox::Cancel:
-           completed = false;
-           break;
+        completed = false;
+        break;
 
       default:
-           completed = false;
-           break;
+        completed = false;
+        break;
     }
   }
 
@@ -141,15 +150,20 @@ void KstDoc::closeDocument() {
 }
 
 bool KstDoc::newDocument() {
-  deleteContents();
-  modified = false;
-  absFilePath = QDir::homeDirPath();
-  title = i18n("Untitled");
+  // FIXME: implement this properly
+  //if (KMessageBox::Yes == KMessageBox::warningYesNo(KstApp::inst(), i18n("Are you sure you wish to erase all your data and plots?"))) {
+    deleteContents();
+    _modified = false;
+    _absFilePath = QDir::homeDirPath();
+    _title = i18n("Untitled");
+    createScalars();
+    emit updateDialogs();
+  //}
 
   return true;
 }
 
-bool KstDoc::openDocument(const KURL &url, const QString &o_file,
+bool KstDoc::openDocument(const KURL& url, const QString& o_file,
                           int o_n, int o_f, int o_s, bool o_ave) {
   // Temp variables
   KstDataSourcePtr file;
@@ -162,29 +176,32 @@ bool KstDoc::openDocument(const KURL &url, const QString &o_file,
     return false;
   }
 
-  title = url.fileName(false);
-  absFilePath = url.path();
-  if (title == 0L)
-    title = absFilePath;
-  QDomDocument doc(title);
+  _title = url.fileName(false);
+  _absFilePath = url.path();
+  if (_title.isNull()) { // isEmpty()???
+    _title = _absFilePath;
+  }
+  QDomDocument doc(_title);
 
-  if ( !f.open( IO_ReadOnly ) ) {
+  if (!f.open(IO_ReadOnly)) {
     KMessageBox::sorry(0L, i18n("%1: File exists, but kst could not open it.").arg(url.path()));
     return false;
   }
+
   if (!doc.setContent(&f)) {
     KMessageBox::sorry(0L, i18n("%1: Not a valid kst plot specification file.").arg(url.path()));
     f.close();
     return false;
   }
+
   f.close();
 
   QDomElement docElem = doc.documentElement();
 
   QDomNode n = docElem.firstChild();
-  while( !n.isNull() ) {
+  while (!n.isNull()) {
     QDomElement e = n.toElement(); // try to convert the node to an element.
-    if( !e.isNull() ) { // the node was really an element.
+    if (!e.isNull()) { // the node was really an element.
       if (e.tagName() == "windowsize") {
         QDomNode nn = e.firstChild();
         QSize size = static_cast<KstApp*>(parent())->size();
@@ -199,10 +216,13 @@ bool KstDoc::openDocument(const KURL &url, const QString &o_file,
         }
         static_cast<KstApp*>(parent())->resize(size);
       } else if (e.tagName() == "plotcols") {
-        KST::plotList.setPlotCols(e.text().toInt());
+// FIXMEPLOTLIST
+//        KST::plotList.setPlotCols(e.text().toInt());
       } else if (e.tagName() == "updateDelay") {
+// FIXME XXXX
         setDelay(e.text().toInt());
       } else if (e.tagName() == "kstfile") {
+        KstWriteLocker dswl(&KST::dataSourceList.lock());
         if (o_file == "|") {
           file = KstDataSource::loadSource(e);
         } else {
@@ -218,25 +238,67 @@ bool KstDoc::openDocument(const KURL &url, const QString &o_file,
         new KstScalar(e);
       } else if (e.tagName() == "vector") {
         vector = new KstRVector(e, o_file, o_n, o_f, o_s, o_ave);
+        KST::addVectorToList(KstVectorPtr(vector));
         // Vectors are automatically added to the global list.
       } else if (e.tagName() == "filterset") {
+        KST::filterSetList.lock().writeLock();
         KST::filterSetList.append(new KstFilterSet(e));
+        KST::filterSetList.lock().writeUnlock();
       } else if (e.tagName() == "filteredvector") {
         new KstFilteredVector(e);
         // Vectors are automatically added to the global list.
       } else if (e.tagName() == "plugin") {
-        KST::dataObjectList.append(new KstPlugin(e));
+        KstDataObjectPtr p = new KstPlugin(e);
+        KstWriteLocker dowl(&KST::dataObjectList.lock());
+        KST::dataObjectList.append(p);
       } else if (e.tagName() == "curve") {
-        KST::dataObjectList.append(new KstVCurve(e));
+        KstDataObjectPtr p = new KstVCurve(e);
+        KstWriteLocker dowl(&KST::dataObjectList.lock());
+        KST::dataObjectList.append(p);
+      } else if (e.tagName() == "fit") {
+        KstDataObjectPtr p = new KstFitCurve(e);
+        KstWriteLocker dowl(&KST::dataObjectList.lock());
+        KST::dataObjectList.append(p);
       } else if (e.tagName() == "equation") {
-        KST::dataObjectList.append(new KstEquationCurve(e));
+        KstDataObjectPtr p = new KstEquationCurve(e);
+        KstWriteLocker dowl(&KST::dataObjectList.lock());
+        KST::dataObjectList.append(p);
       } else if (e.tagName() == "psd") {
-        KST::dataObjectList.append(new KstPSDCurve(e));
+        KstDataObjectPtr p = new KstPSDCurve(e);
+        KstWriteLocker dowl(&KST::dataObjectList.lock());
+        KST::dataObjectList.append(p);
       } else if (e.tagName() == "histogram") {
-        KST::dataObjectList.append(new KstHistogram(e));
+        KstDataObjectPtr p = new KstHistogram(e);
+        KstWriteLocker dowl(&KST::dataObjectList.lock());
+        KST::dataObjectList.append(p);
+      } else if (e.tagName() == "event") {
+        KstDataObjectPtr p = new EventMonitorEntry(e);
+        KstWriteLocker dowl(&KST::dataObjectList.lock());
+        KST::dataObjectList.append(p);
       } else if (e.tagName() == "plot") {
         KstBaseCurveList l = kstObjectSubList<KstDataObject,KstBaseCurve>(KST::dataObjectList);
-        KST::plotList.append(new KstPlot(e, l));
+        Kst2DPlotPtr plotPtr = new Kst2DPlot(e);
+        KstApp::inst()->plotHolderWhileOpeningDocument().insert(plotPtr->tagName(), plotPtr);
+      } else if (e.tagName() == "window") {
+        KstViewWindow *p = dynamic_cast<KstViewWindow*>(KstApp::inst()->activeWindow());
+        KstViewWindow *toDelete = 0L;
+
+        if (!p) {
+          KMdiIterator<KMdiChildView*> *it = KstApp::inst()->createIterator();
+          p = dynamic_cast<KstViewWindow*>(it->currentItem());
+          KstApp::inst()->deleteIterator(it);
+        }
+
+        if (p && p->view()->children().count() == 0) {
+          toDelete = p;
+        }
+
+        p = new KstViewWindow(e);
+        KstApp::inst()->addWindow(p);
+        p->activate();
+        if (toDelete) {
+          KstApp::inst()->closeWindow(toDelete);
+        }
       } else {
         KstDebug::self()->log(i18n("Unsupported element '%1' in file %2.").arg(e.tagName()).arg(file->fileName()), KstDebug::Warning);
       }
@@ -244,35 +306,98 @@ bool KstDoc::openDocument(const KURL &url, const QString &o_file,
     n = n.nextSibling();
   }
 
-  for (KstDataObjectList::Iterator i = KST::dataObjectList.begin();
-       i != KST::dataObjectList.end();
-       ++i) {
-    (*i)->loadInputs();
-  }
+  //
+  // if we have anything left in plotHolderWhileOpeningDocument then
+  //  we are most likely reading an old style document, so we create
+  //  a default view and fill it with whatever is left...
+  //
+  if (KstApp::inst()->plotHolderWhileOpeningDocument().count() > 0) {
+    if (!KstApp::inst()->activeWindow()) {
+      Kst2DPlotMap::Iterator it;
+      Kst2DPlotPtr plot;
+      KstViewWindow *pWindow;
+      QString strWindow;
 
-  modified = false;
-  return true;
-}
+      strWindow = KstApp::inst()->newWindow(QString::null);
+      pWindow = dynamic_cast<KstViewWindow*>(KstApp::inst()->findWindow(strWindow));
 
-bool KstDoc::saveDocument(const QString &filename) {
-  int i;
-  backupFile(filename);
-  QFile f(filename);
+      if (pWindow) {
+        for ( it  = KstApp::inst()->plotHolderWhileOpeningDocument().begin();
+              it != KstApp::inst()->plotHolderWhileOpeningDocument().end(); ++it ) {
+          plot = *it;
 
-  if (f.exists() && (filename != getAbsFilePath())) {
-    if (KMessageBox::warningYesNo(0L, i18n("%1: A file of this name already exists.").arg(filename),
-                                  i18n("Warning"),i18n("&Overwrite"),
-                                  i18n("&Cancel")) == KMessageBox::No) {
-      return false;
+          if (plot->_width > 0.0 && plot->_height > 0.0 ) {
+            pWindow->view()->appendChild(plot.data(), true);
+            if (plot->_width > 1.0 && plot->_height > 1.0) {
+              plot->resizeFromAspect( (double)plot->_pos_x / (double)plot->_width,
+                                     (double)plot->_pos_y / (double)plot->_height,
+                                     1.0 / (double)plot->_width,
+                                     1.0 / (double)plot->_height );
+            } else {
+              plot->resizeFromAspect( plot->_pos_x, plot->_pos_y,
+                                      plot->_width, plot->_height );
+            }
+          }
+        }
+      }
     }
   }
 
-  if (!f.open(IO_WriteOnly|IO_Truncate)) {
-    KMessageBox::sorry(0L, i18n("%1: Could not open file for saving. The plot description has not been saved. Try a different filename or directory.").arg(filename));
-    return false;
+  KstApp::inst()->plotHolderWhileOpeningDocument().clear();
+
+  // Lazy load data
+  KstDataObjectList bitBucket;
+  {
+    KST::dataObjectList.lock().readLock();
+    KstDataObjectList dol = QDeepCopy<KstDataObjectList>(KST::dataObjectList);
+    KST::dataObjectList.lock().readUnlock();
+    for (KstDataObjectList::Iterator i = dol.begin(); i != dol.end(); ++i) {
+      assert(*i);
+      //kdDebug() << "Load inputs for " << (*i)->tagName() << " " << (void*)*i << endl;
+      (*i)->KstRWLock::writeLock();
+      bool rc = (*i)->loadInputs();
+      (*i)->KstRWLock::writeUnlock();
+      if (!rc) {
+        // schedule for removal
+        bitBucket.append(*i);
+      }
+    }
   }
 
-  QTextStream ts(&f);
+  KST::dataObjectList.lock().writeLock();
+  for (KstDataObjectList::Iterator i = bitBucket.begin(); i != bitBucket.end(); ++i) {
+    KST::dataObjectList.remove(*i);
+  }
+  KST::dataObjectList.lock().writeUnlock();
+
+  if (!bitBucket.isEmpty()) {
+    KMessageBox::sorry(0L, i18n("The Kst file could not be loaded in its entirety due to missing objects or data."));
+  }
+
+  createScalars();
+
+  // Update plots now that lazy loading is done
+  KMdiIterator<KMdiChildView*> *it = KstApp::inst()->createIterator();
+  while (it->currentItem()) {
+    KstViewWindow *c = dynamic_cast<KstViewWindow*>(it->currentItem());
+    if (c) {
+      Kst2DPlotList allplots = c->view()->findChildrenType<Kst2DPlot>(true);
+      for (Kst2DPlotList::Iterator i = allplots.begin(); i != allplots.end(); ++i) {
+        (*i)->draw();
+      }
+    }
+    it->next();
+  }
+  KstApp::inst()->deleteIterator(it);
+
+  emit updateDialogs();
+
+  _modified = false;
+  return true;
+}
+
+void KstDoc::saveDocument(QTextStream& ts) {
+  int i;
 
   ts << "<?xml version=\"1.0\" encoding=\"UTF-8\"?>" << endl;
   ts << "<kstdoc>" << endl;
@@ -283,17 +408,21 @@ bool KstDoc::saveDocument(const QString &filename) {
   ts << " </windowsize>" << endl;
 
   // save plot cols
-  ts << " <plotcols>" << KST::plotList.getPlotCols() << "</plotcols>" << endl;
+// FIXMEPLOTLIST
+//  ts << " <plotcols>" << KST::plotList.getPlotCols() << "</plotcols>" << endl;
 
   // save update delay
+//FIXME XXXX
   ts << " <updateDelay>" << delay() << "</updateDelay>" << endl;
 
   // save files
+  KST::dataSourceList.lock().readLock();
   for (i = 0; i < (int)KST::dataSourceList.count(); i++) {
     ts << " <kstfile>" << endl;
     KST::dataSourceList[i]->save(ts);
     ts << " </kstfile>" << endl;
   }
+  KST::dataSourceList.lock().readUnlock();
 
   // save orphan scalars
   for (KstScalarList::Iterator it = KST::scalarList.begin(); it != KST::scalarList.end(); ++it) {
@@ -313,16 +442,20 @@ bool KstDoc::saveDocument(const QString &filename) {
   }
 
   // save data objects
+  KST::dataObjectList.lock().readLock();
   for (KstDataObjectList::Iterator it = KST::dataObjectList.begin(); it != KST::dataObjectList.end(); ++it) {
     (*it)->save(ts);
   }
+  KST::dataObjectList.lock().readUnlock();
 
   // save filters
+  KST::filterSetList.lock().readLock();
   for (KstFilterSetList::Iterator it = KST::filterSetList.begin(); it != KST::filterSetList.end(); ++it) {
     ts << " <filterset>" << endl;
     (*it)->save(ts);
     ts << " </filterset>" << endl;
   }
+  KST::filterSetList.lock().readUnlock();
 
   KstFilteredVectorList fvl = kstObjectSubList<KstVector,KstFilteredVector>(KST::vectorList);
   for (KstFilteredVectorList::Iterator it = fvl.begin(); it != fvl.end(); ++it) {
@@ -332,165 +465,111 @@ bool KstDoc::saveDocument(const QString &filename) {
   }
 
   // save plots
-  for (i = 0; i < (int)KST::plotList.count(); i++) {
-    ts << " <plot>" << endl;
-    KST::plotList.at(i)->save(ts);
-    ts << " </plot>" << endl;
-  }
-
-  ts << "</kstdoc>" << endl;
-
-  f.close();
-
-  modified = false;
-  return true;
-}
-
-void KstDoc::deleteContents()
-{
-  /* delete the plots from plotList */
-  KST::plotList.clear();
-
-  /* delete the dataobjects */
-  KST::dataObjectList.clear();
-
-  /* delete the vectors from the vectorList */
-  KST::vectorList.clear();
-
-  /* delete the files from the filelist */
-  KST::dataSourceList.clear();
-}
-
-
-class FakeLocker {
-  public:
-    FakeLocker(int *x) : _x(x) { *_x = 1; }
-    ~FakeLocker() { *_x = 0; }
-  private:
-    int *_x;
-};
-
-/*****************
-    Return values:
-    NO_CHANGE: Nothing changed and so no view updates are required
-    UPDATE:    There was a change, and we are done now - go do a view update
-******************/
-/** Update Strategy **/
-/* Update kstfiles, always, even if they are unused */
-/* updata data vectors, even if they are unused, so stats stay correct */
-/*     If reading is happening, don't go on until done (stay responsive) */
-/* update all curves (curves, psd's, hs, eqs) */
-/*     Each curve calls update on any vector it depends on */
-/*     Slave vectors call update on masters */
-/*     Masters update their slaves at the end */
-/*     Multiple re-updates are stopped by looking for new update_counter */
-/* update all plugins - same rule as curves */
-/* We need to create a KstDataObject, which is inherited by vectors, curves */
-/*  and plugins.  update, tags, save, usage so slaves can know about any */
-/*  masters (inc. plugins) */
-/****/
-KstObject::UpdateType KstDoc::update(bool force) {
-  unsigned i;
-  int lastFrame = -1;
-  QString message;
-  KstObject::UpdateType tU, U = KstObject::NO_CHANGE;
-
-  // Prevent reentrancy due to event processing!!
-  if (_lock != 0) {
-    kdDebug() << "LOCKED" << endl;
-    return U;
-  }
-
-  FakeLocker ml(&_lock);
-
-  UpdateCounter++;
-  if (UpdateCounter < 1) {
-    UpdateCounter = 1; // check for wrap around
-  }
-
-  // Update the files
-  for (i = 0; i < KST::dataSourceList.count(); i++) {
-    KST::dataSourceList[i]->update();
-  }
-
-  kapp->processEvents();
-  if (stopping) {
-    return KstObject::NO_CHANGE;
-  }
-
-  KstRVectorList rvl = kstObjectSubList<KstVector,KstRVector>(KST::vectorList);
-  // Update data vectors
-  for (i = 0; i < rvl.count(); i++) {
-    tU = rvl[i]->update(UpdateCounter);
-    if (tU != KstObject::NO_CHANGE) {
-      kapp->processEvents();
-      if (stopping) {
-        return KstObject::NO_CHANGE;
-      }
-      lastFrame = rvl[i]->numFrames() + rvl[i]->startFrame();
-      if (tU == KstObject::READ) {
-        U = KstObject::READ;
-      } else if (U == KstObject::NO_CHANGE) {
-        U = KstObject::UPDATE;
-      }
-    }
-  }
-
-  if (lastFrame > 0) {
-    emit newFrameMsg(lastFrame);
-  }
-
-  /* update the data objects - except curves! */
-  for (KstDataObjectList::Iterator it = KST::dataObjectList.begin(); it != KST::dataObjectList.end(); ++it) {
-    if (!dynamic_cast<KstBaseCurve*>((*it).data())) {
-      (*it)->update(UpdateCounter);
-    }
-  }
-
-  /* update all vectors  -  brute force */
-  // FIXME: can't update slave vectors, so for now we just check for
-  //        filtered vectors - yuck
-  KstVectorList vl = KST::vectorList;
-  for (KstVectorList::Iterator it = vl.begin(); it != vl.end(); ++it) {
-    //if (dynamic_cast<KstRVector*>((*it).data())) {
-    if (!dynamic_cast<KstFilteredVector*>((*it).data())) {
-      continue;
-    }
-
-    tU = (*it)->update(UpdateCounter);
-    if (tU != KstObject::NO_CHANGE) {
-      kapp->processEvents();
-      if (stopping) {
-        return KstObject::NO_CHANGE;
-      }
-      if (tU == KstObject::READ && U != KstObject::UPDATE) {
-        U = KstObject::READ;
-      } else if (U == KstObject::NO_CHANGE) {
-        U = KstObject::UPDATE;
-      }
-    }
-  }
-
-  // update curves next
-  if ((U == KstObject::UPDATE) || force) {
-    KstDataObjectList dol = KST::dataObjectList;
-    for (KstDataObjectList::Iterator it = dol.begin(); it != dol.end(); ++it) {
-      KstBaseCurve *bc = dynamic_cast<KstBaseCurve*>((*it).data());
-      if (bc) {
-        bc->update(UpdateCounter);
-        kapp->processEvents();
-        if (stopping) {
-          return KstObject::NO_CHANGE;
+  KstApp *pApp = KstApp::inst();
+  KMdiIterator<KMdiChildView*> *it = pApp->createIterator();
+  while (it->currentItem()) {
+    KstViewWindow *c = dynamic_cast<KstViewWindow*>(it->currentItem());
+    if (c) {
+      KstPlotBaseList plots = c->view()->findChildrenType<KstPlotBase>(true);
+      for (KstPlotBaseList::Iterator i = plots.begin(); i != plots.end(); ++i) {
+        Kst2DPlotPtr plot = dynamic_cast<Kst2DPlot*>((*i).data());
+        if (plot) {
+          ts << " <plot>" << endl;
+          plot->save(ts);
+          ts << " </plot>" << endl;
+        } else {
         }
       }
     }
+    it->next();
+  }
+  pApp->deleteIterator(it);
 
-    /* all updated: time to reset new and shift */
-    for (i = 0; i < KST::vectorList.count(); i++) {
-      KST::vectorList[i]->newSync();
+  // save windows
+  it = KstApp::inst()->createIterator();
+  if (it) {
+    while (it->currentItem()) {
+      KstViewWindow *v = dynamic_cast<KstViewWindow*>(it->currentItem());
+      if (v) {
+        ts << " <window>" << endl;
+        v->save(ts);
+        ts << " </window>" << endl;
+      }
+      it->next();
+    }
+    KstApp::inst()->deleteIterator(it);
+  }
+
+  ts << "</kstdoc>" << endl;
+}
+
+bool KstDoc::saveDocument(const QString &filename) {
+  backupFile(filename);
+  QFile f(filename);
+
+  if (f.exists() && (filename != absFilePath())) {
+    if (KMessageBox::warningYesNo(0L, i18n("%1: A file of this name already exists.").arg(filename),
+                                  i18n("Warning"),i18n("&Overwrite"),
+                                  i18n("&Cancel")) == KMessageBox::No) {
+      return false;
     }
   }
-  return U;
+
+  if (!f.open(IO_WriteOnly|IO_Truncate)) {
+    KMessageBox::sorry(0L, i18n("%1: Could not open file for saving. The plot description has not been saved. Try a different filename or directory.").arg(filename));
+    return false;
+  }
+
+  QTextStream ts(&f);
+
+  saveDocument(ts);
+
+  f.close();
+
+  _modified = false;
+  return true;
+}
+
+void KstDoc::deleteContents() {
+  KMdiIterator<KMdiChildView*>* pIterator;
+
+  KST::vectorDefaults.sync();
+
+  KST::filterSetList.lock().writeLock();
+  KST::filterSetList.clear();
+  KST::filterSetList.lock().writeUnlock();
+
+  if (KstApp::inst()) {
+    pIterator = KstApp::inst()->createIterator();
+    if (pIterator) {
+      while (pIterator->currentItem()) {
+        KMdiChildView *pView = pIterator->currentItem();
+        pIterator->next();
+        if (pView) {
+          KstApp::inst()->closeWindow(pView);
+        }
+      }
+      KstApp::inst()->deleteIterator(pIterator);
+    }
+  }
+
+  KST::vectorList.lock().writeLock();
+  KST::vectorList.clear();
+  KST::vectorList.lock().writeUnlock();
+
+  KST::scalarList.lock().writeLock();
+  KST::scalarList.clear();
+  KST::scalarList.lock().writeUnlock();
+
+  KST::dataSourceList.lock().writeLock();
+  KST::dataSourceList.clear();
+  KST::dataSourceList.lock().writeUnlock();
+
+  KST::dataObjectList.lock().writeLock();
+  KST::dataObjectList.clear();
+  KST::dataObjectList.lock().writeUnlock();
+
+  emit updateDialogs();
 }
 
 void KstDoc::samplesUp() {
@@ -518,7 +597,7 @@ void KstDoc::samplesUp() {
   }
 
   setModified();
-  update();
+  forceUpdate();
 
   emit dataChanged();
 }
@@ -551,7 +630,7 @@ void KstDoc::samplesDown() {
   }
 
   setModified();
-  update();
+  forceUpdate();
 
   emit dataChanged();
 }
@@ -576,20 +655,22 @@ void KstDoc::samplesEnd() {
   }
 
   setModified();
-  update();
+  forceUpdate();
 
   emit dataChanged();
 }
 
 void KstDoc::wasModified() {
-  modified = true;
-  update();
-  updateDialogs();
+  _modified = true;
+  forceUpdate();
+  KstApp::inst()->paintAll();
 }
 
-RemoveStatus KstDoc::removeDataObject(const QString &tag) {
+RemoveStatus KstDoc::removeDataObject(const QString& tag) {
   // FIXME: necessary?  We don't allow removal if it's in a plot...
   /* delete the curve from all plots */
+// FIXMEPLOTLIST
+#if 0
   for (unsigned i_plot = 0; i_plot < KST::plotList.count(); i_plot++) {
     KstPlot *plot = KST::plotList.at(i_plot);
     for (KstBaseCurveList::iterator it = plot->Curves.begin(); it != plot->Curves.end(); ++it) {
@@ -604,11 +685,13 @@ RemoveStatus KstDoc::removeDataObject(const QString &tag) {
       }
     }
   }
+#endif
 
+  KST::dataObjectList.lock().writeLock();
   KST::dataObjectList.removeTag(tag);
+  KST::dataObjectList.lock().writeUnlock();
   setModified();
-  update();
-  updateDialogs();
+  forceUpdate();
   return OBJECT_DELETED;
 }
 
@@ -618,10 +701,11 @@ void KstDoc::purge() {
   KstRVectorList rvl = kstObjectSubList<KstVector,KstRVector>(KST::vectorList);
   KstFilteredVectorList fvl = kstObjectSubList<KstVector,KstFilteredVector>(KST::vectorList);
 
+  KST::dataObjectList.lock().writeLock();
   int cnt = rvl.count() + fvl.count() + KST::dataObjectList.count();
   int prg = 0;
 
-  KProgressDialog *dlg = new KProgressDialog((QWidget*)dynamic_cast<KstApp*>(parent())->viewObject(), "Progress Dialog");
+  KProgressDialog *dlg = new KProgressDialog((QWidget*)static_cast<KstApp*>(parent()), "Progress Dialog");
 
   dlg->setAllowCancel(false);
   dlg->progressBar()->setTotalSteps(cnt);
@@ -642,12 +726,15 @@ void KstDoc::purge() {
     dlg->progressBar()->setProgress(++prg);
     kapp->processEvents();
   }
+  KST::dataObjectList.lock().writeUnlock();
 
   for (KstRVectorList::Iterator it = rvl.begin(); it != rvl.end(); ++it) {
     //kdDebug() << "VECTOR: " << (*it)->tagName() << " USAGE: " << (*it)->getUsage() << endl;
     if ((*it)->getUsage() == 1) {
       //kdDebug() << "    -> REMOVED" << endl;
+      KST::vectorList.lock().writeLock();
       KST::vectorList.remove((*it).data());
+      KST::vectorList.lock().writeUnlock();
       modified = true;
     }
     dlg->progressBar()->setProgress(++prg);
@@ -660,7 +747,9 @@ void KstDoc::purge() {
     //kdDebug() << "VECTOR: " << (*it)->tagName() << " USAGE: " << (*it)->getUsage() << endl;
     if ((*it)->getUsage() == 1) {
       //kdDebug() << "    -> REMOVED" << endl;
+      KST::vectorList.lock().writeLock();
       KST::vectorList.remove((*it).data());
+      KST::vectorList.lock().writeUnlock();
       modified = true;
     }
     dlg->progressBar()->setProgress(++prg);
@@ -670,11 +759,85 @@ void KstDoc::purge() {
   fvl.clear();
 
   setModified(modified);
-  updateDialogs();
+  emit updateDialogs();
 
   delete dlg;
 }
 
+
+void KstDoc::forceUpdate() {
+  static_cast<KstApp*>(parent())->forceUpdate();
+}
+
+
+bool KstDoc::event(QEvent *e) {
+  KMdiIterator<KMdiChildView*>* pIterator;
+  KstViewWindow* pView;
+
+  if (e->type() == KstEventTypeThread) {
+    _updating = true; // block update thread from sending events 'till we're done
+
+    ThreadEvent *te = static_cast<ThreadEvent*>(e);
+    if (te->_eventType == ThreadEvent::UpdateDataDialogs) {
+      kdDebug() << "Update data dialogs" << endl;
+      emit dataChanged();
+      KstApp::inst()->paintAll();
+    } else if (te->_eventType == ThreadEvent::UpdateAllDialogs) {
+      kdDebug() << "Update ALL dialogs" << endl;
+
+      if (KstApp::inst()) {
+        pIterator = KstApp::inst()->createIterator();
+        if (pIterator) {
+          while (pIterator->currentItem()) {
+            pView = dynamic_cast<KstViewWindow*>(pIterator->currentItem());
+            if (pView) {
+              pView->view()->update();
+            }
+            pIterator->next();
+          }
+          KstApp::inst()->deleteIterator(pIterator);
+        }
+      }
+
+      emit updateDialogs();
+    } else if (te->_eventType == ThreadEvent::Repaint) {
+      KstApp::inst()->paintAll();
+    } else if (te->_eventType == ThreadEvent::Done) {
+    }
+    kapp->processEvents(); // actually do the paints and check for inputs
+    _updating = false; // ok, we're done... update thread can send us a new event.
+
+    return true;
+  }
+
+  return false;
+}
+
+
+void KstDoc::createScalars() const {
+  // Should not be orphans, just unparented so they are not saved
+  new KstScalar(i18n("CONST_MKSA_SPEED_OF_LIGHT"), 2.99792458e8 , false);
+  new KstScalar(i18n("CONST_MKSA_GRAVITATIONAL_CONSTANT"), 6.673e-11, false);
+  new KstScalar(i18n("CONST_MKSA_PLANCKS_CONSTANT_H"), 6.62606876e-34, false);
+  new KstScalar(i18n("CONST_MKSA_PLANCKS_CONSTANT_HBAR"), 1.05457159642e-34, false);
+  new KstScalar(i18n("CONST_MKSA_ASTRONOMICAL_UNIT"), 1.49597870691e11, false);
+  new KstScalar(i18n("CONST_MKSA_VACUUM_PERMITTIVITY"), 8.854187817e-12, false);
+  new KstScalar(i18n("CONST_MKSA_VACUUM_PERMEABILITY"), 1.25663706144e-6, false);
+  new KstScalar(i18n("CONST_MKSA_GRAV_ACCEL"), 9.80665e0, false);
+  new KstScalar(i18n("CONST_MKSA_MASS_MUON"), 1.88353109e-28, false);
+  new KstScalar(i18n("CONST_MKSA_MASS_PROTON"), 1.67262158e-27, false);
+  new KstScalar(i18n("CONST_MKSA_MASS_NEUTRON"), 1.67492716e-27, false);
+  new KstScalar(i18n("CONST_MKSA_RYDBERG"), 2.17987190389e-18, false);
+  new KstScalar(i18n("CONST_MKSA_BOLTZMANN"), 1.3806503e-23, false);
+  new KstScalar(i18n("CONST_MKSA_SOLAR_MASS"), 1.98892e30, false);
+  new KstScalar(i18n("CONST_MKSA_BOHR_RADIUS"), 5.291772083e-11, false);
+  new KstScalar(i18n("CONST_MKSA_ELECTRON_CHARGE"), 1.602176462e-19, false);
+  new KstScalar(i18n("CONST_MKSA_MOLAR_GAS"), 8.314472e0, false);
+  new KstScalar(i18n("CONST_MKSA_STANDARD_GAS_VOLUME"), 2.2710981e-2, false);
+  new KstScalar(i18n("CONST_PI"), 3.141592654e0, false);
+}
+
+
 static int write_all(int fd, const char *buf, size_t len) {
    while (len > 0)
    {
@@ -753,14 +916,16 @@ static bool backupFile(const QString& qFilename, const QString& backupDir,
        int n = ::read( fd, buffer, 32*1024 );
        if (n == -1)
        {
-          if (errno == EINTR)
+          if (errno == EINTR) {
               continue;
+          }
           ::close(fd);
           ::close(fd2);
           return false;
        }
-       if (n == 0)
+       if (n == 0) {
           break; // Finished
+       }
 
        if (write_all( fd2, buffer, n))
        {
