@@ -59,7 +59,13 @@ void KstFilter::apply(const KstVectorPtr in, KstVectorPtr out) {
   for (it = table.begin(); it != table.end(); ++it) {
     if ((*it)._type == Plugin::Data::IOValue::FloatType) {
       KstScalarPtr p = _inputScalars[(*it)._name];
-      inScalars[scalarCnt++] = p ? p->value() : 0.0;
+      if (p) {
+        p->readLock();
+        inScalars[scalarCnt++] = p->value();
+        p->readUnlock();
+      } else {
+        inScalars[scalarCnt++] = 0.0;
+      }
     }
   }
 
@@ -76,17 +82,17 @@ void KstFilter::apply(const KstVectorPtr in, KstVectorPtr out) {
 }
 
 
-bool KstFilter::setPlugin(KSharedPtr<Plugin> plugin) {
+bool KstFilter::setPlugin(KstSharedPtr<Plugin> plugin) {
   _plugin = plugin;
   return true;
 }
 
 
-KstFilterSet::KstFilterSet() : KstObjectList<KstFilterPtr>(), KShared() {
+KstFilterSet::KstFilterSet() : KstObjectList<KstFilterPtr>(), KstShared() {
 }
 
 
-KstFilterSet::KstFilterSet(QDomElement& e) : KstObjectList<KstFilterPtr>(), KShared() {
+KstFilterSet::KstFilterSet(QDomElement& e) : KstObjectList<KstFilterPtr>(), KstShared() {
   QDomNode n = e.firstChild();
   while (!n.isNull()) {
     QDomElement e = n.toElement();
@@ -107,7 +113,7 @@ KstFilterSet::~KstFilterSet() {
 
 
 void KstFilterSet::parseFilter(const QString& nm, QDomElement& e) {
-  KSharedPtr<Plugin> pp = PluginCollection::self()->plugin(nm);
+  KstSharedPtr<Plugin> pp = PluginCollection::self()->plugin(nm);
   if (!pp) {
     return;
   }
@@ -116,6 +122,7 @@ void KstFilterSet::parseFilter(const QString& nm, QDomElement& e) {
   fp->setPlugin(pp);
   fp->setTagName(nm);
 
+  KST::scalarList.lock().writeLock();
   QDomNode n = e.firstChild();
   while (!n.isNull()) {
     QDomElement e = n.toElement();
@@ -134,6 +141,7 @@ void KstFilterSet::parseFilter(const QString& nm, QDomElement& e) {
     }
     n = n.nextSibling();
   }
+  KST::scalarList.lock().writeUnlock();
 
   // would be a good idea to iterate through the plugin arguments and be sure
   // that we have all the required inputs before appending.
@@ -151,7 +159,7 @@ void KstFilterSet::apply(const KstVectorPtr in, KstVectorPtr out) {
     return;
   }
 
-  KstVector *tmpVector1, *tmpVector2;
+  KstVectorPtr tmpVector1, tmpVector2;
 
   Iterator it = begin();
   if (left == 1) {
@@ -159,6 +167,7 @@ void KstFilterSet::apply(const KstVectorPtr in, KstVectorPtr out) {
     return;
   }
 
+  // Will auto-delete since they're KstSharedPtr
   tmpVector1 = new KstVector;
   tmpVector2 = new KstVector;
   (*it)->apply(in, tmpVector1);
@@ -173,14 +182,10 @@ void KstFilterSet::apply(const KstVectorPtr in, KstVectorPtr out) {
     }
 
     // Swap the vectors
-    KstVector *tmp = tmpVector1;
+    KstVectorPtr tmp = tmpVector1;
     tmpVector1 = tmpVector2;
     tmpVector2 = tmp;
   }
-
-  // this will delete them
-  KST::vectorList.remove(tmpVector1);
-  KST::vectorList.remove(tmpVector2);
 }
 
 
