@@ -29,16 +29,16 @@ class KstFilter : public KstPlugin {
     virtual ~KstFilter();
 
     virtual void apply(const KstVectorPtr in, KstVectorPtr out);
-    virtual bool setPlugin(KSharedPtr<Plugin> plugin);
+    virtual bool setPlugin(KstSharedPtr<Plugin> plugin);
 
   private:
     void *_localData;
 };
 
-typedef KSharedPtr<KstFilter> KstFilterPtr;
+typedef KstSharedPtr<KstFilter> KstFilterPtr;
 
 
-class KstFilterSet : public KstObjectList<KstFilterPtr>, public KShared {
+class KstFilterSet : public KstObjectList<KstFilterPtr>, public KstShared {
   public:
     KstFilterSet();
     KstFilterSet(QDomElement& e);
@@ -57,7 +57,7 @@ class KstFilterSet : public KstObjectList<KstFilterPtr>, public KShared {
 };
 
 
-typedef KSharedPtr<KstFilterSet> KstFilterSetPtr;
+typedef KstSharedPtr<KstFilterSet> KstFilterSetPtr;
 
 class KstFilterSetList : public QValueList<KstFilterSetPtr> {
   public:
@@ -72,6 +72,11 @@ class KstFilterSetList : public QValueList<KstFilterSetPtr> {
       }
       return KstFilterSetPtr();
     }
+
+    KstRWLock& lock() const { return _lock; }
+
+  private:
+    mutable KstRWLock _lock;
 };
 
 
