@@ -24,11 +24,13 @@
 
 KstFilteredVector::KstFilteredVector(KstVectorPtr parent, KstFilterSetPtr filterSet)
 : KstVector(parent->tagName() + "-" + filterSet->name(), parent->length()), _parent(parent), _filterSet(filterSet) {
+  // FIXME: RACE: added to global list before constructed
   update();
 }
 
 
 KstFilteredVector::KstFilteredVector(QDomElement& e) : KstVector() {
+  // FIXME: RACE: added to global list before constructed
   QDomNode n = e.firstChild();
   while(!n.isNull()) {
     QDomElement e = n.toElement();
@@ -36,12 +38,16 @@ KstFilteredVector::KstFilteredVector(QDomElement& e) : KstVector() {
       if (e.tagName() == "tag") {
         setTagName(e.text());
       } else if (e.tagName() == "parent") {
+        KST::vectorList.lock().readLock();
         _parent = *KST::vectorList.findTag(e.text());
+        KST::vectorList.lock().readUnlock();
         if (_parent) {
           resize(_parent->length());
         }
       } else if (e.tagName() == "filterset") {
+        KST::filterSetList.lock().readLock();
         _filterSet = KST::filterSetList.find(e.text());
+        KST::filterSetList.lock().readUnlock();
       }
     }
     n = n.nextSibling();
