@@ -15,21 +15,32 @@
  *   (at your option) any later version.                                   *
  *                                                                         *
  ***************************************************************************/
+
+// include files for Qt
 #include <qpushbutton.h>
 #include <qcheckbox.h>
+#include <qcombobox.h>
 #include <qspinbox.h>
 #include <qtimer.h>
+#include <qradiobutton.h>
+#include <qimage.h>
+
+// include files for KDE
 #include <kurlrequester.h>
 #include <kimageio.h>
 #include <kfiledialog.h>
 #include <klocale.h>
 #include <kdebug.h>
 
+// application specific includes
 #include "kstgraphfiledialog_i.h"
 
 KstGraphFileDialogI::KstGraphFileDialogI(QWidget* parent, const char* name,
                                          bool modal, WFlags fl)
 : KstGraphFileDialog(parent, name, modal, fl) {
+  QString str;
+  int i;
+
   autoSaveTimer = new QTimer(this);
 
   connect(_square,       SIGNAL(clicked()),         this, SLOT(square()));
@@ -41,6 +52,18 @@ KstGraphFileDialogI::KstGraphFileDialogI(QWidget* parent, const char* name,
 
   _url->setFilter(KImageIO::mimeTypes().join(" "));
   _url->setMode(KFile::File);
+
+  _comboBoxFormats->insertStrList(QImageIO::outputFormats());
+  _comboBoxFormats->setCurrentItem(0);
+  for (i=0; i<_comboBoxFormats->count(); i++) {
+    str = _comboBoxFormats->text( i );
+    if (str.upper() == "PNG") {
+      _comboBoxFormats->setCurrentItem(i);
+      break;
+    }
+  }
+  
+  _radioButtonAll->setChecked(true);
 }
 
 KstGraphFileDialogI::~KstGraphFileDialogI() {
@@ -62,7 +85,7 @@ void KstGraphFileDialogI::xsizeChanged(int x) {
 }
 
 void KstGraphFileDialogI::reqGraphFile() {
-  emit graphFileReq(_url->url(), _xSize->value(), _ySize->value());
+  emit graphFileReq(_url->url(), _comboBoxFormats->currentText(), _xSize->value(), _ySize->value(), _radioButtonAll->isChecked());
 }
 
 void KstGraphFileDialogI::setAutoSave() {
