@@ -36,6 +36,11 @@ KstHistogram::KstHistogram(const QString &in_tag, KstVectorPtr in_V,
                            KstHsNormType in_norm_mode,
                            const QColor &in_color)
 : KstBaseCurve() {
+  setHasPoints(false);
+  setHasLines(true);
+  setLineWidth(0);
+  setLineStyle(0);
+
   commonConstructor(in_tag, in_V, xmin_in, xmax_in, in_n_bins, in_norm_mode,
                     in_color);
 }
@@ -61,17 +66,20 @@ KstHistogram::KstHistogram(QDomElement &e)
       if (e.tagName() == "tag") {
         in_tag = e.text();
       } else if (e.tagName() == "vectag") {
+        KST::vectorList.lock().readLock();
         KstVectorList::Iterator it = KST::vectorList.findTag(e.text());
-        if (it != KST::vectorList.end())
+        if (it != KST::vectorList.end()) {
           in_V = *it;
+        }
+        KST::vectorList.lock().readUnlock();
       } else if (e.tagName() == "color") {
         in_color.setNamedColor(e.text());
       } else if (e.tagName() == "hasLines") {
-	HasLines = (e.text() != "0");
+        HasLines = (e.text() != "0");
       } else if (e.tagName() == "hasPoints") {
-	HasPoints = (e.text() != "0");
+        HasPoints = (e.text() != "0");
       } else if (e.tagName() == "pointType") {
-	Point.setType(e.text().toInt());
+        Point.setType(e.text().toInt());
       } else if (e.tagName() == "NormMode") {
         if (e.text()=="NUMBER") in_norm_mode = KST_HS_NUMBER;
         else if (e.text()=="PERCENT") in_norm_mode = KST_HS_PERCENT;
@@ -121,7 +129,9 @@ void KstHistogram::commonConstructor(const QString &in_tag, KstVectorPtr in_V,
   }
 
   NBins = in_n_bins;
-  if (NBins<2) NBins = 2;
+  if (NBins<2) {
+    NBins = 2;
+  }
   Bins = (unsigned long *)malloc(in_n_bins*sizeof(unsigned long *));
 
   Color = in_color;
@@ -329,11 +339,10 @@ QString KstHistogram::propertyString() const {
 }
 
 void KstHistogram::_showDialog() {
-  KstHsDialogI::globalInstance()->show_I(tagName());
+  KstHsDialogI::globalInstance()->show_Edit(tagName());
 }
 
-void KstHistogram::AutoBin(KstVectorPtr V,
-                           int *n, double *max, double *min) {
+void KstHistogram::AutoBin(KstVectorPtr V, int *n, double *max, double *min) {
   double m;
 
   *max = V->max();
@@ -359,5 +368,4 @@ void KstHistogram::AutoBin(KstVectorPtr V,
   m = (*max - *min)/(100.0*double(*n));
   *max += m;
   *min -= m;
-
 }
