@@ -63,18 +63,17 @@ public:
   double getVMin() const;
   int getVNumSamples() const;
 
-  bool isNormNum()        const { return (NormMode == KST_HS_NUMBER); }
+  bool isNormNum()        const { return NormMode == KST_HS_NUMBER; }
   void setIsNormNum()           { NormMode = KST_HS_NUMBER; }
-  bool isNormPercent()    const { return (NormMode == KST_HS_PERCENT); }
+  bool isNormPercent()    const { return NormMode == KST_HS_PERCENT; }
   void setIsNormPercent()       { NormMode = KST_HS_PERCENT; }
-  bool isNormFraction()   const { return (NormMode == KST_HS_FRACTION); }
+  bool isNormFraction()   const { return NormMode == KST_HS_FRACTION; }
   void setIsNormFraction()      { NormMode = KST_HS_FRACTION; }
-  bool isNormOne()        const { return (NormMode == KST_HS_MAX_ONE); }
+  bool isNormOne()        const { return NormMode == KST_HS_MAX_ONE; }
   void setIsNormOne()           { NormMode = KST_HS_MAX_ONE; }
 
   static void AutoBin(const KstVectorPtr, int *n, double *max, double *min);
 
-protected:
   virtual void _showDialog();
 
 private:
@@ -95,7 +94,7 @@ private:
   KstHsNormType NormMode;
 };
 
-typedef KSharedPtr<KstHistogram> KstHistogramPtr;
+typedef KstSharedPtr<KstHistogram> KstHistogramPtr;
 typedef KstObjectList<KstHistogramPtr> KstHistogramList;
 
 #endif
