@@ -35,7 +35,10 @@ public:
 	virtual QStringList vectorList();
 	virtual QStringList objectList();
 	virtual QStringList curveList();
+	virtual QString activeWindow();
+	virtual QStringList windowList();
 	virtual QStringList plotList();
+	virtual QStringList plotList(const QString& window);
   virtual QStringList pluginList();
   virtual QStringList filterList();
   virtual QStringList filterSetList();
@@ -62,13 +65,24 @@ public:
   virtual bool clearVector(const QString& name);
   virtual int vectorSize(const QString& name);
 
+  virtual bool printImage(const QString& windowname, const QString& url);
   virtual bool printImage(const QString& filename);
+  virtual bool printPostScript(const QString& windowname, const QString& filename);
   virtual bool printPostScript(const QString& filename);
 
-  virtual const QString& createPlot(const QString& name);
+  virtual QString createWindow(const QString& name);
+  virtual QString createPlot(const QString& window, const QString& name);
+  virtual QString createPlot(const QString& name);
+
+  virtual bool deletePlot(const QString& window, const QString& name);
   virtual bool deletePlot(const QString& name);
+
   virtual QStringList plotContents(const QString& name);
+  bool addCurveToPlot(KMdiChildView *win, const QString& plot, const QString& curve);
+  virtual bool addCurveToPlot(const QString& window, const QString& plot, const QString& curve);
   virtual bool addCurveToPlot(const QString& plot, const QString& curve);
+  bool removeCurveFromPlot(KMdiChildView *win, const QString& plot, const QString& curve);
+  virtual bool removeCurveFromPlot(const QString& window, const QString& plot, const QString& curve);
   virtual bool removeCurveFromPlot(const QString& plot, const QString& curve);
 
   virtual const QString& createCurve(const QString& name, const QString& xVector, const QString& yVector, const QString& xErrorVector, const QString& yErrorVector);
@@ -86,6 +100,57 @@ public:
   virtual bool open(const QString& fileName);
 
 
+ //------------------Suggested functions-------------------------
+
+  virtual bool setMDIMode(int mode); //set the MDI mode
+
+  virtual bool setTileMode(int mode);  //set the tile mode in childframe mode
+
+  virtual bool closeWindow(const QString& windowname);  //close window
+
+  virtual bool activateWindow(const QString& windowname);  //make window active
+
+  virtual bool positionWindow(int topleftX,
+  			      int topleftY,
+			      int width,
+			      int height); //position/resize window
+
+  virtual bool maximizePlot(const QString& plotName); //zoom (maximize) plot
+
+  virtual bool setPlotAxes(const QString& plotName,
+  			   int XLower,
+  			   int XUpper,
+			   int YLower,
+			   int YUpper); //change plot axes
+
+  virtual QString createHistogram(const QString& name,
+  			       const QString& vector,
+  			       double min,
+			       double max,
+			       int numBins,
+			       int normalizationType,
+                               const QColor& colour); //create histogram
+
+  virtual QString createPowerSpectrum(const QString & name,
+				   const QString& vector,
+  				   bool appodize,
+				   bool removeMean,
+				   int fftLength,
+				   const QString& rateUnits,
+				   double sampleRate,
+				   const QString& vectorUnits,
+                                   const QColor& colour); //create power spectrum
+
+  virtual QString createEvent(const QString& name,
+   			   const QString& expression,
+  			   const QString& description,
+			   int debugLogType,
+			   const QString& email); //create event
+
+  virtual QStringList createPlugin(const QString& pluginName,
+                            const QStringList& scalarInputs,
+                            const QStringList& vectorInputs);  //create plugin
+
 private:
   KstDoc *_doc;
   KstApp *_app;
