@@ -24,6 +24,8 @@
 #include <qpainter.h>
 #include <qstring.h>
 
+#include "kstplotbase.h"
+
 typedef enum {CxBy, CxTy, CxCy, LxBy, LxTy,
               LxCy, RxBy, RxTy, RxCy} KstJustifyType;
 
@@ -73,12 +75,12 @@ public:
 
   void setDoScalarReplacement(bool in_do);
 
-  inline double x() const {return _x;}
-  inline double y() const {return _y;}
+  inline double x() const { return _x; }
+  inline double y() const { return _y; }
 
   QRegion extents;
   int     v_offset;
-  
+
 private:
   /** Computes the font size, given the size of the painter, and Size */
   int fontSize(QPainter &p);
@@ -109,7 +111,7 @@ private:
 
 class KstLabelDrag : public QDragObject {
   public:
-    KstLabelDrag(QWidget *dragSource, int p, int n, const QPoint& hotSpot, const QPixmap& pm);
+    KstLabelDrag(QWidget *dragSource, const QString& window, KstPlotBasePtr plot, int n, const QPoint& hotSpot, const QPixmap& pm);
     virtual ~KstLabelDrag();
 
     virtual const char *format(int i = 0) const;
@@ -117,7 +119,9 @@ class KstLabelDrag : public QDragObject {
     virtual QByteArray encodedData(const char *mime) const;
 
   private:
-    int _plot, _label;
+    int _label;
+    QString _window;
+    KstPlotBasePtr _plot;
     QPoint _hs;
 };
 
