@@ -14,53 +14,51 @@
  *   (at your option) any later version.                                   *
  *                                                                         *
  ***************************************************************************/
-#include <qwidget.h>
+
+#include <qcombobox.h>
 #include <qfontdatabase.h>
-#include <qstring.h>
 #include <qlistbox.h>
 #include <qpushbutton.h>
-#include <qcombobox.h>
 #include <qradiobutton.h>
-#include <klineedit.h>
+#include <qstring.h>
+#include <qwidget.h>
+
 #include <kfontcombo.h>
+#include <klineedit.h>
 #include <knuminput.h>
+#include <kmdimainfrm.h>
 
-#include "kstlabeldialog_i.h"
 #include "kstdatacollection.h"
-#include "kstplotlist.h"
+#include "kstlabeldialog_i.h"
 #include "scalarselector.h"
+#include "kstviewwindow.h"
+#include "kst.h"
 
-KstLabelDialogI::KstLabelDialogI(QWidget *parent,
-                                 const char* name,
-                                 bool modal,
-                                 WFlags fl) :
-  KstLabelDialog(parent, name, modal, fl) {
-    _x = _y = 0.0;
-    _i_plot = 0;
+KstLabelDialogI::KstLabelDialogI(QWidget *parent, const char* name, bool modal,
+                                 WFlags fl)
+: KstLabelDialog(parent, name, modal, fl) {
+  _x = _y = 0.0;
+  _i_plot = 0;
 
-    FontComboBox->setCurrentFont(this->font().family());
+  FontComboBox->setCurrentFont(this->font().family());
 
-    connect(Apply, SIGNAL(clicked()),
-            this, SLOT(apply()));
-    connect(OK, SIGNAL(clicked()),
-            this, SLOT(ok()));
-    connect(Delete, SIGNAL(clicked()),
-            this, SLOT(deleteL()));
+  connect(Apply, SIGNAL(clicked()), this, SLOT(apply()));
+  connect(OK, SIGNAL(clicked()), this, SLOT(ok()));
+  connect(Delete, SIGNAL(clicked()), this, SLOT(deleteL()));
 }
 
 KstLabelDialogI::~KstLabelDialogI() {
 }
 
-void KstLabelDialogI::showI(int i_plot, int i_label,
-                            double in_x, double in_y) {
+void KstLabelDialogI::showI(Kst2DPlotPtr plot, int i_label, double in_x, double in_y) {
   _x = in_x;
   _y = in_y;
-  _i_plot = i_plot;
+  _i_plot = plot;
   if (i_label < 0) {
     _editing = false;
     _i_label = 0;
     LabelText->setText(QString::null);
-  } else if ((int)KST::plotList.at(i_plot)->labelList.count() > i_label) {
+  } else if (plot->labelList.count() > (unsigned) i_label) {
     _editing = true;
     _i_label = i_label;
   } else {
@@ -74,31 +72,27 @@ void KstLabelDialogI::showI(int i_plot, int i_label,
 
 void KstLabelDialogI::updateI() {
   _scalars->update();
-  if (_i_plot >= (int)KST::plotList.count())
-    _i_plot = (int)KST::plotList.count() - 1;
 
-  if (_editing && _i_plot >= 0) {
-    KstPlot *plot = KST::plotList.at(_i_plot);
-
-    if ((int)plot->labelList.count()<=_i_label) {
+  if (_editing && _i_plot) {
+    if ((int)_i_plot->labelList.count() <= _i_label) {
       _editing = false;
       _i_label = 0;
     } else {
-      KstLabel *label = plot->labelList.at(_i_label);
+      KstLabel *label = _i_plot->labelList.at(_i_label);
       LabelText->setText(label->text());
       FontSize->setValue(label->size());
       switch (label->justification()) {
-          case LxBy:
-            LeftJustify->setChecked(true);
-            break;
-          case CxBy:
-            CenterJustify->setChecked(true);
-            break;
-          case RxBy:
-            RightJustify->setChecked(true);
-            break;
-          default:
-            LeftJustify->setChecked(true);
+        case LxBy:
+          LeftJustify->setChecked(true);
+          break;
+        case CxBy:
+          CenterJustify->setChecked(true);
+          break;
+        case RxBy:
+          RightJustify->setChecked(true);
+          break;
+        default:
+          LeftJustify->setChecked(true);
       }
       FontComboBox->setCurrentFont(label->fontName());
       Rotation->setValue((int)label->rotation());
@@ -110,25 +104,21 @@ void KstLabelDialogI::updateI() {
 
 void KstLabelDialogI::apply() {
   //updateI();
-
   if (_editing) {
     applyEdits();
   } else{
     applyAsNew();
   }
 
+  KMdiChildView *c = KstApp::inst()->activeWindow();
+  if (c) {
+    static_cast<KstViewWindow*>(c)->view()->paint(P_PLOT);
+  }
+
   emit applied();
 }
 
 void KstLabelDialogI::applyAsNew() {
-
-  if (_i_plot >= (int)KST::plotList.count()) {
-    // some surprising badness...  shouldn't happen - so just bail
-    _i_plot = (int)KST::plotList.count()-1;
-    _editing = false;
-    return;
-  }
-
   KstLabel *new_label;
   KstJustifyType J;
 
@@ -145,36 +135,30 @@ void KstLabelDialogI::applyAsNew() {
   new_label->setFontName(FontComboBox->currentText());
   new_label->setSize(FontSize->value()); // FIXME: spin box name is wrong
 
-  KST::plotList.at(_i_plot)->labelList.append(new_label);
-  _i_label = KST::plotList.at(_i_plot)->labelList.count()-1;
+  _i_plot->labelList.append(new_label);
+  _i_label = _i_plot->labelList.count() - 1;
   _editing = true;
+  _i_plot->update();
 }
 
 void KstLabelDialogI::applyEdits() {
-
-  if (_i_plot >= (int)KST::plotList.count()) {
-    // some surprising badness...  shouldn't happen - so just bail
-    _i_plot = (int)KST::plotList.count()-1;
-    _editing = false;
-    return;
-  }
-
-  if (_i_plot>=0) {
-    KstPlot *plot = KST::plotList.at(_i_plot);
-
-    if ((int)plot->labelList.count()<=_i_label) {
+  if (_i_plot) {
+    if ((int)_i_plot->labelList.count() <= _i_label) {
     // some surprising badness...  shouldn't happen - so just bail
       _editing = false;
       _i_label = 0;
       return;
     } else {
-
-      KstLabel *label = plot->labelList.at(_i_label);
+      KstLabel *label = _i_plot->labelList.at(_i_label);
       KstJustifyType J;
 
-      if (LeftJustify->isChecked()) J = LxBy;
-      else if (CenterJustify->isChecked()) J = CxBy;
-      else J = RxBy;
+      if (LeftJustify->isChecked()) {
+        J = LxBy;
+      } else if (CenterJustify->isChecked()) {
+        J = CxBy;
+      } else {
+        J = RxBy;
+      }
 
       label->setText(LabelText->text());
       label->setJustification(J);
@@ -182,6 +166,7 @@ void KstLabelDialogI::applyEdits() {
       label->setFontName(FontComboBox->currentText());
       label->setRotation((float)Rotation->value());
     }
+    _i_plot->update();
   }
 }
 
@@ -196,26 +181,18 @@ void KstLabelDialogI::deleteL() {
     return;
   }
 
-  if (_i_plot >= (int)KST::plotList.count()) {
-    // some surprising badness...  shouldn't happen - so just bail
-    _i_plot = (int)KST::plotList.count()-1;
-    _editing = false;
-    close();
-    return;
-  }
-  if (_i_plot>=0) {
-    KstPlot *plot = KST::plotList.at(_i_plot);
-
-    if ((int)plot->labelList.count()<=_i_label) {
-    // some surprising badness...  shouldn't happen - so just bail
+  if (_i_plot) {
+    if ((int)_i_plot->labelList.count() <= _i_label) {
+      // some surprising badness...  shouldn't happen - so just bail
       _editing = false;
       _i_label = 0;
       close();
       return;
     } else {
-      plot->labelList.remove(_i_label);
+      _i_plot->labelList.remove(_i_label);
       emit applied();
     }
+    _i_plot->update();
   }
   _editing = false;
   close();
@@ -223,3 +200,4 @@ void KstLabelDialogI::deleteL() {
 
 
 #include "kstlabeldialog_i.moc"
+// vim: ts=2 sw=2 et
