@@ -18,20 +18,24 @@
 #ifndef KSTOBJECT_H
 #define KSTOBJECT_H
 
-#include <ksharedptr.h>
+#include "kstsharedptr.h"
+
+#include "rwlock.h"
+
+#include <qmutex.h>
+#include <qobject.h>
 #include <qstring.h>
 #include <qstringlist.h>
-#include <qobject.h>
 
 class KstObjectPrivate;
 
 // BINARY COMPATIBILITY IS NOT YET GUARANTEED
-class KstObject : public KShared, public QObject {
+class KstObject : public KstShared, public QObject, public KstRWLock {
 public:
   KstObject();
   virtual ~KstObject();
 
-  typedef enum {NO_CHANGE=0, READ, UPDATE} UpdateType;
+  typedef enum {NO_CHANGE=0, UPDATE} UpdateType;
 
   virtual UpdateType update(int updateCounter = -1) = 0;
   virtual const QString& tagName() const;
@@ -61,15 +65,22 @@ private:
   //       them.  You can simulate virtual functions with the virtual_hook().
 };
 
+typedef KstSharedPtr<KstObject> KstObjectPtr;
 
 #include <qvaluelist.h>
 
 template<class T>
 class KstObjectList : public QValueList<T> {
+  friend class QDeepCopy<KstObjectList<T> >;
   public:
     KstObjectList() : QValueList<T>() {}
-    KstObjectList(const QValueList<T>& x) : QValueList<T>(x) {}
-    virtual ~KstObjectList() {}
+    KstObjectList(const KstObjectList<T>& x) : QValueList<T>(x) {}
+    virtual ~KstObjectList() { }
+
+    KstObjectList& operator=(const KstObjectList& l) {
+      this->QValueList<T>::operator=(l);
+      return *this;
+    }
 
     virtual QStringList tagNames() {
       QStringList rc;
@@ -115,12 +126,19 @@ class KstObjectList : public QValueList<T> {
       }
       return it;
     }
+
+    KstRWLock& lock() const { return _lock; }
+
+  private:
+    mutable KstRWLock _lock;
 };
 
+/* Does locking for you automatically. */
 template<class T, class S>
-KstObjectList<KSharedPtr<S> > kstObjectSubList(KstObjectList<KSharedPtr<T> >& list) {
-  KstObjectList<KSharedPtr<S> > rc;
-  typename KstObjectList<KSharedPtr<T> >::Iterator it;
+KstObjectList<KstSharedPtr<S> > kstObjectSubList(KstObjectList<KstSharedPtr<T> >& list) {
+  list.lock().readLock();
+  KstObjectList<KstSharedPtr<S> > rc;
+  typename KstObjectList<KstSharedPtr<T> >::Iterator it;
 
   for (it = list.begin(); it != list.end(); ++it) {
     S *x = dynamic_cast<S*>((*it).data());
@@ -129,7 +147,27 @@ KstObjectList<KSharedPtr<S> > kstObjectSubList(KstObjectList<KSharedPtr<T> >& li
     }
   }
 
-return rc;
+  list.lock().readUnlock();
+  return rc;
+}
+
+
+/* Does locking for you automatically. */
+template<class T, class S>
+void kstObjectSplitList(KstObjectList<KstSharedPtr<T> >& list, KstObjectList<KstSharedPtr<S> >& inclusive, KstObjectList<KstSharedPtr<T> >& exclusive) {
+  list.lock().readLock();
+  typename KstObjectList<KstSharedPtr<T> >::Iterator it;
+
+  for (it = list.begin(); it != list.end(); ++it) {
+    S *x = dynamic_cast<S*>((*it).data());
+    if (x != 0L) {
+      inclusive.append(x);
+    } else {
+      exclusive.append(*it);
+    }
+  }
+
+  list.lock().readUnlock();
 }
 
 
