@@ -16,44 +16,47 @@
  ***************************************************************************/
 
 // include files for Qt
+#include <qcheckbox.h>
 #include <qdir.h>
 #include <qfileinfo.h>
-#include <qwidget.h>
-#include <qtextstream.h>
-#include <qlineedit.h>
-#include <qspinbox.h>
 #include <qfontdatabase.h>
-#include <qcheckbox.h>
-#include <qstring.h>
-#include <klistbox.h>
-#include <qspinbox.h>
+#include <qlineedit.h>
 #include <qmessagebox.h>
+#include <qspinbox.h>
+#include <qstring.h>
 #include <qtabwidget.h>
-
-// include files for KDE
-#include <klocale.h>
-#include <kcolorbutton.h>
-#include <kdualcolorbutton.h>
-#include <kmessagebox.h>
-#include <kfontcombo.h>
-#include <kdebug.h>
+#include <qtextstream.h>
+#include <qwidget.h>
 #include <qpushbutton.h>
 #include <qradiobutton.h>
 
+// include files for KDE
+#include <kcolorbutton.h>
+#include <kdebug.h>
+#include <kdualcolorbutton.h>
+#include <kfontcombo.h>
+#include <klistbox.h>
+#include <klocale.h>
+#include <kmessagebox.h>
+#include <kmdimainfrm.h>
+
+// application specific includes
+#include "kst.h"
 #include "kstdatacollection.h"
 #include "kstdebug.h"
 #include "kstlegend.h"
 #include "kstplotdialog_i.h"
 #include "kstsettings.h"
+#include "kstviewwindow.h"
 
 struct LegendLayout {
   QString Label;
-  KstLegendLayoutType	type;
+  KstLegendLayoutType type;
 };
 
 struct LegendAlignment {
   QString Label;
-  KstLegendAlignmentType	type;
+  KstLegendAlignmentType type;
 };
 
 const LegendAlignment LegendAlignments[] = {
@@ -74,8 +77,8 @@ const LegendLayout LegendLayouts[] = {
   { i18n("Bottom right"), RightBottom }
 };
 
-const int iNumLegendLayouts = sizeof( LegendLayouts ) / sizeof( LegendLayout );
-const int iNumLegendAlignments = sizeof( LegendAlignments ) / sizeof( LegendAlignment );
+const unsigned int iNumLegendLayouts = sizeof( LegendLayouts ) / sizeof( LegendLayout );
+const unsigned int iNumLegendAlignments = sizeof( LegendAlignments ) / sizeof( LegendAlignment );
 
 /*
  *  Constructs a KstPlotDialogI which is a child of 'parent', with the
@@ -88,6 +91,7 @@ KstPlotDialogI::KstPlotDialogI(KstDoc *in_doc, QWidget* parent,
                                const char* name, bool modal, WFlags fl)
 : KstPlotDialog(parent, name, modal, fl ) {
   QString str;
+  unsigned int i;
 
   doc = in_doc;
   setScalarDestTopLabel();
@@ -98,20 +102,24 @@ KstPlotDialogI::KstPlotDialogI(KstDoc *in_doc, QWidget* parent,
   connect(Edit, SIGNAL(clicked()), this, SLOT(edit_I()));
   connect(Delete, SIGNAL(clicked()), this, SLOT(delete_I()));
 
-  connect(DisplayedCurveList, SIGNAL(selectionChanged(QListBoxItem*)),
+  connect(DisplayedCurveList, SIGNAL(clicked(QListBoxItem*)),
           this, SLOT(updateButtons()));
-  connect(AvailableCurveList, SIGNAL(selectionChanged(QListBoxItem*)),
+  connect(AvailableCurveList, SIGNAL(clicked(QListBoxItem*)),
+          this, SLOT(updateButtons()));
+  connect(DisplayedCurveList, SIGNAL(selectionChanged()),
+          this, SLOT(updateButtons()));
+  connect(AvailableCurveList, SIGNAL(selectionChanged()),
           this, SLOT(updateButtons()));
 
   /* adding/removing curves */
   connect(CurveUnplot, SIGNAL(clicked()),
           this, SLOT(removeDisplayedCurve()));
   connect(DisplayedCurveList, SIGNAL(executed(QListBoxItem*)),
-          this, SLOT(removeDisplayedCurve()));
+          this, SLOT(removeDisplayedCurve(QListBoxItem*)));
   connect(CurvePlot, SIGNAL(clicked()),
           this, SLOT(addDisplayedCurve()));
   connect(AvailableCurveList, SIGNAL(executed(QListBoxItem*)),
-          this, SLOT(addDisplayedCurve()));
+          this, SLOT(addDisplayedCurve(QListBoxItem*)));
 
   /* labels */
   connect(TopLabelText, SIGNAL(textChanged(const QString &)),
@@ -157,6 +165,8 @@ KstPlotDialogI::KstPlotDialogI(KstDoc *in_doc, QWidget* parent,
   QFontDatabase qfd;
   FontComboBox->setFonts(qfd.families());
 
+  connect(_window, SIGNAL(activated(int)), this, SLOT(updatePlotList()));
+
   /* Legend */
   connect(LegendFontSize, SIGNAL(valueChanged(int)),
           this, SLOT(updateSampLegendLabel()));
@@ -179,17 +189,20 @@ KstPlotDialogI::KstPlotDialogI(KstDoc *in_doc, QWidget* parent,
 
   LegendPosition->hide();
   LegendPositionLabel->hide();
-  LegendPosition->setEditable( false );
-  for( int i=0; i<iNumLegendLayouts; i++ ) {
+  LegendPosition->setEditable(false);
+  for (i = 0; i < iNumLegendLayouts; i++) {
     str = LegendLayouts[i].Label;
-    LegendPosition->insertItem( str );
+    LegendPosition->insertItem(str);
   }
 
-  LegendAlignment->setEditable( false );
-  for( int i=0; i<iNumLegendAlignments; i++ ) {
+  LegendAlignment->setEditable(false);
+  for (i = 0; i < iNumLegendAlignments; i++) {
     str = LegendAlignments[i].Label;
-    LegendAlignment->insertItem( str );
+    LegendAlignment->insertItem(str);
   }
+
+  radioButtonThisPlot->setChecked( true );
+  radioButtonLegendThisPlot->setChecked( true );
 }
 
 KstPlotDialogI::~KstPlotDialogI() {
@@ -197,21 +210,36 @@ KstPlotDialogI::~KstPlotDialogI() {
   SampleLabel = 0L;
 }
 
+void KstPlotDialogI::show_I(const QString& strWindow, const QString& strPlot) {
+  _strWindow = strWindow;
+  _strPlot = strPlot;
+
+  show();
+  raise();
+  update();
+}
+
 void KstPlotDialogI::show_I() {
   show();
   raise();
   update();
 }
 
-void KstPlotDialogI::newTab( ) {
-    updateLabels();
-    updateSampLegendLabel();
+void KstPlotDialogI::newTab() {
+  updateLabels();
+  updateSampLegendLabel();
+}
+
+void KstPlotDialogI::updateWindow() {
+  updateWindowList();
 }
 
 void KstPlotDialogI::update(int new_index) {
-  int index;
+  Q_UNUSED(new_index);
+
   int i_plot;
-  KstPlot *plot;
+  KstApp *app = KstApp::inst();
+  Kst2DPlotPtr plot;
   KstBaseCurvePtr curve;
   double xmin, ymin, xmax, ymax;
 
@@ -219,36 +247,10 @@ void KstPlotDialogI::update(int new_index) {
     return;
   }
 
-  KstBaseCurveList curves = kstObjectSubList<KstDataObject, KstBaseCurve>(KST::dataObjectList);
-  /* Fill the scalar combo */
-  ScalarList->clear();
-  for (index = 0; index < (int)KST::scalarList.count(); index++) {
-    ScalarList->insertItem(KST::scalarList[index]->tagLabel());
-  }
-
-  /* Fill the Select combo, and set it to the right item */
-  if (new_index >= 0) {
-    index = new_index;
-  } else if (Select->count() > 0) {
-    index = Select->currentItem();
-  } else {
-    index = Select->count()-1;
-  }
-
-  Select->clear();
-  for (i_plot = 0; i_plot < (int)KST::plotList.count(); i_plot++) {
-    Select->insertItem(KST::plotList.at(i_plot)->tagName());
-  }
-
-  if (index >= 0 && index < Select->count()) {
-    Select->setCurrentItem(index);
-  } else if (Select->count() > 0) {
-    Select->setCurrentItem(Select->count()-1);
-  }
-
-  /* Fill the displayed and available curve lists */
-  DisplayedCurveList->clear();
-  AvailableCurveList->clear();
+  updateScalarCombo();
+  updateWindowList();
+  updatePlotList();
+  updateCurveLists();
 
   if (Select->count() > 0) {
     i_plot = Select->currentItem();
@@ -258,23 +260,21 @@ void KstPlotDialogI::update(int new_index) {
     PlotPosition->setValue(0);
   }
 
-  if (i_plot >= 0) {
-    plot = KST::plotList.at(i_plot);
+  KMdiChildView *c = app->findWindow(_window->currentText());
 
-//     Colors->setForeground(plot->foregroundColor());
-//     Colors->setBackground(plot->backgroundColor());
+  if (c) {
+    KstTopLevelViewPtr view = static_cast<KstViewWindow*>(c)->view();
+    _reGrid->setChecked(view->onGrid());
+    _plotColumns->setValue(view->columns());
+  }
+
+  if (i_plot >= 0 && c) {
+    KstViewObjectPtr obj = static_cast<KstViewWindow*>(c)->view()->findChild(Select->currentText());
+    plot = dynamic_cast<Kst2DPlot*>(obj.data());
 
     /* insert the current plot name in the plot name edit box */
     Name->setText(plot->tagName());
 
-    /* fill the plotted and available curves boxes */
-    for (KstBaseCurveList::iterator it = curves.begin(); it != curves.end(); it++) {
-      if (plot->Curves.find(*it) == plot->Curves.end()) {
-        AvailableCurveList->insertItem((*it)->tagName());
-      } else {
-        DisplayedCurveList->insertItem((*it)->tagName());
-      }
-    }
     /* file the log axis check boxes */
     XIsLog->setChecked(plot->isXLog());
     YIsLog->setChecked(plot->isYLog());
@@ -342,85 +342,113 @@ void KstPlotDialogI::update(int new_index) {
     legendColors->setBackground(plot->Legend->getColorBackground());
 
     updateSampLegendLabel();
-  } else {
-    for (KstBaseCurveList::iterator it = curves.begin(); it != curves.end(); it++) {
-      AvailableCurveList->insertItem((*it)->tagName());
-    }
-  }
 
-  PlotCols->setValue(KST::plotList.getPlotCols());
+    /*initialize the plot color widget */
+    plotColors->setForeground(plot->foregroundColor());
+    plotColors->setBackground(plot->backgroundColor());
+  }
 
   //UpdateDelay->setValue(doc->delay());
 
   updateButtons();
 }
 
-void KstPlotDialogI::applyLabels(KstPlot *plot) {
+void KstPlotDialogI::fill2DPlotList( Kst2DPlotList& plots ) {
+  KstApp* pApp = KstApp::inst();
+  KMdiIterator<KMdiChildView*> *it = pApp->createIterator();
+  while (it->currentItem()) {
+    KstViewWindow *c = dynamic_cast<KstViewWindow*>(it->currentItem());
+    if (c) {
+      KstPlotBaseList allplots = c->view()->findChildrenType<KstPlotBase>(true);
+      for (KstPlotBaseList::Iterator i = allplots.begin(); i != allplots.end(); ++i) {
+        Kst2DPlotPtr plot = dynamic_cast<Kst2DPlot*>((*i).data());
+        if (plot) {
+          plots += plot;
+        }
+      }
+    }
+    it->next();
+  }
+  pApp->deleteIterator(it);
+}
+
+
+void KstPlotDialogI::applyLabels(Kst2DPlotPtr plot) {
   QString strAlignment;
   QString strLayout;
-  KstLegendLayoutType layout = LeftTop;
-  KstLegendAlignmentType alignment = AlignmentRight;
-  int i, i_plot, plot_min, plot_max;
+  Kst2DPlotList plots;
+  unsigned int i;
 
   plot->XLabel->setText((XAxisText)->text());
   plot->YLabel->setText((YAxisText)->text());
   plot->TopLabel->setText((TopLabelText)->text());
 
-  /** Set the font info */
-  if (applyFontsToAll->isChecked()) {
-    i_plot = plot_min = 0;
-    plot_max = KST::plotList.count() - 1;
+  if (radioButtonThisPlot->isChecked()) {
+    plots += plot;
+  } else if (radioButtonAllPlotsInWindow->isChecked()) {
+    KMdiChildView *c = KstApp::inst()->findWindow(_window->currentText());
+    if (c) {
+      plots  = static_cast<KstViewWindow*>(c)->view()->findChildrenType<Kst2DPlot>(true);
+    }
   } else {
-    plot_min = plot_max = i_plot = Select->currentItem();
+    fill2DPlotList( plots );
   }
 
-  for (i_plot = plot_min; i_plot <= plot_max; i_plot++) {
-    plot = KST::plotList.at(i_plot);
+  for (i=0; i<plots.size(); i++) {
+    plot = plots[i];
 
-    /* set fonts */
-    /* X Axis Font */
     plot->XLabel->setFontName(FontComboBox->currentText());
     plot->XLabel->setSize(XLabelFontSize->value());
 
-    /* Y Axis Font */
     plot->YLabel->setFontName(FontComboBox->currentText());
     plot->YLabel->setSize(YLabelFontSize->value());
 
-    /* Top Label Font */
     plot->TopLabel->setFontName(FontComboBox->currentText());
     plot->TopLabel->setSize(TopLabelFontSize->value());
 
-    /* TickLabel Font */
     plot->TickLabel->setFontName(FontComboBox->currentText());
     plot->TickLabel->setSize(NumberFontSize->value());
   }
+}
 
-  /* legend */
-  if (applyLegendSettingsToAll->isChecked()) {
-    i_plot = plot_min = 0;
-    plot_max = KST::plotList.count() - 1;
+
+void KstPlotDialogI::applyLegends(Kst2DPlotPtr plot) {
+  QString strAlignment;
+  QString strLayout;
+  KstLegendLayoutType layout = LeftTop;
+  KstLegendAlignmentType alignment = AlignmentRight;
+  Kst2DPlotList plots;
+  unsigned int i;
+
+  if (radioButtonLegendThisPlot->isChecked()) {
+    plots += plot;
+  } else if (radioButtonAllPlotsInWindow->isChecked()) {
+    KMdiChildView *c = KstApp::inst()->findWindow(_window->currentText());
+    if (c) {
+      plots  = static_cast<KstViewWindow*>(c)->view()->findChildrenType<Kst2DPlot>(true);
+    }
   } else {
-    plot_min = plot_max = i_plot = Select->currentItem();
+    fill2DPlotList( plots );
   }
 
   strLayout = LegendPosition->currentText();
-  for( i=0; i<iNumLegendLayouts; i++) {
-    if( strLayout.compare( LegendLayouts[i].Label ) == 0 ) {
+  for (i = 0; i < iNumLegendLayouts; i++) {
+    if (strLayout.compare(LegendLayouts[i].Label) == 0) {
       layout = LegendLayouts[i].type;
       break;
     }
   }
 
   strAlignment = LegendAlignment->currentText();
-  for( i=0; i<iNumLegendAlignments; i++) {
-    if( strAlignment.compare( LegendAlignments[i].Label ) == 0 ) {
+  for (i = 0; i < iNumLegendAlignments; i++) {
+    if (strAlignment.compare(LegendAlignments[i].Label) == 0) {
       alignment = LegendAlignments[i].type;
       break;
     }
   }
 
-  for (i_plot = plot_min; i_plot <= plot_max; i_plot++) {
-    plot = KST::plotList.at(i_plot);
+  for (i=0; i<plots.size(); i++) {
+    plot = plots[i];
 
     plot->Legend->setFontName(LegendFontComboBox->currentText());
     plot->Legend->setSize(LegendFontSize->value());
@@ -433,7 +461,7 @@ void KstPlotDialogI::applyLabels(KstPlot *plot) {
   }
 }
 
-void KstPlotDialogI::applyLimits(KstPlot *plot) {
+void KstPlotDialogI::applyLimits(Kst2DPlotPtr plot) {
 
   /* do X Scale */
   if (XAC->isChecked()) {
@@ -472,22 +500,44 @@ void KstPlotDialogI::applyLimits(KstPlot *plot) {
   plot->setLog(XIsLog->isChecked(), YIsLog->isChecked());
 }
 
+void KstPlotDialogI::applyPlotColors(Kst2DPlotPtr plot) {
+  plot->setForegroundColor(plotColors->foreground());
+  plot->setBackgroundColor(plotColors->background());
+
+}
+
+
 void KstPlotDialogI::new_I() {
-  KstPlot *plot;
+  Kst2DPlotPtr plot;
 
-  /** verify that the plotname is unique */
-  if (KST::plotList.FindKstPlot(Name->text()) != 0) {
-    QString message = i18n("Could not create a new plot.\n"
-                           "%1: not a unique plot name.\n"
-                           "Change it to a unique name.").arg(Name->text());
-
-    KMessageBox::sorry(NULL,message);
-    Name->setFocus();
-    return;
+  KMdiChildView *c = KstApp::inst()->findWindow(_window->currentText());
+  if (!c) {
+    QString name = KstApp::inst()->newWindow();
+    c = KstApp::inst()->findWindow(name);
+    _window->insertItem(name);
+    _window->setCurrentText(name);
   }
 
-  plot = new KstPlot(Name->text());
-  KST::plotList.append(plot);
+  // FIXME: might want to ensure that the plot name is not in any other window
+  /** verify that the plotname is unique */
+  if (c) {
+    KstViewObjectPtr obj = static_cast<KstViewWindow*>(c)->view()->findChild(Name->text());
+    if (obj) {
+      QString message = i18n("Could not create a new plot.\n"
+          "%1: not a unique plot name.\n"
+          "Change it to a unique name.").arg(Name->text());
+
+      KMessageBox::sorry(0L, message);
+      Name->setFocus();
+      return;
+    }
+  }
+
+  plot = static_cast<KstViewWindow*>(c)->view()->createPlot<Kst2DPlot>(Name->text());
+
+  if (_reGrid->isChecked()) {
+    static_cast<KstViewWindow*>(c)->view()->cleanup(_plotColumns->value());
+  }
 
   KstBaseCurveList curves = kstObjectSubList<KstDataObject, KstBaseCurve>(KST::dataObjectList);
   /* add the curves */
@@ -501,18 +551,21 @@ void KstPlotDialogI::new_I() {
   curves.clear();
 
   applyLabels(plot);
+  applyLegends(plot);
   //plot->GenerateDefaultLabels();
 
   applyLimits(plot);
 
 //   plot->setForegroundColor(Colors->foreground());
 //   plot->setBackgroundColor(Colors->background());
+  applyPlotColors(plot);
+
+  static_cast<KstViewWindow*>(c)->view()->paint(P_PLOT);
 
   update();
 
   changePosition(PlotPosition->value());
 
-  KST::plotList.arrangePlots(PlotCols->value());
   //doc->setDelay(UpdateDelay->value());
 
   update();
@@ -520,32 +573,45 @@ void KstPlotDialogI::new_I() {
 }
 
 void KstPlotDialogI::edit_I() {
-  KstPlot *plot;
+  Kst2DPlotPtr plot;
 
   int index = Select->currentItem();
   if (index < 0) {
     KMessageBox::sorry(0L, i18n("You need to select an active plot to edit."));
     return;
   }
-  if (index >= (int)KST::plotList.count()) {
+
+  KMdiChildView *c = KstApp::inst()->findWindow(_window->currentText());
+  if (!c) {
+    return;
+  }
+
+  Kst2DPlotList plots = static_cast<KstViewWindow*>(c)->view()->findChildrenType<Kst2DPlot>(true);
+
+  if (index >= (int)plots.count()) {
     new_I();
   } else {
-    plot = KST::plotList.at(index);
+    plot = *plots.at(index);
+
+
     plot->setTagName(Name->text());
     /* add the curves */
     plot->Curves.clear();
     KstBaseCurveList curves = kstObjectSubList<KstDataObject, KstBaseCurve>(KST::dataObjectList);
     for (unsigned i = 0; i < DisplayedCurveList->count(); i++) {
-	    KstBaseCurveList::Iterator it = curves.findTag(DisplayedCurveList->text(i));
-	    if (it != curves.end()) {
-		    plot->Curves.append(*it);
-	    }
+      KstBaseCurveList::Iterator it = curves.findTag(DisplayedCurveList->text(i));
+      if (it != curves.end()) {
+        plot->Curves.append(*it);
+      }
     }
     curves.clear();
 
     applyLabels(plot);
+    applyLegends(plot);
     applyLimits(plot);
+    applyPlotColors(plot);
 
+//  FIXME: why is this commented out?
 //     if (ApplyToAll->isChecked()) {
 //       for (unsigned i = 0; i < KST::plotList.count(); i++) {
 //         KST::plotList.at(i)->setForegroundColor(Colors->foreground());
@@ -557,8 +623,12 @@ void KstPlotDialogI::edit_I() {
 //     }
     changePosition(PlotPosition->value());
 
-    KST::plotList.arrangePlots(PlotCols->value());
-    //doc->setDelay(UpdateDelay->value());
+    if (_reGrid->isChecked()) {
+      static_cast<KstViewWindow*>(c)->view()->cleanup(_plotColumns->value());
+    }
+
+    plot->setDirty();
+    static_cast<KstViewWindow*>(c)->view()->paint(P_PLOT);
 
     update();
     emit docChanged();
@@ -568,14 +638,26 @@ void KstPlotDialogI::edit_I() {
 void KstPlotDialogI::delete_I() {
   int index = Select->currentItem();
   if (index < 0) {
-    KMessageBox::sorry(0L,i18n("You need to select an active plot to delete."));
+    KMessageBox::sorry(0L, i18n("You need to select an active plot to delete."));
     return;
   }
 
-  KST::plotList.remove(index);
-  KST::plotList.arrangePlots(PlotCols->value());
+  KMdiChildView *c = KstApp::inst()->findWindow(_window->currentText());
+  if (!c) {
+    return;
+  }
+
+  KstViewObjectPtr obj = static_cast<KstViewWindow*>(c)->view()->findChild(Select->currentText());
+  static_cast<KstViewWindow*>(c)->view()->removeChild(obj);
+
+  if (_reGrid->isChecked()) {
+    static_cast<KstViewWindow*>(c)->view()->cleanup(_plotColumns->value());
+  }
+
   update();
 
+  static_cast<KstViewWindow*>(c)->view()->paint(P_PLOT);
+
   emit docChanged();
 }
 
@@ -701,35 +783,84 @@ void KstPlotDialogI::updateSampLegendLabel() {
   }
 }
 
-void KstPlotDialogI::removeDisplayedCurve(){
-  int i_curve = DisplayedCurveList->currentItem();
-  if (i_curve >= 0) {
-    AvailableCurveList->insertItem(DisplayedCurveList->currentText());
-    DisplayedCurveList->removeItem(i_curve);
+
+void KstPlotDialogI::removeDisplayedCurve() {
+  uint count = DisplayedCurveList->count();
+  int i;
+
+  if (count>0) {
+    for (i=count-1; i>=0; i--) {
+      if (DisplayedCurveList->isSelected(i)) {
+        AvailableCurveList->insertItem(DisplayedCurveList->text(i));
+        DisplayedCurveList->removeItem(i);
+      }
+    }
+    updateButtons();
   }
-  updateButtons();
 }
 
-void KstPlotDialogI::addDisplayedCurve(){
-  int i_curve = AvailableCurveList->currentItem();
-  if (i_curve >= 0) {
-    DisplayedCurveList->insertItem(AvailableCurveList->currentText());
-    AvailableCurveList->removeItem(i_curve);
+
+void KstPlotDialogI::removeDisplayedCurve(QListBoxItem* pItem){
+  QString str;
+  int index;
+
+  index = DisplayedCurveList->index(pItem);
+  if (index!=-1) {
+    str = DisplayedCurveList->text(index);
+    DisplayedCurveList->removeItem(index);
+    AvailableCurveList->insertItem(str);
+    updateButtons();
+  }
+}
+
+
+void KstPlotDialogI::addDisplayedCurve() {
+  uint count = AvailableCurveList->count();
+  int i;
+
+  if (count>0) {
+    for (i=count-1; i>=0; i--) {
+      if (AvailableCurveList->isSelected(i)) {
+        DisplayedCurveList->insertItem(AvailableCurveList->text(i));
+        AvailableCurveList->removeItem(i);
+      }
+    }
+    updateButtons();
+  }
+}
+
+
+void KstPlotDialogI::addDisplayedCurve(QListBoxItem* pItem){
+  QString str;
+  int index;
+
+  index = AvailableCurveList->index(pItem);
+  if (index!=-1) {
+    str = AvailableCurveList->text(index);
+    AvailableCurveList->removeItem(index);
+    DisplayedCurveList->insertItem(str);
+    updateButtons();
   }
-  updateButtons();
 }
 
 
 void KstPlotDialogI::applyAutoLabels() {
-  if (KST::plotList.count() < 1)
+  KMdiChildView *c = KstApp::inst()->findWindow(_window->currentText());
+  if (!c) {
     return;
+  }
 
-  KstPlot *plot = KST::plotList.at(Select->currentItem());
-  plot->GenerateDefaultLabels();
-  update();
+  KstViewObjectPtr obj = static_cast<KstViewWindow*>(c)->view()->findChild(Select->currentText());
+  Kst2DPlotPtr plot = dynamic_cast<Kst2DPlot*>(obj.data());
+  if (plot) {
+    plot->GenerateDefaultLabels();
+    update();
+  }
 }
 
 void KstPlotDialogI::changePosition(int ni) {
+// FIXMEPLOTLIST - not applicable anymore?
+#if 0
   if (ni >= Select->count()) {
     ni = Select->count() - 1;
   }
@@ -737,12 +868,148 @@ void KstPlotDialogI::changePosition(int ni) {
   int ci = Select->currentItem();
   KST::plotList.insert(ni, KST::plotList.take(ci));
   Select->setCurrentItem(ni);
+#endif
 }
 
 void KstPlotDialogI::updateButtons() {
-  CurvePlot->setEnabled(AvailableCurveList->selectedItem() != 0L);
-  CurveUnplot->setEnabled(DisplayedCurveList->selectedItem() != 0L);
+  bool bSelected;
+  uint count;
+  uint i;
+
+  bSelected = false;
+  count = AvailableCurveList->count( );
+  for (i=0; i<count; i++) {
+    if( AvailableCurveList->isSelected(i) ) {
+      bSelected = true;
+    }
+  }
+  if (bSelected && !CurvePlot->isEnabled()) {
+    CurvePlot->setEnabled(TRUE);
+  } else if (!bSelected && CurvePlot->isEnabled()) {
+    CurvePlot->setEnabled(FALSE);
+  }
+
+  bSelected = false;
+  count = DisplayedCurveList->count( );
+  for (i=0; i<count; i++) {
+    if( DisplayedCurveList->isSelected(i) ) {
+      bSelected = true;
+    }
+  }
+  if (bSelected && !CurveUnplot->isEnabled()) {
+    CurveUnplot->setEnabled(TRUE);
+  } else if (!bSelected && CurveUnplot->isEnabled()) {
+    CurveUnplot->setEnabled(FALSE);
+  }
 }
 
+void KstPlotDialogI::updateCurveLists() {
+  KstApp* app = KstApp::inst();
+  Kst2DPlotPtr plot;
+  KstBaseCurveList curves = kstObjectSubList<KstDataObject, KstBaseCurve>(KST::dataObjectList);
+  KstBaseCurveList::iterator it;
+
+  DisplayedCurveList->clear();
+  AvailableCurveList->clear();
+
+  KMdiChildView *c = app->findWindow(_window->currentText());
+  if (Select->count() && c) {
+    KstViewObjectPtr obj = static_cast<KstViewWindow*>(c)->view()->findChild(Select->currentText());
+    plot = dynamic_cast<Kst2DPlot*>(obj.data());
+
+    for (it = curves.begin(); it != curves.end(); it++) {
+      if (plot->Curves.find(*it) == plot->Curves.end()) {
+        AvailableCurveList->insertItem((*it)->tagName());
+      } else {
+        DisplayedCurveList->insertItem((*it)->tagName());
+      }
+    }
+  } else {
+    for (it = curves.begin(); it != curves.end(); it++) {
+      AvailableCurveList->insertItem((*it)->tagName());
+    }
+  }
+}
+
+void KstPlotDialogI::updateScalarCombo() {
+  int index;
+
+  ScalarList->clear();
+  KST::scalarList.lock().readLock();
+  for (index = 0; index < (int)KST::scalarList.count(); index++) {
+    ScalarList->insertItem(KST::scalarList[index]->tagLabel());
+  }
+  KST::scalarList.lock().readUnlock();
+}
+
+void KstPlotDialogI::updateWindowList() {
+  KstApp *app = KstApp::inst();
+  KMdiIterator<KMdiChildView*> *it = app->createIterator();
+  QString strOld;
+
+  if (_window->count()) {
+    strOld = _window->currentText();
+  }
+
+  _window->clear();
+
+  while (it->currentItem()) {
+    KstViewWindow *v = dynamic_cast<KstViewWindow*>(it->currentItem());
+    if (v) {
+      _window->insertItem(v->caption());
+      if (!strOld.isEmpty()) {
+        if (strOld.compare(v->caption()) == 0) {
+          _window->setCurrentItem(_window->count()-1);
+        }
+      }
+      if (!_strWindow.isEmpty()) {
+        if (_strWindow.compare(v->caption()) == 0) {
+          _window->setCurrentItem(_window->count()-1);
+          strOld = "";
+        }
+      }
+    }
+    it->next();
+  }
+
+  app->deleteIterator(it);
+
+  _strWindow = "";
+}
+
+void KstPlotDialogI::updatePlotList() {
+  KstApp *app = KstApp::inst();
+  KMdiChildView *c = app->findWindow(_window->currentText());
+  QString strOld;
+
+  if (Select->count()) {
+    strOld = Select->currentText();
+  }
+
+  Select->clear();
+
+  if (c) {
+    Kst2DPlotList plots = static_cast<KstViewWindow*>(c)->view()->findChildrenType<Kst2DPlot>(true);
+
+    for (Kst2DPlotList::Iterator i = plots.begin(); i != plots.end(); ++i) {
+      Select->insertItem((*i)->tagName());
+      if (!strOld.isEmpty()) {
+        if (strOld.compare((*i)->tagName()) == 0) {
+          Select->setCurrentItem(Select->count()-1);
+        }
+      }
+      if (!_strPlot.isEmpty()) {
+        if (_strPlot.compare((*i)->tagName()) == 0) {
+          Select->setCurrentItem(Select->count()-1);
+          strOld = "";
+        }
+      }
+    }
+  }
+
+  _strPlot = "";
+}
+
+
 #include "kstplotdialog_i.moc"
 // vim: et ts=2 sw=2
