@@ -74,7 +74,9 @@ KstPlugin::KstPlugin(QDomElement &e) : KstDataObject(e) {
       } else if (e.tagName() == "iscalar") {
         _inputScalarLoadQueue.append(qMakePair(e.attribute("name"), e.text()));
       } else if (e.tagName() == "ovector") {
-        _outputVectors.insert(e.attribute("name"), new KstVector(e.text()));
+        KstVectorPtr v = new KstVector(e.text());
+        _outputVectors.insert(e.attribute("name"), v);
+        KST::addVectorToList(v);
       } else if (e.tagName() == "oscalar") {
         _outputScalars.insert(e.attribute("name"), new KstScalar(e.text()));
       }
@@ -93,10 +95,13 @@ KstPlugin::KstPlugin(QDomElement &e) : KstDataObject(e) {
     for (QValueList<Plugin::Data::IOValue>::ConstIterator it = otable.begin();
                                                            it != otable.end();
                                                                          ++it) {
+      // FIXME: i18n?
       if ((*it)._type == Plugin::Data::IOValue::TableType) {
         _outArrayCnt++;
         if (!_outputVectors.contains((*it)._name)) {
-          _outputVectors.insert((*it)._name, new KstVector(tagName() + " vector - " + (*it)._name));
+          KstVectorPtr v = new KstVector(tagName() + " vector - " + (*it)._name);
+          _outputVectors.insert((*it)._name, v);
+          KST::addVectorToList(v);
         }
       } else if ((*it)._type == Plugin::Data::IOValue::FloatType) {
         _outScalarCnt++;
@@ -132,8 +137,14 @@ KstObject::UpdateType KstPlugin::update(int update_counter) {
     return NO_CHANGE;
   }
 
-  if (KstObject::checkUpdateCounter(update_counter))
+  if (KstObject::checkUpdateCounter(update_counter)) {
     return NO_CHANGE;
+  }
+
+  if (update_counter != -1) {
+    bool force = false;
+    // FIXME: we should only update if one of our dependencies was updated
+  }
 
   int *inArrayLens = 0L, *outArrayLens = 0L;
   double *inScalars = 0L, *outScalars = 0L;
@@ -213,21 +224,19 @@ KstObject::UpdateType KstPlugin::update(int update_counter) {
                        outVectors, outArrayLens, outScalars);
   }
 
-  if (rc != 0) {
-    // Error
-  }
-
-  itcnt = 0;
-  vitcnt = 0;
-  // Read back the output vectors and scalars
-  for (QValueList<Plugin::Data::IOValue>::ConstIterator it = otable.begin();
-                                                         it != otable.end();
-                                                                        ++it) {
-    if ((*it)._type == Plugin::Data::IOValue::TableType) {
-      vectorRealloced(_outputVectors[(*it)._name], outVectors[vitcnt], outArrayLens[vitcnt]);
-      vitcnt++;
-    } else if ((*it)._type == Plugin::Data::IOValue::FloatType) {
-      _outputScalars[(*it)._name]->setValue(outScalars[itcnt++]);
+  if (rc == 0) {
+    itcnt = 0;
+    vitcnt = 0;
+    // Read back the output vectors and scalars
+    for (QValueList<Plugin::Data::IOValue>::ConstIterator it = otable.begin();
+        it != otable.end();
+        ++it) {
+      if ((*it)._type == Plugin::Data::IOValue::TableType) {
+        vectorRealloced(_outputVectors[(*it)._name], outVectors[vitcnt], outArrayLens[vitcnt]);
+        vitcnt++;
+      } else if ((*it)._type == Plugin::Data::IOValue::FloatType) {
+        _outputScalars[(*it)._name]->setValue(outScalars[itcnt++]);
+      }
     }
   }
 
@@ -287,7 +296,7 @@ QString KstPlugin::propertyString() const {
 }
 
 
-bool KstPlugin::setPlugin(KSharedPtr<Plugin> plugin) {
+bool KstPlugin::setPlugin(KstSharedPtr<Plugin> plugin) {
   if (plugin == _plugin) {
     return true;
   }
@@ -319,7 +328,9 @@ bool KstPlugin::setPlugin(KSharedPtr<Plugin> plugin) {
                                                          it != otable.end();
                                                                         ++it) {
     if ((*it)._type == Plugin::Data::IOValue::TableType) {
-      _outputVectors.insert((*it)._name, new KstVector);
+      KstVectorPtr v = new KstVector;
+      KST::addVectorToList(v);
+      _outputVectors.insert((*it)._name, v);
       _outArrayCnt++;
     } else if ((*it)._type == Plugin::Data::IOValue::FloatType) {
       _outputScalars.insert((*it)._name, new KstScalar);
@@ -332,13 +343,13 @@ bool KstPlugin::setPlugin(KSharedPtr<Plugin> plugin) {
 }
 
 
-KSharedPtr<Plugin> KstPlugin::plugin() const {
+KstSharedPtr<Plugin> KstPlugin::plugin() const {
   return _plugin;
 }
 
 
 void KstPlugin::_showDialog() {
-  KstPluginDialogI::globalInstance()->show_I(tagName());
+  KstPluginDialogI::globalInstance()->show_Edit(tagName());
 }
 
 // vim: ts=2 sw=2 et
