@@ -43,14 +43,14 @@ public:
   virtual bool slaveVectorsUsed() const;
   bool isValid() const;
 
-  virtual bool setPlugin(KSharedPtr<Plugin> plugin);
-  KSharedPtr<Plugin> plugin() const;
+  virtual bool setPlugin(KstSharedPtr<Plugin> plugin);
+  KstSharedPtr<Plugin> plugin() const;
 
   virtual QString propertyString() const;
 
 protected:
   virtual void _showDialog();
-  KSharedPtr<Plugin> _plugin;
+  KstSharedPtr<Plugin> _plugin;
 
 private:
   void commonConstructor();
@@ -58,7 +58,7 @@ private:
   void *_localData;
 };
 
-typedef KSharedPtr<KstPlugin> KstPluginPtr;
+typedef KstSharedPtr<KstPlugin> KstPluginPtr;
 typedef KstObjectList<KstPluginPtr> KstPluginList;
 
 #endif
