@@ -51,13 +51,16 @@
 #include <assert.h>
 #include <stdlib.h>
 
+#define DIALOGTYPE KstPluginDialogI
+#define DTYPE "Plugin"
+#include "dataobjectdialog.h"
 
 KstPluginDialogI *KstPluginDialogI::_inst = 0L;
-static KStaticDeleter<KstPluginDialogI> _plInst;
+static KStaticDeleter<KstPluginDialogI> _plPluginInst;
 
 KstPluginDialogI *KstPluginDialogI::globalInstance() {
   if (!_inst) {
-    _inst = _plInst.setObject(new KstPluginDialogI);
+    _inst = _plPluginInst.setObject(new KstPluginDialogI);
   }
 return _inst;
 }
@@ -68,10 +71,7 @@ const QString newPluginString = i18n("<New_Plugin>");
 KstPluginDialogI::KstPluginDialogI(QWidget* parent, const char* name,
                                    bool modal, WFlags fl)
 : KstPluginDialog(parent, name, modal, fl) {
-  connect(Select, SIGNAL(activated(int)), this, SLOT(update(int)));
-  connect(New, SIGNAL(clicked()), this, SLOT(new_I()));
-  connect(Edit, SIGNAL(clicked()), this, SLOT(edit_I()));
-  connect(Delete, SIGNAL(clicked()), this, SLOT(delete_I()));
+  Init();
   connect(PluginCombo, SIGNAL(activated(int)), this, SLOT(pluginChanged(int)));
   connect(_pluginManager, SIGNAL(clicked()), this, SLOT(showPluginManager()));
 
@@ -81,22 +81,20 @@ KstPluginDialogI::KstPluginDialogI(QWidget* parent, const char* name,
 
 
 KstPluginDialogI::~KstPluginDialogI() {
+  DP = 0L;
 }
 
+KstPluginPtr KstPluginDialogI::_getPtr(const QString &tagin) {
+  KstPluginList c =
+    kstObjectSubList<KstDataObject, KstPlugin>(KST::dataObjectList);
 
-void KstPluginDialogI::show_I(const QString &field) {
-  int new_index = -1;
-
-  KstPluginList plugins = kstObjectSubList<KstDataObject, KstPlugin>(KST::dataObjectList);
-  new_index = plugins.findIndexTag(field);
-
-  updatePluginList();
-  update(new_index);
-  show();
-  raise();
+  if ( c.findTag(tagin) == c.end() ) {
+    return 0L;
+  } else {
+    return *c.findTag(tagin);
+  }
 }
 
-
 void KstPluginDialogI::updatePluginList() {
   PluginCollection *pc = PluginCollection::self();
   QString previous = _pluginList[PluginCombo->currentItem()];
@@ -112,7 +110,7 @@ void KstPluginDialogI::updatePluginList() {
                                                                      ++it) {
     if (it.data()._filter == false) {
       _pluginList += it.data()._name;
-      PluginCombo->insertItem(i18n("%1 (v%2) - %3").arg(it.data()._name)
+      PluginCombo->insertItem(i18n("%1 (v%2) - %3").arg(it.data()._readableName)
                               .arg(it.data()._version)
                               .arg(it.data()._description));
       if (it.data()._name == previous) {
@@ -131,73 +129,98 @@ void KstPluginDialogI::updatePluginList() {
 }
 
 
-void KstPluginDialogI::show_New() {
-  updatePluginList();
-  update(-2);
-  show();
-  raise();
-}
+void KstPluginDialogI::updateForm() {
+  KstSharedPtr<Plugin> pPtr;
 
-
-void KstPluginDialogI::update(int new_index) {
-  QString selectedVector;
-  QString selectedScalar;
-  int oldSelect = Select->currentItem();
-  int oldCount = Select->count();
-
-  KstPluginList pl = kstObjectSubList<KstDataObject, KstPlugin>(KST::dataObjectList);
-
-  Select->clear();
-
-  for (KstPluginList::iterator it = pl.begin(); it != pl.end(); ++it) {
-    Select->insertItem((*it)->tagName());
-  }
-
-  if (new_index == -2 || Select->count() == 0) {
-    Select->insertItem(i18n("P%1-%2").arg(Select->count()+1).arg(newPluginString));
-    Select->setCurrentItem(Select->count()-1);
-  } else if (new_index >= 0 && new_index < Select->count()) {
-    Select->setCurrentItem(new_index);
-  } else if (oldCount != Select->count()) {
-    Select->setCurrentItem(Select->count()-1);
-  } else {
-    Select->setCurrentItem(oldSelect < Select->count() ? oldSelect : Select->count()-1);
-  }
-
-  if (_pluginList.isEmpty()) {
-    updatePluginList();
-  }
-
-  KstPluginPtr plugin = pl[Select->currentItem()];
-  if (!plugin) {
-    return;
-  }
-
-  int oldPluginIndex = PluginCombo->currentItem();
-  int newPluginIndex = _pluginList.findIndex(plugin->plugin()->data()._name);
-  bool updateCombos = (newPluginIndex != oldPluginIndex);
-
-  if (updateCombos) {
-    PluginCombo->setCurrentItem(newPluginIndex);
-    pluginChanged(PluginCombo->currentItem());
-  }
-
-  KSharedPtr<Plugin> pPtr = PluginCollection::self()->plugin(_pluginList[PluginCombo->currentItem()]);
-  if( pPtr != 0 ) {
-    // Update input vector and scalar combos
+  pPtr = PluginCollection::self()->plugin(_pluginList[PluginCombo->currentItem()]);
+  if( pPtr != NULL ) {
     const QValueList<Plugin::Data::IOValue>& itable = pPtr->data()._inputs;
     for (QValueList<Plugin::Data::IOValue>::ConstIterator it = itable.begin(); it != itable.end(); ++it) {
+      if ((*it)._type == Plugin::Data::IOValue::TableType) { // vector
+        QObject *field = _frameWidget->child((*it)._name.latin1(), "VectorSelector");
+        assert(field);
+        if (field) {
+          VectorSelector *vs = static_cast<VectorSelector*>(field);
+          vs->update();
+        }
+      } else {
+        QObject *field = _frameWidget->child((*it)._name.latin1(), "ScalarSelector");
+        assert(field);
+        if (field) {
+          ScalarSelector *ss = static_cast<ScalarSelector*>(field);
+          ss->update();
+        }
+      }
+    }
+  }
+}
+
+void KstPluginDialogI::_fillFieldsForEdit() {
+  if ( DP == 0L ) return; // shouldn't be needed
+  _tagName->setText( DP->tagName() );
+
+  updatePluginList();
+
+  int i = _pluginList.findIndex(DP->plugin()->data()._name);
+  PluginCombo->setCurrentItem(i);
+  pluginChanged(PluginCombo->currentItem());
+
+  fillVectorScalarCombos(DP->plugin());
+
+  show();
+  fixupLayout();
+}
+
+void KstPluginDialogI::_fillFieldsForNew() {
+  updatePluginList();
+
+  KstPluginList plugins =
+    kstObjectSubList<KstDataObject, KstPlugin>(KST::dataObjectList);
+
+  /* set tag name */
+  QString new_label;
+  new_label.sprintf("PL%d-", signed(plugins.count())+1);
+  new_label += newPluginString;
+  _tagName->setText(new_label);
+
+  updatePluginList();
+
+  int i = 0;
+  PluginCombo->setCurrentItem(i);
+  pluginChanged(PluginCombo->currentItem());
+
+  KstSharedPtr<Plugin> pPtr = PluginCollection::self()->plugin(_pluginList[PluginCombo->currentItem()]);
+
+  fillVectorScalarCombos(pPtr);
+
+}
+
+void KstPluginDialogI::fillVectorScalarCombos(KstSharedPtr<Plugin> pPtr) {
+  QString selectedVector;
+  QString selectedScalar;
+  bool DPvalid = false;
+
+  if (DP != 0) {
+    if (DP.data() && DP->isValid()) {
+      DPvalid = true;
+    }
+  }
+
+
+  if( pPtr != 0 ) {
+    // Update input vector and scalar combos
+    const QValueList<Plugin::Data::IOValue>& itable = pPtr->data()._inputs;
+    for (QValueList<Plugin::Data::IOValue>::ConstIterator it = itable.begin();
+         it != itable.end(); ++it) {
       if ((*it)._type == Plugin::Data::IOValue::TableType) { // vector
         QObject *field = _frameWidget->child((*it)._name.latin1(), "VectorSelector");
         assert(field);
         if (field) {
           VectorSelector *vs = static_cast<VectorSelector*>(field);
           selectedVector = vs->selectedVector();
-          if (!updateCombos) {
-            vs->update();
-          }
-          if (plugin.data() && plugin->isValid()) {
-            vs->setSelection(plugin->inputVectors()[(*it)._name]->tagName());
+          vs->update();
+          if (DPvalid) {
+            vs->setSelection(DP->inputVectors()[(*it)._name]->tagName());
           } else {
             vs->setSelection(selectedVector);
           }
@@ -208,11 +231,9 @@ void KstPluginDialogI::update(int new_index) {
         if (field) {
           ScalarSelector *ss = static_cast<ScalarSelector*>(field);
           selectedScalar = ss->selectedScalar( );
-          if (!updateCombos) {
             ss->update();
-          }
-          if (plugin.data() && plugin->isValid()) {
-            ss->setSelection(plugin->inputScalars()[(*it)._name]->tagName());
+          if (DPvalid) {
+            ss->setSelection(DP->inputScalars()[(*it)._name]->tagName());
           } else {
             ss->setSelection(selectedScalar);
           }
@@ -221,8 +242,7 @@ void KstPluginDialogI::update(int new_index) {
     }
 
     // Update output vector and scalar lineedits
-    //    const QValueList<Plugin::Data::IOValue>& otable = plugin->plugin()->data()._outputs;
-    if (plugin.data() && plugin->isValid()) {
+    if (DPvalid) {
       const QValueList<Plugin::Data::IOValue>& otable = pPtr->data()._outputs;
       for (QValueList<Plugin::Data::IOValue>::ConstIterator it = otable.begin(); it != otable.end(); ++it) {
         QObject *field = _frameWidget->child((*it)._name.latin1(), "QLineEdit");
@@ -231,9 +251,9 @@ void KstPluginDialogI::update(int new_index) {
           QLineEdit *li = static_cast<QLineEdit*>(field);
           QString ts;
           if ((*it)._type == Plugin::Data::IOValue::TableType) { // vector
-            ts = plugin->outputVectors()[(*it)._name]->tagName();
+            ts = DP->outputVectors()[(*it)._name]->tagName();
           } else { // scalar
-            ts = plugin->outputScalars()[(*it)._name]->tagName();
+            ts = DP->outputScalars()[(*it)._name]->tagName();
           }
           li->setText(ts);
         }
@@ -250,8 +270,10 @@ void KstPluginDialogI::update(int new_index) {
       }
     }
   }
+}
 
-  QTimer::singleShot(0, this, SLOT(fixupLayout()));
+void KstPluginDialogI::update() {
+  updatePluginList();
 }
 
 
@@ -259,10 +281,13 @@ void KstPluginDialogI::fixupLayout() {
   _frameWidget->resize(_frameWidget->sizeHint());
   _pluginFrame->resize(_pluginFrame->sizeHint());
   resize(sizeHint());
+  updateGeometry();
 }
 
 
-bool KstPluginDialogI::saveInputs(KstPluginPtr plugin, KSharedPtr<Plugin> p) {
+bool KstPluginDialogI::saveInputs(KstPluginPtr plugin, KstSharedPtr<Plugin> p) {
+  KST::vectorList.lock().readLock();
+  KST::scalarList.lock().readLock();
   const QValueList<Plugin::Data::IOValue>& itable = p->data()._inputs;
   for (QValueList<Plugin::Data::IOValue>::ConstIterator it = itable.begin(); it != itable.end(); ++it) {
     if ((*it)._type == Plugin::Data::IOValue::TableType) {
@@ -276,14 +301,28 @@ bool KstPluginDialogI::saveInputs(KstPluginPtr plugin, KSharedPtr<Plugin> p) {
       assert(field);
       ScalarSelector *ss = static_cast<ScalarSelector*>(field);
       KstScalarPtr s = *KST::scalarList.findTag(ss->selectedScalar());
-      plugin->inputScalars().insert((*it)._name, s);
+      if (s == *KST::scalarList.end()) {
+        bool ok;
+        double val = ss->_scalar->currentText().toDouble(&ok);
+
+        if (ok) {
+          KstScalarPtr pSNew = new KstScalar(ss->_scalar->currentText(), val, true, false, false);
+          plugin->inputScalars().insert((*it)._name, pSNew);
+        } else {
+          plugin->inputScalars().insert((*it)._name, s);
+        }
+      } else {
+        plugin->inputScalars().insert((*it)._name, s);
+      }
     }
   }
+  KST::scalarList.lock().readUnlock();
+  KST::vectorList.lock().readUnlock();
 return true;
 }
 
 
-bool KstPluginDialogI::saveOutputs(KstPluginPtr plugin, KSharedPtr<Plugin> p) {
+bool KstPluginDialogI::saveOutputs(KstPluginPtr plugin, KstSharedPtr<Plugin> p) {
   const QValueList<Plugin::Data::IOValue>& otable = p->data()._outputs;
 
   for (QValueList<Plugin::Data::IOValue>::ConstIterator it = otable.begin(); it != otable.end(); ++it) {
@@ -293,7 +332,7 @@ bool KstPluginDialogI::saveOutputs(KstPluginPtr plugin, KSharedPtr<Plugin> p) {
     QLineEdit *li = static_cast<QLineEdit*>(field);
 
     if (li->text().isEmpty()) {
-      QString tagName = Select->currentText();
+      QString tagName = _tagName->text();
       if (tagName.endsWith(newPluginString)) {
         tagName = plugin->tagName();
       }
@@ -305,43 +344,46 @@ bool KstPluginDialogI::saveOutputs(KstPluginPtr plugin, KSharedPtr<Plugin> p) {
       if (!KST::vectorTagNameNotUnique(nt, false)) {
         plugin->outputVectors()[(*it)._name]->setTagName(nt);
       } else if (plugin->outputVectors()[(*it)._name]->tagName() != nt) {
-          while (KST::vectorTagNameNotUnique(nt, false)) {
-            nt += "'";
-          }
-          plugin->outputVectors()[(*it)._name]->setTagName(nt);
+        while (KST::vectorTagNameNotUnique(nt, false)) {
+          nt += "'";
+        }
+        plugin->outputVectors()[(*it)._name]->setTagName(nt);
       }
     } else if ((*it)._type == Plugin::Data::IOValue::FloatType) {
       if (!KST::vectorTagNameNotUnique(nt, false)) {
         plugin->outputScalars()[(*it)._name]->setTagName(nt);
       } else if (plugin->outputScalars()[(*it)._name]->tagName() != nt) {
-          while (KST::vectorTagNameNotUnique(nt, false)) {
-            nt += "'";
-          }
-          plugin->outputScalars()[(*it)._name]->setTagName(nt);
+        while (KST::vectorTagNameNotUnique(nt, false)) {
+          nt += "'";
+        }
+        plugin->outputScalars()[(*it)._name]->setTagName(nt);
       }
     }
   }
 
-return true;
+  return true;
 }
 
 
-void KstPluginDialogI::new_I() {
+bool KstPluginDialogI::new_I() {
   KstPluginPtr plugin;
-  QString tagName = Select->currentText();
+  QString tagName = _tagName->text();
 
-  if (KST::dataTagNameNotUnique(tagName, true, this)) {
-    Select->setFocus();
-    return;
+  if (!tagName.endsWith(newPluginString)) {
+    if (KST::dataTagNameNotUnique(tagName, true, this)) {
+      _tagName->setFocus();
+      return false;
+    }
   }
 
   int pitem = PluginCombo->currentItem();
   if (pitem >= 0 && PluginCombo->count() > 0) {
-    KSharedPtr<Plugin> pPtr = PluginCollection::self()->plugin(_pluginList[pitem]);
+    KstSharedPtr<Plugin> pPtr = PluginCollection::self()->plugin(_pluginList[pitem]);
     if (pPtr != 0L) {
       plugin = new KstPlugin;
       if (!saveInputs(plugin, pPtr)) {
-        return;
+        plugin = 0L;
+        return false;
       }
 
       plugin->setPlugin(pPtr);
@@ -356,112 +398,85 @@ void KstPluginDialogI::new_I() {
       }
       plugin->setTagName(tagName);
       if (!saveOutputs(plugin, pPtr)) {
-        return;
+        plugin = 0L;
+        return false;
       }
     }
   }
 
   if (!plugin || !plugin->isValid()) {
-    KMessageBox::sorry(this, i18n("There is an error in the plugin you entered. Please fix it."));
-    return;
+    KMessageBox::sorry(this, i18n("There is an error in the plugin you entered. "
+                                  "Please fix it."));
+    return false;
   }
 
+  plugin->update(-1);
+  KST::dataObjectList.lock().writeLock();
   KST::dataObjectList.append(plugin.data());
+  KST::dataObjectList.lock().writeUnlock();
   plugin = 0L;
   emit modified();
+
+  return true;
 }
 
 
-void KstPluginDialogI::edit_I() {
-  int index = Select->currentItem();
+bool KstPluginDialogI::edit_I() {
 
-  if (Select->count() == 0 || index < 0) {
-    new_I();
-    return;
-  }
-
-  QString tagName = Select->currentText();
-  KstPluginList pl = kstObjectSubList<KstDataObject, KstPlugin>(KST::dataObjectList);
-  KstPluginPtr plugin = pl[index];
-
-  if (!plugin.data()) {
-    new_I();
-    return;
-  }
-
-  int pitem = PluginCombo->currentItem();
-
-  if (tagName.endsWith(newPluginString)) {
-    QString tmpTagName;
-    int i = 0;
-    do {
-      tmpTagName = i18n("P%1-%2").arg(++i).arg(_pluginList[pitem]);
-    } while (KST::dataTagNameNotUnique(tmpTagName, false));
-    tagName = tmpTagName;
-  }
-
-  //kdDebug() << "Apply as edit [" << tagName << "]" << endl;
-
-  if (tagName != plugin->tagName()) {
-    if (KST::dataTagNameNotUnique(tagName, true, this)) {
-      Select->setFocus();
-      return;
+  if (_tagName->text() != DP->tagName()) {
+    if (KST::dataTagNameNotUnique(_tagName->text())) {
+      _tagName->setFocus();
+      return false;
     }
   }
 
-  plugin->setTagName(tagName);
+  DP->setTagName(_tagName->text());
 
-  plugin->inputVectors().clear();
-  plugin->inputScalars().clear();
+  if (!DP.data()) { // something is dreadfully wrong - this should never happen
+    return false;
+  }
 
-  KSharedPtr<Plugin> pPtr = PluginCollection::self()->plugin(_pluginList[pitem]);
+  int pitem = PluginCombo->currentItem();
+  KstSharedPtr<Plugin> pPtr = PluginCollection::self()->plugin(_pluginList[pitem]);
+
+  DP->inputVectors().clear();
+  DP->inputScalars().clear();
 
   // Save the vectors and scalars
-  if (!saveInputs(plugin, pPtr)) {
-    return;
+  if (!saveInputs(DP, pPtr)) {
+    KMessageBox::sorry(this, i18n("There is an error in the plugin you "
+                                  "entered. Please fix it."));
+    return false;
   }
 
   if (pitem >= 0 && PluginCombo->count() > 0) {
-    plugin->setPlugin(pPtr);
+    DP->setPlugin(pPtr);
   }
 
-  if (!saveOutputs(plugin, pPtr)) {
-    return;
+  if (!saveOutputs(DP, pPtr)) {
+    KMessageBox::sorry(this, i18n("There is an error in the plugin you "
+                                  "entered. Please fix it."));
+    return false;
   }
 
-  if (!plugin->isValid()) {
+  if (!DP->isValid()) {
     KMessageBox::sorry(this, i18n("There is an error in the plugin you entered. Please fix it."));
-    return;
+    return false;
   }
-
-  plugin = 0L; // release
-  pl.clear(); // release
+  DP->update(-1);
   emit modified();
+  return true;
 }
 
-
-void KstPluginDialogI::delete_I() {
-  QString tag = Select->currentText();
-
-  if (tag.isEmpty()) {
-    KMessageBox::sorry(this, i18n("You need to select an active plugin to delete."));
-  } else {
-    KST::dataObjectList.removeTag(tag);
-  }
-
-  emit modified();
-}
-
-
 void KstPluginDialogI::generateEntries(bool input, int& cnt, QWidget *parent, QGridLayout *grid, const QValueList<Plugin::Data::IOValue>& table) {
 QString scalarLabelTemplate, vectorLabelTemplate;
 
   if (input) {
-    scalarLabelTemplate = i18n("Input Scalar - %1:");
-    vectorLabelTemplate = i18n("Input Vector - %1:");
+    scalarLabelTemplate = i18n("Input scalar - %1:");
+    vectorLabelTemplate = i18n("Input vector - %1:");
   } else {
-    scalarLabelTemplate = i18n("Output Scalar - %1:");
-    vectorLabelTemplate = i18n("Output Vector - %1:");
+    scalarLabelTemplate = i18n("Output scalar - %1:");
+    vectorLabelTemplate = i18n("Output vector - %1:");
   }
 
   for (QValueList<Plugin::Data::IOValue>::ConstIterator it = table.begin(); it != table.end(); ++it) {
@@ -496,6 +511,7 @@ QString scalarLabelTemplate, vectorLabelTemplate;
         connect(w->_scalar, SIGNAL(activated(const QString&)), this, SLOT(updateScalarTooltip(const QString&)));
         connect(widget, SIGNAL(newScalarCreated()), this, SIGNAL(modified()));
         KstScalarPtr p = *KST::scalarList.findTag(w->_scalar->currentText());
+        w->allowDirectEntry( true );
         if (p) {
           QToolTip::add(w->_scalar, QString::number(p->value()));
         }
@@ -539,15 +555,16 @@ void KstPluginDialogI::pluginChanged(int idx) {
   if (idx >= 0 && PluginCombo->count() > 0) {
     const QString& pluginName = _pluginList[idx];
     const Plugin::Data& pluginData = PluginCollection::self()->pluginList()[PluginCollection::self()->pluginNameList()[pluginName]];
+    int cnt = 3;
 
     // Setup the grid and the "static" entries
     int variables = pluginData._inputs.count() + pluginData._outputs.count();
     QGridLayout *grid = new QGridLayout(_frameWidget, 4+variables, 2);
     grid->setMargin(6);
     grid->setSpacing(5);
-    grid->addWidget(new QLabel("Plugin Name:", _frameWidget), 0, 0);
+    grid->addWidget(new QLabel(i18n("Plugin name:"), _frameWidget), 0, 0);
     grid->addWidget(new QLabel(pluginData._name, _frameWidget), 0, 1);
-    grid->addWidget(new QLabel("Description:", _frameWidget), 1, 0);
+    grid->addWidget(new QLabel(i18n("Description:"), _frameWidget), 1, 0);
     QLabel *lab = new QLabel(pluginData._description, _frameWidget);
     lab->setAlignment(lab->alignment() | Qt::WordBreak);
     grid->addWidget(lab, 1, 1);
@@ -558,8 +575,6 @@ void KstPluginDialogI::pluginChanged(int idx) {
     line->setFrameShape(QFrame::HLine);
     grid->addMultiCellWidget(line, 2, 2, 0, 1);
 
-    int cnt = 3;
-
     // Generate the input values
     generateEntries(true, cnt, _frameWidget, grid, pluginData._inputs);
 
@@ -569,7 +584,7 @@ void KstPluginDialogI::pluginChanged(int idx) {
       line->setFrameShape(QFrame::HLine);
       line->setFrameShadow(QFrame::Sunken);
       line->setFrameShape(QFrame::HLine);
-      grid->addWidget(line, cnt, 0);
+      grid->addMultiCellWidget(line, cnt, cnt, 0, 1);
       cnt++;
     }
 
