@@ -30,7 +30,8 @@
 class KstPSDCurve: public KstBaseCurve {
 public:
   KstPSDCurve(const QString &in_tag, KstVectorPtr in_V, double freq,
-              int len,
+              bool average, int len,
+              bool in_apodize, bool in_removeMean,
               const QString &VUnits, const QString &RUnits,
               const QColor &in_color);
   KstPSDCurve(QDomElement &e);
@@ -42,6 +43,15 @@ public:
   virtual void save(QTextStream &ts);
   virtual QString propertyString() const;
 
+  bool getApodize() const;
+  void setApodize(bool in_apodize);
+
+  bool getRemoveMean() const;
+  void setRemoveMean(bool in_removeMean);
+
+  bool getAverage() const;
+  void setAverage(bool in_average);
+
   double getFreq() const;
   void setFreq(double in_freq);
 
@@ -61,12 +71,6 @@ public:
 
   virtual bool slaveVectorsUsed() const;
 
-  bool appodize()                         const { return Appodize; }
-  bool removeMean()                       const { return RemoveMean; }
-  void setAppodize(bool in_appodize)      { Appodize = in_appodize; }
-  void setRemoveMean(bool in_remove_mean) { RemoveMean = in_remove_mean; }
-
-protected:
   virtual void _showDialog();
 
 private:
@@ -87,17 +91,20 @@ private:
   void GenW();
 
   void commonConstructor(const QString &in_tag, KstVectorPtr in_V,
-                         double freq, int len,
+                         double freq, bool average, int len,
+                         bool apodize, bool removeMean,
                          const QString &VUnits, const QString &RUnits,
                          const QColor &in_color);
 
-  bool Appodize;
+  void _adjustLengths();
+  bool Apodize;
   bool RemoveMean;
+  bool Average;
 
   KstVectorMap::Iterator _sVector, _fVector;
 };
 
-typedef KSharedPtr<KstPSDCurve> KstPSDCurvePtr;
+typedef KstSharedPtr<KstPSDCurve> KstPSDCurvePtr;
 typedef KstObjectList<KstPSDCurvePtr> KstPSDCurveList;
 
 #endif
