@@ -20,9 +20,13 @@
 #include <stdio.h>
 #include <stdlib.h>
 
+#include <qmessagebox.h>
+
 #include <kdebug.h>
 #include <klocale.h>
 
+#include "rwlock.h"
+#include "enodes.h"
 #include "kstdatacollection.h"
 #include "kstdebug.h"
 #include "kstrvector.h"
@@ -56,11 +60,13 @@ KstRVector::KstRVector(QDomElement &e, const QString &o_file,
       if (e.tagName() == "tag") {
         setTagName(e.text());
       } else if (e.tagName() == "filename") {
+        KST::dataSourceList.lock().readLock();
         if (o_file == "|") {
           in_file = *KST::dataSourceList.findFileName(e.text());
         } else {
           in_file = *KST::dataSourceList.findFileName(o_file);
         }
+        KST::dataSourceList.lock().readUnlock();
       } else if (e.tagName() == "field") {
         in_field = e.text();
       } else if (e.tagName() == "start") {
@@ -144,6 +150,7 @@ void KstRVector::commonRVConstructor(KstDataSourcePtr in_file,
   update(-1);
 }
 
+
 void KstRVector::change(KstDataSourcePtr in_file, const QString &in_field,
                         const QString &in_tag,
                         int in_f0, int in_n,
@@ -202,12 +209,13 @@ void KstRVector::changeFrames(int in_f0, int in_n,
   }
 }
 
-KstRVector::~KstRVector() {
+KstRVector::~KstRVector() {  
   _file = 0;
 
   if (AveReadBuf != NULL) {
     free(AveReadBuf);
-    AveReadBuf = NULL;
+
+   AveReadBuf = NULL;
   }
 }
 
@@ -344,6 +352,7 @@ KstObject::UpdateType KstRVector::update(int update_counter) {
 }
 
 KstObject::UpdateType KstRVector::doUpdate(bool force) {
+  KstObject::UpdateType  update;
   int i, j, k, shift, n_read=0;
   int ave_nread;
   int new_f0, new_nf, tmp_fn;
@@ -486,7 +495,9 @@ KstObject::UpdateType KstRVector::doUpdate(bool force) {
     NumShifted = _size;
   }
 
-  return KstVector::update(-1);
+  update = KstVector::update(-1);
+  
+  return update;
 }
 
 
@@ -512,8 +523,10 @@ void KstRVector::reload() {
   // FIXME: inefficient
   KstDataSourcePtr newsrc = KstDataSource::loadSource(_file->fileName(), _file->fileType());
   if (newsrc) {
+    KST::dataSourceList.lock().writeLock();
     KST::dataSourceList.remove(_file);
     KST::dataSourceList.append(newsrc);
+    KST::dataSourceList.lock().writeUnlock();
     _file = newsrc;
     reset();
   }
