@@ -24,6 +24,10 @@
 
 #include "kstdatasource.h"
 
+namespace Equation {
+  class Node;
+}
+
 /**A class for handling data vectors for kst.
  *@author cbn
  */
@@ -41,7 +45,7 @@ public:
              int o_s = -1, bool o_ave = false);
 
   virtual ~KstRVector();
-
+  
   /** change the properties of a kstrvector */
   void change(KstDataSourcePtr file, const QString &field, const QString &tag,
               int f0, int n, int skip,
@@ -153,7 +157,7 @@ private:
   void checkIntegrity();
 };
 
-typedef KSharedPtr<KstRVector> KstRVectorPtr;
+typedef KstSharedPtr<KstRVector> KstRVectorPtr;
 typedef KstObjectList<KstRVectorPtr> KstRVectorList;
 
 #endif
