@@ -28,7 +28,7 @@ class KstScalar : public KstObject {
   Q_OBJECT
   Q_PROPERTY(bool orphan READ orphan WRITE setOrphan)
   public:
-    KstScalar(const QString& in_tag = QString::null, double val = 0.0);
+    KstScalar(const QString& in_tag = QString::null, double val = 0.0, bool orphan = false, bool displayable = true, bool bLock = true);
     KstScalar(QDomElement& e);
     virtual ~KstScalar();
 
@@ -53,18 +53,21 @@ class KstScalar : public KstObject {
     void setValue(double inV);
 
     bool orphan() const { return _orphan; }
-
     void setOrphan(bool orphan) { _orphan = orphan; }
     
+    bool displayable() const { return _displayable; }
+    void setDisplayable(bool displayable) { _displayable = displayable; }
+
   signals:
     void trigger();
 
   private:
     double _value;
     bool _orphan;
+    bool _displayable;
 };
 
-typedef KSharedPtr<KstScalar> KstScalarPtr;
+typedef KstSharedPtr<KstScalar> KstScalarPtr;
 typedef KstObjectList<KstScalarPtr> KstScalarList;
 typedef KstObjectMap<KstScalarPtr> KstScalarMap;
 
