@@ -14,6 +14,8 @@
  *   (at your option) any later version.                                   *
  *                                                                         *
  ***************************************************************************/
+
+#include <qmessagebox.h>
 #include <klocale.h>
 #include <kdebug.h>
 
@@ -24,17 +26,17 @@
 #include "kstdoc.h"
 #include "kstvcurve.h"
 
-static const QString XVECTOR = "X";
-static const QString YVECTOR = "Y";
+static const QString COLOR_XVECTOR = "X";
+static const QString COLOR_YVECTOR = "Y";
 static const QString EXVECTOR = "EX";
 static const QString EYVECTOR = "EY";
 
-KstVCurve::KstVCurve(const QString &in_tag, KstVectorPtr in_X, KstVectorPtr in_Y,
-		   KstVectorPtr in_EX, KstVectorPtr in_EY,
-                   const QColor &in_color)
+KstVCurve::KstVCurve(const QString &in_tag, KstVectorPtr in_X, KstVectorPtr in_Y, KstVectorPtr in_EX, KstVectorPtr in_EY, const QColor &in_color)
 : KstBaseCurve() {
   setHasPoints(false);
   setHasLines(true);
+  setLineWidth(0);
+  setLineStyle(0);
 
   commonConstructor(in_tag, in_color);
   VX = in_X;
@@ -44,7 +46,6 @@ KstVCurve::KstVCurve(const QString &in_tag, KstVectorPtr in_X, KstVectorPtr in_Y
   update();
 }
 
-
 KstVCurve::KstVCurve(QDomElement &e)
 : KstBaseCurve(e) {
   QString in_tag, xname, yname, exname, eyname;
@@ -73,16 +74,21 @@ KstVCurve::KstVCurve(QDomElement &e)
       } else if (e.tagName() == "hasLines") {
         HasLines = (e.text() != "0");
       } else if (e.tagName() == "hasPoints") {
-	HasPoints = (e.text() != "0");
+        HasPoints = (e.text() != "0");
       } else if (e.tagName() == "pointType") {
-	Point.setType(e.text().toInt());
+        Point.setType(e.text().toInt());
+      } else if (e.tagName() == "lineWidth") {
+        LineWidth = e.text().toInt();
+      } else if (e.tagName() == "lineStyle") {
+        LineStyle = e.text().toInt();
       }
+
     }
     n = n.nextSibling();
   }
 
-  _inputVectorLoadQueue.append(qMakePair(XVECTOR, xname));
-  _inputVectorLoadQueue.append(qMakePair(YVECTOR, yname));
+  _inputVectorLoadQueue.append(qMakePair(COLOR_XVECTOR, xname));
+  _inputVectorLoadQueue.append(qMakePair(COLOR_YVECTOR, yname));
   _inputVectorLoadQueue.append(qMakePair(EXVECTOR, exname));
   _inputVectorLoadQueue.append(qMakePair(EYVECTOR, eyname));
   commonConstructor(in_tag, in_color);
@@ -95,19 +101,26 @@ void KstVCurve::commonConstructor(const QString &in_tag, const QColor &in_color)
   setTagName(in_tag);
 }
 
+KstVCurve::~KstVCurve() {
+}
 
-void KstVCurve::loadInputs() {
+bool KstVCurve::loadInputs() {
   QValueList<QPair<QString,QString> >::Iterator i;
+  KST::vectorList.lock().readLock();
   for (i = _inputVectorLoadQueue.begin(); i != _inputVectorLoadQueue.end(); ++i) {
-    if ((*i).first == XVECTOR) {
+    if ((*i).first == COLOR_XVECTOR) {
       VX = *KST::vectorList.findTag((*i).second);
       if (!VX.data()) {
-	      KstDebug::self()->log(i18n("Unable to find X vector for %2: [%1]").arg((*i).second).arg(tagName()), KstDebug::Warning);
+        KstDebug::self()->log(i18n("Unable to find X vector for %2: [%1]").arg((*i).second).arg(tagName()), KstDebug::Warning);
+        KST::vectorList.lock().readUnlock();
+        return false;
       }
-    } else if ((*i).first == YVECTOR) {
+    } else if ((*i).first == COLOR_YVECTOR) {
       VY = *KST::vectorList.findTag((*i).second);
       if (!VY.data()) {
-	      KstDebug::self()->log(i18n("Unable to find Y vector for %2: [%1]").arg((*i).second).arg(tagName()), KstDebug::Warning);
+        KstDebug::self()->log(i18n("Unable to find Y vector for %2: [%1]").arg((*i).second).arg(tagName()), KstDebug::Warning);
+        KST::vectorList.lock().readUnlock();
+        return false;
       }
     } else if ((*i).first == EXVECTOR) {
       EX = *KST::vectorList.findTag((*i).second);
@@ -115,17 +128,31 @@ void KstVCurve::loadInputs() {
       EY = *KST::vectorList.findTag((*i).second);
     }
   }
+  KST::vectorList.lock().readUnlock();
 
-  update();
-}
+  VX->readLock();
+  VY->readLock();
+  if (EX) {
+    EX->readLock();
+  }
+  if (EY) {
+    EY->readLock();
+  }
+  update(); // FIXME: this update happens in the wrong thread
+  VX->readUnlock();
+  VY->readUnlock();
+  if (EX) {
+    EX->readUnlock();
+  }
+  if (EY) {
+    EY->readUnlock();
+  }
 
-
-KstVCurve::~KstVCurve() {
+  return true;
 }
 
 KstObject::UpdateType KstVCurve::update(int update_counter) {
-
-  if (KstObject::checkUpdateCounter(update_counter))
+  if (!VX || !VY || KstObject::checkUpdateCounter(update_counter))
     return NO_CHANGE;
 
   VX->update(update_counter);
@@ -188,11 +215,19 @@ void KstVCurve::getEYPoint(int i, double &x, double &y, double &ey) {
 }
 
 QString KstVCurve::getXVTag() const{
-  return VX->tagName();
+  if (!VX.data()) {
+    return QString::null;
+  } else {
+    return VX->tagName();
+  }
 }
 
 QString KstVCurve::getYVTag() const{
-  return VY->tagName();
+  if (!VY.data()) {
+    return QString::null;
+  } else {
+    return VY->tagName();
+  }
 }
 
 QString KstVCurve::getXETag() const{
@@ -235,6 +270,8 @@ void KstVCurve::save(QTextStream &ts) {
   ts << "  <hasLines>" << HasLines << "</hasLines>" << endl;
   ts << "  <hasPoints>" << HasPoints << "</hasPoints>" << endl;
   ts << "  <pointType>" << Point.getType() << "</pointType>" << endl;
+  ts << "  <lineWidth>" << LineWidth << "</lineWidth>" << endl;
+  ts << "  <lineStyle>" << LineStyle << "</lineStyle>" << endl;
   ts << " </curve>" << endl;
 }
 
@@ -275,7 +312,7 @@ QString KstVCurve::propertyString() const {
 }
 
 void KstVCurve::_showDialog() {
-  KstCurveDialogI::globalInstance()->show_I(tagName());
+  KstCurveDialogI::globalInstance()->show_Edit(tagName());
 }
 
 int KstVCurve::samplesPerFrame() const {
@@ -283,3 +320,4 @@ int KstVCurve::samplesPerFrame() const {
   return rvp ? rvp->samplesPerFrame() : 1;
 }
 
+// vim: ts=2 sw=2 et
