@@ -66,7 +66,7 @@ public:
 
   virtual KstCurveType type() const;
 
-  virtual void loadInputs();
+  virtual bool loadInputs();
 
   virtual bool xIsRising() const {return VX->isRising(); }
 
@@ -77,7 +77,6 @@ public:
 
   virtual int samplesPerFrame() const;
 
-protected:
   virtual void _showDialog();
 
 private:
@@ -95,7 +94,7 @@ private:
   double _ns_miny;
 };
 
-typedef KSharedPtr<KstVCurve> KstVCurvePtr;
+typedef KstSharedPtr<KstVCurve> KstVCurvePtr;
 typedef KstObjectList<KstVCurvePtr> KstVCurveList;
 
 #endif
