@@ -24,37 +24,46 @@
 
 class KstVectorDialogI : public KstVectorDialog {
   Q_OBJECT
-public:
+ public:
   KstVectorDialogI(QWidget* parent = 0, const char* name = 0,
                    bool modal = FALSE, WFlags fl = 0 );
   virtual ~KstVectorDialogI();
 
-public slots:
-  /** update the entries in the vector dialog to represent current vectors */
-  void update(int new_index = -1);
+ public slots:
+  /** update no longer does anything */
+  void update();
 
-  void browseFile();
-
-  /** Calls update(), then shows/raises the dialog */
-  void show_I();
-  void show_I(const QString &field);
-  void show_New();
-
-  void new_I();
-  void edit_I();
-  void delete_I();
+  bool new_I();
+  bool edit_I();
 
   static KstVectorDialogI *globalInstance();
 
   void updateCompletion();
-
-signals:
+ signals:
   void modified();
   void vectorCreated(KstVectorPtr v);
 
-private:
+ private:
   static KstVectorDialogI *_inst;
   KCompletion *_fieldCompletion;
+  KstRVectorPtr _getPtr(const QString &tagin);
+
+  KstRVectorPtr DP;
+  bool _newDialog;
+
+ /***********************************/
+ /** defined in dataobjectdialog.h **/
+ public slots:
+  void show_Edit(const QString &field);
+  void show_New();
+  void OK();
+  void Init();
+  void close();
+  void reject();
+ private:
+  void _fillFieldsForEdit();
+  void _fillFieldsForNew();
+
 };
 
 #endif
