@@ -3,7 +3,6 @@
                              -------------------
     begin                :
     copyright            : (C) 2004 The University of British Columbia
-                           (C) 2004 Andrew R. Walker
     email                :
  ***************************************************************************/
 
@@ -37,35 +36,33 @@ KstViewScalarsDialogI::KstViewScalarsDialogI(QWidget* parent,
                                            WFlags fl) :
   KstViewScalarsDialog(parent, name, modal, fl) {
    
+
+  QBoxLayout* pLayout;
+  
   tableScalars = new KstScalarTable( this, "tableScalars" );
   tableScalars->setNumRows( 0 );
   tableScalars->setNumCols( 2 );
   tableScalars->setReadOnly( TRUE );
   tableScalars->setSorting( FALSE );
   tableScalars->setSelectionMode( QTable::Single );
-  KstViewScalarsDialogLayout->addWidget( tableScalars );
-    
-  layout2 = new QHBoxLayout( 0, 0, 6, "layout2"); 
-  QSpacerItem* spacer = new QSpacerItem( 440, 20, QSizePolicy::Expanding, QSizePolicy::Minimum );
-  layout2->addItem( spacer );
-  
-  Cancel = new QPushButton( this, "Cancel" );
-  layout2->addWidget( Cancel );
-  KstViewScalarsDialogLayout->addLayout( layout2 );
-  
-  connect( Cancel, SIGNAL( clicked() ), this, SLOT( close() ) );
-  
-  if( tableScalars->numCols( ) != 2 ) {  
-    int i;
-  
-    for( i=0; i<tableScalars->numCols( );  ) {
-      tableScalars->removeColumn( 0 );
+  pLayout = dynamic_cast<QBoxLayout*>(layout());
+  if (pLayout) {
+    pLayout->insertWidget( 0, tableScalars );
+
+    if( tableScalars->numCols( ) != 2 ) {
+      int i;
+
+      for( i=0; i<tableScalars->numCols( );  ) {
+        tableScalars->removeColumn( 0 );
+      }
+      tableScalars->insertColumns( 0, 2 );
     }
-    tableScalars->insertColumns( 0, 2 );
+
+    tableScalars->setReadOnly( true );
+    languageChange( );
   }
   
-  tableScalars->setReadOnly( true );
-  languageChange( );
+  connect( Cancel, SIGNAL( clicked() ), this, SLOT( close() ) );
 }
 
 KstViewScalarsDialogI::~KstViewScalarsDialogI() {
@@ -96,9 +93,10 @@ void KstViewScalarsDialogI::showViewScalarsDialog() {
 void KstViewScalarsDialogI::languageChange()
 {
   setCaption( i18n( "View Scalar Values" ) );
-  Cancel->setText( tr("Close") );
-  tableScalars->horizontalHeader()->setLabel( 0, tr("Scalar") );
-  tableScalars->horizontalHeader()->setLabel( 1, tr("Value") );    
+  tableScalars->horizontalHeader()->setLabel( 0, i18n("Scalar") );
+  tableScalars->horizontalHeader()->setLabel( 1, i18n("Value") );
+
+  KstViewScalarsDialog::languageChange();      
 }
 
 void KstViewScalarsDialogI::updateDefaults(int index) {
