@@ -3,7 +3,6 @@
                              -------------------
     begin                :
     copyright            : (C) 2004 The University of British Columbia
-                           (C) 2004 Andrew R. Walker
     email                :
  ***************************************************************************/
 
@@ -30,10 +29,6 @@ public:
                          bool modal = FALSE, WFlags fl = 0 );
   virtual ~KstViewScalarsDialogI();
   KstScalarTable* tableScalars;
-  QPushButton* Cancel;
-
-protected:
-  QHBoxLayout* layout2;
 
 protected slots:
   virtual void languageChange();
