@@ -16,24 +16,33 @@
  ***************************************************************************/
 
 #include <stdlib.h>
+#include <stdio.h>
+#include <iostream>
+
 #include <kdebug.h>
 #include <kcmdlineargs.h>
 #include <kaboutdata.h>
 #include <kimageio.h>
 #include <klocale.h>
-#include <stdio.h>
-#include <iostream>
+#include <kmdimainfrm.h>
 
 #include "kst.h"
-#include "plugincollection.h"
-#include "kstvcurve.h"
-#include "kstpsdcurve.h"
-#include "kstequationcurve.h"
-#include "ksthistogram.h"
-#include "kstplotlist.h"
+#include "kst2dplot.h"
 #include "kstcolorsequence.h"
 #include "kstdatacollection.h"
+#include "kstequationcurve.h"
+#include "ksthistogram.h"
+#include "kstlegend.h"
+#include "kstpsdcurve.h"
+#include "kstvcurve.h"
 #include "kstversion.h"
+#include "ksttoplevelview.h"
+#include "plugincollection.h"
+
+
+// George is going to freak out if main() gets more complicated. :-)
+// Let's try to come up with a KstCommandLineProcessor class that can
+// properly and cleanly deal with all of this.
 
 static const char description[] = I18N_NOOP("Kst: a data viewing program.");
 
@@ -162,22 +171,25 @@ void SetCMDOptions(KCmdLineArgs *args, struct InType &in,
 /*        create the plots                                      */
 /*                                                              */
 /****************************************************************/
-void CreatePlots(struct InType &in) {
-  KstPlot *plot;
+void CreatePlots(struct InType &in, KstTopLevelViewPtr tlv) {
+  Kst2DPlotPtr plot;
   int i_plot;
   int size_boost;
 
   for (i_plot=0; i_plot < in.n_plots; i_plot++) {
-    plot = new KstPlot(QString("P") + QString::number(i_plot),
-                       1.0/double(in.n_cols), 1.0/double(in.n_rows),
-                       double(i_plot%in.n_cols)/double(in.n_cols),
-                       double(i_plot/in.n_cols)/double(in.n_rows));
-
+    plot = new Kst2DPlot(KST::suggestPlotName());
     size_boost = (in.n_rows + in.n_cols)/2-1;
     size_boost = size_boost*12/2;
     plot->initFonts(QFont::QFont(), size_boost);
-    KST::plotList.append(plot);
+    tlv->appendChild(plot.data());
+
+    plot->resizeFromAspect(double(i_plot%in.n_cols)/double(in.n_cols),
+                            double(i_plot/in.n_cols)/double(in.n_rows),
+                            1.0/double(in.n_cols), 1.0/double(in.n_rows));
+
   }
+  tlv->setColumns(in.n_cols);
+  tlv->setOnGrid(true);
 }
 
 /******************************************************************/
@@ -226,12 +238,13 @@ KstRVector *GetOrCreateVector(QString field, KstDataSourcePtr file, struct InTyp
     V = *vectorList.at(i_v);
     if (V->getField() == field) {
       if (V->filename() == file->fileName()) {
-	return (V);
+        return (V);
       }
     }
   }
   V = new KstRVector(file, field,
-			  QString("V") + QString::number(1 + i_v++)
+
+  QString("V") + QString::number(1 + i_v++)
                           + "-" + field,
                           in.f, in.n, in.skip, in.doskip, in.doave);
   if (!V->isValid()) {
@@ -240,6 +253,7 @@ KstRVector *GetOrCreateVector(QString field, KstDataSourcePtr file, struct InTyp
     exit(0); // fixme: this can cause crashes
   }
 
+  KST::addVectorToList(V);
   return V;
 }
 
@@ -330,7 +344,8 @@ int main(int argc, char *argv[]) {
     KstPSDCurve *psdcurve;
     KstEquationCurve *eqcurve;
     KstHistogram *hscurve;
-    KstPlot *plot;
+    Kst2DPlotPtr plot;
+
     int n_y, n_eq=0;
 
     /* Parse command line args */
@@ -341,7 +356,7 @@ int main(int argc, char *argv[]) {
     // Initialise the plugin loader and collection.
     PluginCollection::self();
 
-    /* get Y axis collums */
+    // get Y axis columns
     ycolList = args->getOptionList("y");
     yEqList = args->getOptionList("ye");
     psdList = args->getOptionList("p");
@@ -355,15 +370,31 @@ int main(int argc, char *argv[]) {
 
       SetCMDOptions(args, in, n_y);
 
-      CreatePlots(in);
+      KstTopLevelViewPtr tlv = kst->activeView();
+
+      if (!tlv) {
+        //
+        // if there was no active view then we create one...
+        //
+        kst->newWindow(false);
+        tlv = kst->activeView();
+      }
+
+      if (!tlv) {
+        exit(0);
+      }
+
+      CreatePlots(in, tlv);
+      Kst2DPlotList plist =
+        kstObjectSubList<KstViewObject, Kst2DPlot>(tlv->children());
 
       i_plot = 0;
-      plot = KST::plotList.at(i_plot);
+      plot = *plist.at(i_plot);
 
       /* make stand alone equations if there are no files */
       if (args->count()<1) {
         if (!yEqList.isEmpty()) {
-	  QString eqS;
+          QString eqS;
           double max, min;
           int n;
           bool xeq;
@@ -378,12 +409,14 @@ int main(int argc, char *argv[]) {
                                        eqS,
                                        min,max,n, KstColorSequence::next());
 
+                KST::dataObjectList.lock().writeLock();
                 KST::dataObjectList.append(eqcurve);
+                KST::dataObjectList.lock().writeUnlock();
                 plot->addCurve(eqcurve);
 
                 if (in.sep_plots) {
                   i_plot++;
-                  if (i_plot <in.n_plots) plot = KST::plotList.at(i_plot);
+                  if (i_plot <in.n_plots) plot = *plist.at(i_plot);
                 }
               }
             }
@@ -403,15 +436,16 @@ int main(int argc, char *argv[]) {
           exit(0);
         }
 
-	if (!file->isValid() || file->frameCount() < 1) {
+        if (!file->isValid() || file->frameCount() < 1) {
           std::cerr << I18N_NOOP("No data in file: ")
-	       << args->arg(i_file)
-	       << ".  Trying to continue..."
-	       << "\n";
+            << args->arg(i_file)
+            << ".  Trying to continue...\n";
           // The file might get data later!
-	}
+        }
 
+        KST::dataObjectList.lock().writeLock();
         KST::dataSourceList.append(file);
+        KST::dataObjectList.lock().writeUnlock();
 
         if (!ycolList.isEmpty()) { // if there are some xy plots
           /* make the x axis vector */
@@ -436,75 +470,85 @@ int main(int argc, char *argv[]) {
               evector = GetOrCreateVector(*(errorList.at(i_ycol)), file, in);
               curve->setYError(KstVectorPtr(evector));
             }
+
+            KST::dataObjectList.lock().writeLock();
             KST::dataObjectList.append(curve);
+            KST::dataObjectList.lock().writeUnlock();
             plot->addCurve(curve);
 
             if (in.sep_plots) {
               i_plot++;
-              if (i_plot < in.n_plots)
-                plot = KST::plotList.at(i_plot);
+              if (i_plot < in.n_plots) {
+                plot =  *plist.at(i_plot);
+              }
             } // end (if they are separate plots)
           } // next y col
         } // end (if there are some xy plots)
-	if (!yEqList.isEmpty()) {
-	  QString eqS;
+        if (!yEqList.isEmpty()) {
+          QString eqS;
           double max, min;
           int n;
           bool xeq, eq_ok;
 
           SetEqXRanges(args->getOption("xe"), &min, &max, &n, &xeq);
-	  for (eq_i = yEqList.begin(); eq_i != yEqList.end(); ++eq_i) {
-	    eqS = *eq_i;
-	    ProcessEq(eqS, file, in, &eq_ok);
+          for (eq_i = yEqList.begin(); eq_i != yEqList.end(); ++eq_i) {
+            eqS = *eq_i;
+            ProcessEq(eqS, file, in, &eq_ok);
             if (xeq) {
               eqcurve =
                 new KstEquationCurve(QString("E")+QString::number(n_eq+1)+
-                                     "-" + eqS,
-                                     eqS,
-                                     min,max,n, KstColorSequence::next());
+                    "-" + eqS,
+                    eqS,
+                    min,max,n, KstColorSequence::next());
             } else {
               if (xvector==NULL)
                 xvector = GetOrCreateVector(args->getOption("x"), file, in);
 
               eqcurve =
                 new KstEquationCurve(QString("E")+QString::number(n_eq+1)+eqS,
-                                     eqS,
-                                     KstVectorPtr(xvector),
-                                     true, KstColorSequence::next());
+                    eqS,
+                    KstVectorPtr(xvector),
+                    true, KstColorSequence::next());
             }
+            KST::dataObjectList.lock().writeLock();
             KST::dataObjectList.append(eqcurve);
+            KST::dataObjectList.lock().writeUnlock();
             plot->addCurve(eqcurve);
 
             if (in.sep_plots) {
               i_plot++;
-              if (i_plot <in.n_plots) plot = KST::plotList.at(i_plot);
+              if (i_plot <in.n_plots) {
+                plot = *plist.at(i_plot);
+              }
             }
-	  }
-	}
+          }
+        }
         if (psdList.count() > 0) { // if there are some psd plots
           KstRVectorList rvl = kstObjectSubList<KstVector,KstRVector>(KST::vectorList);
           for(psd = psdList.begin(); psd != psdList.end(); ++psd ) {
 
-	    yvector = GetOrCreateVector(*psd, file, in);
+            yvector = GetOrCreateVector(*psd, file, in);
 
             color = KstColorSequence::next();
 
-            psdcurve = new KstPSDCurve(QString("P") +
-                                       QString::number(1+i_curve++)
-                                       + "-" + yvector->getField(),
-                                       KstVectorPtr(yvector), in.rate, in.len,
-                                       in.VUnits,in.RUnits,
-                                       color);
+            psdcurve = new KstPSDCurve(
+                QString("P") + QString::number(1+i_curve++) + "-" + yvector->getField(),
+                KstVectorPtr(yvector), in.rate, true, in.len,
+                true, true, in.VUnits, in.RUnits, color);
             if (in.has_points) {
               psdcurve->setHasPoints(true);
               psdcurve->setHasLines(false);
             }
+            KST::dataObjectList.lock().writeLock();
             KST::dataObjectList.append(psdcurve);
+            KST::dataObjectList.lock().writeUnlock();
             plot->addCurve(psdcurve);
 
             if (in.sep_plots) {
               i_plot++;
-              if (i_plot <in.n_plots) plot = KST::plotList.at(i_plot);
+              if (i_plot <in.n_plots) {
+                plot = *plist.at(i_plot);
+              }
             }
           } // next psd
         } // end (if there are some psds)
@@ -514,51 +558,53 @@ int main(int argc, char *argv[]) {
 
           KstRVectorList rvl = kstObjectSubList<KstVector,KstRVector>(KST::vectorList);
           for (hs = hsList.begin(); hs != hsList.end(); ++hs ) {
-	    yvector = GetOrCreateVector(*hs, file, in);
+            yvector = GetOrCreateVector(*hs, file, in);
 
             color = KstColorSequence::next();
 
             KstHistogram::AutoBin(KstVectorPtr(yvector), &N, &max, &min);
 
             hscurve = new KstHistogram(QString("H") +
-                                       QString::number(1+i_curve++)
-                                       + "-" + yvector->getField(),
-                                       KstVectorPtr(yvector), min, max, N,
-                                       KST_HS_NUMBER,
-                                       color);
+                QString::number(1+i_curve++)
+                + "-" + yvector->getField(),
+                KstVectorPtr(yvector), min, max, N,
+                KST_HS_NUMBER,
+                color);
+            KST::dataObjectList.lock().writeLock();
             KST::dataObjectList.append(KstDataObjectPtr(hscurve));
+            KST::dataObjectList.lock().writeUnlock();
             plot->addCurve(hscurve);
 
             if (in.sep_plots) {
               i_plot++;
-              if (i_plot < in.n_plots)
-                plot = KST::plotList.at(i_plot);
+              if (i_plot < in.n_plots) {
+                plot = *plist.at(i_plot);
+              }
             }
           } // next histogram
         } // end (if there are some histograms)
       } // next data file
       for (i_plot = 0; i_plot < in.n_plots; i_plot++) {
-        plot = KST::plotList.at(i_plot);
+        plot = *plist.at(i_plot);
         plot->GenerateDefaultLabels();
         if (in.dolegend) {
           plot->Legend->setShow(true);
           plot->Legend->setFront(true);
         }
       }
-      KST::plotList.setPlotCols(in.n_cols);
     } else if (args->count() > 0) {
       /* open a kst file */
       /* some of the options can be overridden */
       kst->openDocumentFile(args->arg(0),
-			    args->getOption("F"), // override FileName
-			    // override number of frames
-			    args->getOption("n").toInt(&nOK),
-			    // override starting frame
-			    args->getOption("f").toInt(&nOK),
-			    // override skip
-			    args->getOption("s").toInt(&nOK),
-			    // add averaging
-			    args->isSet("a"));
+          args->getOption("F"), // override FileName
+          // override number of frames
+          args->getOption("n").toInt(&nOK),
+          // override starting frame
+          args->getOption("f").toInt(&nOK),
+          // override skip
+          args->getOption("s").toInt(&nOK),
+          // add averaging
+          args->isSet("a"));
     } else {
       //kst->openDocumentFile();
     }
@@ -571,14 +617,14 @@ int main(int argc, char *argv[]) {
 
     if (printfile!="<none>") {
       args->clear();
-      kst->forcedUpdate();
+      kst->forceUpdate();
       kst->immediatePrintToFile(printfile);
       print_and_exit = true;
     }
 
     if (pngfile!="<none>") {
       args->clear();
-      kst->forcedUpdate();
+      kst->forceUpdate();
       kst->immediatePrintToPng(pngfile);
       print_and_exit = true;
     }
@@ -590,6 +636,7 @@ int main(int argc, char *argv[]) {
       args->clear();
       app.setMainWidget(kst);
       kst->show();
+      kst->updateDialogs();
     }
 
     // LEAVE THIS HERE - causes crashes otherwise!
@@ -599,3 +646,4 @@ int main(int argc, char *argv[]) {
   }
   return app.exec();
 }
+// vim: ts=2 sw=2 et
