@@ -23,11 +23,11 @@
 #include <qpair.h>
 #include <qstring.h>
 
-#include <ksharedptr.h>
+#include "kstsharedptr.h"
 
 class KLibrary;
 
-class Plugin : public KShared {
+class Plugin : public KstShared {
 friend class PluginLoader;
 public:
   virtual ~Plugin();
@@ -97,7 +97,10 @@ public:
       // Intro
       bool _filter;
       bool _localdata;
+      bool _isFit;
+      bool _isFitWeighted;
       QString _name;
+      QString _readableName;
       QString _author;
       QString _description;
       QString _version;
