@@ -46,7 +46,7 @@ public:
   /**
    *  Returns 0 on success, nonzero on error.
    */
-  int unloadPlugin(const KSharedPtr<Plugin> p);
+  int unloadPlugin(const KstSharedPtr<Plugin> p);
   int unloadPlugin(const QString& name);
 
   int count() const;
@@ -55,7 +55,7 @@ public:
    *  Returns 0L on error.  Will load the plugin if it is not already
    *  loaded.
    */
-  KSharedPtr<Plugin> plugin(const QString& name);
+  KstSharedPtr<Plugin> plugin(const QString& name);
 
   /**
    *  Returns true if the plugin is loaded.
@@ -105,7 +105,7 @@ protected:
 
 private:
   PluginXMLParser *_parser;
-  QMap<QString, KSharedPtr<Plugin> > _plugins;
+  QMap<QString, KstSharedPtr<Plugin> > _plugins;
 
   // Map: XMLFILE -> Plugin data
   mutable QMap<QString, Plugin::Data> _installedPlugins;
