@@ -54,7 +54,7 @@ PluginLoader::~PluginLoader() {
 Plugin *PluginLoader::loadPlugin(const QString& xmlfile, const QString& object) {
   // First try to load the XML file.
   if (_parser->parseFile(xmlfile) != 0) {
-    KstDebug::self()->log(i18n("Couldn't parse xml file '%1'.").arg(xmlfile), KstDebug::Error);
+    KstDebug::self()->log(i18n("Could not parse xml file '%1'.").arg(xmlfile), KstDebug::Error);
     return 0L;
   }
 
@@ -65,7 +65,7 @@ Plugin *PluginLoader::loadPlugin(const QString& xmlfile, const QString& object)
   // Load the plugin
   plug->_lib = KLibLoader::self()->library(object.local8Bit().data());
   if (!plug->_lib) {
-    KstDebug::self()->log(i18n("Couldn't find library '%1' while loading plugin.").arg(object), KstDebug::Error);
+    KstDebug::self()->log(i18n("Could not find library '%1' while loading plugin.").arg(object), KstDebug::Error);
     delete plug;
     return 0L;
   }
@@ -73,7 +73,7 @@ Plugin *PluginLoader::loadPlugin(const QString& xmlfile, const QString& object)
   plug->_symbol = plug->_lib->symbol(plug->_data._name.latin1());
 
   if (!plug->_symbol) {
-    KstDebug::self()->log(i18n("Couldn't find symbol '%1' in plugin %2.").arg(plug->_data._name).arg(object), KstDebug::Error);
+    KstDebug::self()->log(i18n("Could not find symbol '%1' in plugin %2.").arg(plug->_data._name).arg(object), KstDebug::Error);
     delete plug;
     return 0L;
   }
