@@ -61,12 +61,15 @@ const Plugin::Data& PluginXMLParser::data() const {
 static const QString QS_string = QString::fromLatin1("string");
 static const QString QS_int = QString::fromLatin1("int");
 static const QString QS_name = QString::fromLatin1("name");
+static const QString QS_readableName = QString::fromLatin1("readableName");
 static const QString QS_helptext = QString::fromLatin1("helptext");
 static const QString QS_intro = QString::fromLatin1("intro");
 static const QString QS_interface = QString::fromLatin1("interface");
 static const QString QS_paralist = QString::fromLatin1("paralist");
 static const QString QS_modulename = QString::fromLatin1("modulename");
 static const QString QS_author = QString::fromLatin1("author");
+static const QString QS_fit = QString::fromLatin1("fit");
+static const QString QS_weighted = QString::fromLatin1("weighted");
 static const QString QS_description = QString::fromLatin1("description");
 static const QString QS_descr = QString::fromLatin1("descr");
 static const QString QS_version = QString::fromLatin1("version");
@@ -143,6 +146,7 @@ QDomNode n = element.firstChild();
 
     QString tn = e.tagName().lower();
     if (tn == QS_modulename) {
+      _pluginData._readableName = e.attribute(QS_readableName);
       _pluginData._name = e.attribute(QS_name);
     } else if (tn == QS_localdata) {
       _pluginData._localdata = true;
@@ -152,6 +156,9 @@ QDomNode n = element.firstChild();
       _pluginData._author = e.attribute(QS_name);
     } else if (tn == QS_description) {
       _pluginData._description = e.attribute(QS_text);
+    } else if (tn == QS_fit) {
+      _pluginData._isFit = true;
+      _pluginData._isFitWeighted = (bool)e.attribute(QS_weighted).toInt();
     } else if (tn == QS_version) {
       _pluginData._version = QString("%1.%2").arg(e.attribute(QS_major))
                                              .arg(e.attribute(QS_minor));
@@ -185,7 +192,11 @@ QDomNode n = element.firstChild();
     n = n.nextSibling();
   }
 
-return 0;
+  if (_pluginData._readableName.isEmpty()) {
+    _pluginData._readableName = _pluginData._name;
+  }
+
+  return 0;
 }
 
 
