@@ -15,18 +15,25 @@ void ScalarSelector::allowNewScalars( bool allowed )
 
 void ScalarSelector::update()
 {
+    if (_scalar->listBox()->isVisible()) {
+        QTimer::singleShot(250, this, SLOT(update()));
+        return;
+    }
+
     blockSignals(true);
     QString prev = _scalar->currentText();
     bool found = false;
     _scalar->clear();
     for (KstScalarList::Iterator i = KST::scalarList.begin(); i != KST::scalarList.end(); ++i) {
-	_scalar->insertItem((*i)->tagName());
-	if (!found && (*i)->tagName() == prev) {
-	    found = true;
-	}
+      if ((*i)->displayable()) {
+	      _scalar->insertItem((*i)->tagName());
+      }
+	    if (!found && (*i)->tagName() == prev) {
+	      found = true;
+	    }
     }
     if (found) {
-	_scalar->setCurrentText(prev);
+	    _scalar->setCurrentText(prev);
     }
     blockSignals(false);
 }
@@ -62,7 +69,7 @@ void ScalarSelector::selectionWatcher( const QString & tag )
 void ScalarSelector::setSelection( const QString & tag )
 {
     if (tag.isEmpty()) {
-	return;
+	    return;
     }
     blockSignals(true);
     _scalar->setCurrentText(tag);
@@ -85,3 +92,17 @@ QString ScalarSelector::selectedScalar()
 {
     return _scalar->currentText();
 }
+
+
+void ScalarSelector::allowDirectEntry( bool allowed )
+{
+  if (allowed) {
+    if( !_scalar->editable()) {
+      _scalar->setEditable( TRUE );
+    }
+  } else {
+    if( _scalar->editable()) {
+      _scalar->setEditable( FALSE );
+    }
+  }
+}
