@@ -15,14 +15,26 @@
  *                                                                         *
  ***************************************************************************/
 
+#include "config.h"
 #include "stdinsource.h"
+
 #include <kdebug.h>
 #include <ktempfile.h>
 
 #include <stdlib.h>
 #include <unistd.h>
 
- 
+#ifdef TIME_WITH_SYS_TIME
+# include <sys/time.h>
+# include <time.h>
+#else
+# ifdef HAVE_SYS_TIME_H
+#  include <sys/time.h>
+# else
+#  include <time.h>
+# endif
+#endif
+
 KstStdinSource::KstStdinSource()
 : KstDataSource("stdin", "stdin") {
   _file = new KTempFile();
@@ -50,6 +62,7 @@ KstObject::UpdateType KstStdinSource::update(int u) {
   char instr[4097];
   int i = 0;
   bool new_data = false;
+  bool got_some = false;
 
   FILE *fp = _file->fstream();
 
@@ -68,9 +81,10 @@ KstObject::UpdateType KstStdinSource::update(int u) {
     retval = select(1, &rfds, NULL, NULL, &tv);
 
     new_data = false;
-    if (retval) {
+    if (retval > 0) {
       char *fgs = fgets(instr, 4096, stdin);
       if (fgs && fp) {
+        got_some = true;
         fputs(instr, fp);
         new_data = true;
       }
@@ -79,7 +93,7 @@ KstObject::UpdateType KstStdinSource::update(int u) {
 
   fflush(fp);
 
-  if (new_data) {
+  if (got_some) {
     return _src ? _src->update(u) : KstObject::NO_CHANGE;
   } else {
     return KstObject::NO_CHANGE;
