@@ -25,6 +25,11 @@ QString VectorSelector::selectedVector()
 
 void VectorSelector::update()
 {
+    if (_vector->listBox()->isVisible()) {
+        QTimer::singleShot(250, this, SLOT(update()));
+        return;
+    }
+
     blockSignals(true);
     QString prev = _vector->currentText();
     bool found = false;
